#!/usr/bin/perl
#------------------------------------------------------------------------------
# Programm         : g e d . p l
#------------------------------------------------------------------------------
# Autor            : Christian Dhl
# Erstellt         : 31. Januar 2003
# Zuletzt gendert : 17. Januar 2005
# Aufgabe          : Ein grafischer Editor fr Graphen.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Pakete / Pragmas
#------------------------------------------------------------------------------
use strict;
use warnings;

use Tk;
use Tk::Balloon;
use Tk::Dialog;
use Tk::ErrorDialog;

use Cwd qw/cwd/;        # fr cwd() - Ermittlung des aktuellen
                        #             Arbeitsverzeichnisses

use Math::Trig qw/atan/;
use Data::Dumper qw/Dumper/;
use File::Temp qw//;

use FindBin;
use lib "$FindBin::RealBin";

use Graph::GED::FHist;
use Graph::GED::Geom;
use Graph::GED::Graph2D;
use Graph::GED::TkTools;
use Graph::GED::Historie;


use constant PI => 3.14159;

#------------------------------------------------------------------------------
# Todo-Text:
#------------------------------------------------------------------------------
my $Todotext = <<'TODOTEXT';
- Neue Dateien mit englischen Namen und Kommentaren, alte irgendwann umstellen.

- HTML-Hilfe mit Screenshots erstellen, die man ber Hilfe ... starten kann.
  oder mit Texten und Bildern im Textwidget (dafr ($top, $text) zurckliefern
  und ROText benutzen).

- Automatische Umordnungen der Ecken ...

- Automatisches Vermeiden von Kantenberschneidungen, wenn das geht ...
  Linienberschneidungen diagnostizieren...
  Algorithmus: Wann schneiden sich zwei Strecken in der Ebene?

- Iconleiste : Drucken... Funktionalitt wie?
  Dazu schrieb ptk: unter Unix koennte man den Umweg ueber einen
  Postscript-Viewer gehen. D.h. Postscript in eine temporaere Datei schreiben
  und dann kghostview, ggv oder ghostview aufrufen, je nachdem, was vorhanden
  ist. Dort kann man meist den Drucker auswaehlen und hat gleich eine
  Druckansicht. Oder man verwendet kprinter nur fuer die Druckerauswahl (keine
  Ahnung, was man hier unter GNOME verwenden wuerde). Unter Windows ist die
  Sache schwieriger. Man koennte versuchen tkWinPrint von Tcl/Tk nach Perl/Tk
  zu portieren. Man koennte aus dem Graphen ein PDF erzeugen und dann mit
  PDF::Create oder aehnlichem ein PDF erzeugen und dann Acrobat Reader
  aufrufen.

- Linienzge automatisch verfeinern (mit Skala) (Einfgen von Zwischenecken
  vom Grad 2 an entsprechenden Punkten ["entsprechend" => Spline...]).

- Flchen fllen...
  Idee: Wenn Linien dazukommen und dadurch Flchen aufteilen, wird ein neuer
  Farbwert erzeugt.
  berlegen, ob das nur bei ebenen Graphen gemacht wird.

- Aufspaltung des Programms in verschiedene Richtungen (ev. per Auswahl ber
  Men oder per internem Define?)

- "Slideshow"
  Ganz links Zeitbalken.
  Automatische Abspielmglichkeit als Slider ... und Anwahl einzelner
  Zeitpunkte per Hand.
  Einzelne Zeitpunkte lassen sich als .ged2 speichern, das ganze lsst sich als
  anderes Format speichern, einfach eine Hintereinanderreihung von GED2-For-
  maten, davor die Anzahl Graphen und der Abspiel-Delay.
  ==
  Filmfunktion
  Dafr Regler am linken Rand, Slider fr Bild zu Bildschaltung, "+", "lschen"
  Unter Speichern kann man einzelne Graphen als auch "Filme" speichern.
  ==
  Animation: Aus einzelnem Graph eine Klasse Graph::GED::Animation.pm machen.

- Anbindung an Algorithmen ermglichen, so dass diese visualisiert werden
  knnen.
  Graph::GED::Algorithm::Coloring.pm
  Graph::GED::Algorithm::Serach.pm

- Funktionsbersicht (Auflistung der Funktionen in ihrer Reihenfolge und kurze
  Erklrung).
  Hier ev. Prototypliste mit Kommentaren ergnzen.

- Modus, in dem man Ecken anfgt, die automatisch mit einer (normalen?) Kante
  mit der zuletzt eingefgten verbunden werden.
  [Ev. auch in allen Kantenausprgungen.]

- Mehrfachkantenproblem lsen. Am besten ist es so, dass der Editor erkennt,
  dass da eine zustzliche Kante eingezeichnet wird. Er muss die Kanten dann im
  leichten  Bogen darstellen.
  Bei ungerader Anzahl ist die erste gerade, die zweite rechts davon leicht
  gewlbt, die dritte links, die vierte wieder rechts...
  Bei gerader Anzahl genauso ohne die mittlere Kante und dafr mit etwas
  schwcherer Startwlbung.

- Tk:ProgressBar (in Statuszeile oder extra darber oder in extra Fenster)
  whrend Berechnungen laufen (etwa Liniengraphberechnung).

- Unterstzung mehrerer Graphen (n Stck) nebeneinander, Unterscheidung
  vielleicht durch verschiedene Ecken- und Kantenfarben, bei allen Funktionen
  die Graphen zur Auswahl bringen in Frame mit Canvas-Items.
  Werden zwei Graphen durch eine Kante verbunden, muss daraus einer werden.
  Listbox zum Whlen des Graphen.
  Farben mssen natrlich auch fr alle Graphen whlbar sein.

- Konfigurationsseiten mit Tk::NoteBook (S. 576 Mastering Perl/Tk).

- Iconhintergrnde angleichen (das Wei ist etwas unterschiedlich [oder wirkt
  so wegen der blauen Linien?]).

- Es sollte mglich sein, mehrere Ecken irgendwie mit der Maus zu markieren
  (mit einer Art Lassofunktion) und dann gemeinsam zu behandeln (Lschen und
  insb. Verschieben). Ev. erstmal mit Rechteck oder shift und Anklicken
  realisieren.

- Funktion, die alle Ecken im Raster einrasten lsst, mit Suche nach anderen
  Positionen in der Nhe, bei Zuweisung auf den gleichen Gitterpunkt.

- Pinselfrbetool: Es wird ein Palettenbild rechts zwischen den Buttons ange-
  zeigt, daraus kann der User mit Pinselchen eine Farbe whlen und damit auf
  die Zeichenflche klicken. Mit meint_ecke und dann meint_kante wird getestet,
  welche Ecke oder Kante gemeint war und wenn es eindeutig ist, das entspre-
  chende Element mit dieser Farbe eingefrbt.

- POD erweitern.

- Plttbare Graphen erkennen, Funktion, die plttbare Graphen ohne
  Kantenberschneidungen darstellt.
   - Gibt es dazu schon was?
   - Land mit grter Eckenzahl nach auen?
   - Ecken mit hchstem Grad nach innen?

- Fr ebene Graphen: sonst deaktivierte Meneintrge sind dann anwhlbar:
      * Aus dem Graphen den Graphen erstellen, der entsteht, wenn man in jedes
        Land eine neue Ecke setzt und zwei dieser Ecken mit einer Kante
        verbindet, wenn die Lnder aneinander stoen.
      * Triangulierung: in alle Lnder Kanten einfgen, bis alle Lnder
        dreieckig sind. Optional mit oder ohne uerem Land. (Ev. muss bei mit
        eine Eckenumordnung erfolgen, damit der Graph eben bleibt, aber ev.
        auch schon ohne.:

         2 *-----------* 1
             \         |
               \       |
              3  *     |
                 |     |
                 |     |
                 |     |
                 |     |
                 |     |
                 |     |
                 |     |
              4  *-----* 5

        Fgt man eine Kante 2-4 oder 2-5 ein, so liegt die direkte Verbindung
        auerhalb des Landes.)

- Funktion, die den Graphen bei Proportionserhaltung vergrert und zentriert
  so dass er wieder voll zu sehen ist.

- load as new selecton

- Irgendwann so umstellen, dass die Kantenklassen keine Indizes, sondern
  Eckenobjekte enthalten?
  Die Ecken wren dann in der Eckenmenge und auch direkt an den Kanten.
  Da die Objekte ja Referenzen sind, msste es gehen ohne redundante Infor-
  mationen vorzuhalten. Wird eine Ecke mit allen anstoenden Kanten entfernt,
  mste das Eckenobjekt von der Garbage Collection im Speicher freigegeben
  werden.
  Am besten setzt man dabei die Kantenenden auf undef, damit Kanten und Ecke
  in einem Schritt entfernt werden knnen. (Ich wei nicht, ob die GC schlau
  genug wre zu erkennen, dass die Referenzierung von einem unreferenzierten
  Objekt aus stattfindet [Kanten verweisen noch auf Ecke].)

  -> $self->ecke($kante->a())->x()
  daraus wrde dann
  -> $self->$kante->a()->x()

  Aber der Nachteil ist, man kann den Kanten nicht mehr ansehen, an welche
  Eckennummern sie stoen.
  Erstmal so lassen. Die nderung wre nicht kompliziert, man msst das
  Eckenarray aus Graph2D nach Graph verschieben und die Funktionalitt
  anpassen.
  Fr obigen unschnen Aufruf fge ich Funktionen in Graph2D ein:
  x_von_a, y_von_a, x_von_e, y_von_e, die man mit einer Kante aufruft
  und die einem die entsprechende Koordinate zurckgeben.
  Dann ist das Innenleben wieder ein Stck weiter gekapselt.

- Funktionalitt Kantenauswahl lschen: eine wie auch immer geartete Auswahl:
  etwa alle Ecken leuchten in paarweisen Farben auf wenn es uneindeutig ist
  und man muss dann auf eine der Ecken klicken.

- make install oder entsprechende Funktionalitt anbieten.

- Optional LaTeX-Export mit Dokumentrahmen umgeben. Vielleicht den jetzigen
  Menpunkt durch untermen ersetzen, wo man beide Varianten zur Auswahl hat.
  Oder ein Zwischendialog, wo man diese Wahl ber Checkbuttons o.. treffen
  kann.

- LaTeX-Export um neue Elemente wie Beschriftungen, Werte und Kantenstile
  ergnzen.
  Die Funktion gehrt in Graph2D.

- Ev. blaue Hilfstexte rechtsbndig. (-anchor => 'w') Testen und entscheiden.

- Operationen wie Kanten einfgen oder Ecke bewegen abbrechbar machen.
  (ber Escape oder ber Klick auf anderen Button?)

- Ecken- und Kantenbeschreibungen und -gewichte: Farbe und normal / fett /
  kursiv / fett und kursiv auswhlbar.

- Aus %Konfig eigene Konfigurationsklasse Graph::GED::Config.pm machen?
  Wahrscheinlich ist das bertrieben.

- Eventuell kann man jetzt auf das Anzeigefeld mit der gewhlten Funktionalitt
  verzichten.

- In Bildern eckenartige Gebilde erkennen und "echte" Ecken ber diese legen.
  Eventuell auch fr Kanten.

- Gitterklasse, Verwaltung ber Graph2d. Wenn dies steht, raste_ein in
  Gitterklasse und nach verkleinern / vergrern ins Gitter einrasten, falls
  die Option aktiv ist.

- FHist: $FindBin:RealBin verwenden!

- Historie muss zum Graphen gehren! (Fr Animation.)

Renees Wunschliste:
~~~~~~~~~~~~~~~~~~~
- Export von Graphen in ein Bildformat wie png oder jpg.

Ishkas Wunschliste:
~~~~~~~~~~~~~~~~~~~
- Punkt einfgen, der mit allen Punkten verbunden ist
- Punkt verschieben und dabei alle mit diesem Punkt verbundene Punkte mitverschieben
- gleiches mit Lschen
- bei gerichteten Graphen: von einem Punkt aus alle erreichbaren einfrben
- gleiches mit lschen (ebenso: alle nicht-erreichbaren lschen)
- alternativ zu beiden vorhergehenden Vorschlgen: Punkten Flags geben, die
  man ber Auswahl ndern kann (ala: Alle mit dem als nchsten anzuklickenden
  Punkt verbundenen Punkte flag xyz um eins erhhen, ...) und ber diese Flags
  Punkte auswhlen und zB. lschen, verschieben und was auch immer

Bugreports:
~~~~~~~~~~~
- wenn debug auf 0 steht wird immer noch alles mgliche ausgegeben.
- Speichern und Export scheint unter Linux die Extensionen ohne Punkt dazwischen
  an die Namen anzuhngen. Unter Windows luft es richtig.
- Nichtverschwinden des Kontextmens unter Linux beheben.
- Probleme beim Vorbelegen der Namen fr das Wandeln in LaTeX bzw. PS unter
  Windows 98. (Unter Win2k ok)
- Probleme mit dem Dateipfad unter Windows 98. (Unter Win2k ok)
TODOTEXT


#------------------------------------------------------------------------------
# globale Variablen:
#------------------------------------------------------------------------------
my %Linientypen = ('e_ins' => 'Ecke einfgen',
                   'e_del' => 'Ecke lschen',
                   'e_mer' => 'Ecke mergen',
                   'e_mv1' => 'Ecke verschieben (schnelle Methode)',
                   'e_mv2' => 'Ecke verschieben (schne Methode)',
                   'k_ins' => 'normale Kante einfgen',
                   'k_del' => 'Kante lschen',
                   'k_dla' => 'Kanten lschen',
                   'k_inh' => 'Kante einfgen: Pfeil hin',
                   'k_inz' => 'Kante einfgen: Pfeil zurck',
                   'k_inb' => 'Kante einfgen: Pfeil hin und zurck',
                  );

my %Konfig = (
              debug         => 0,                    # Wert fr Debugausgaben,
                                                     # je hher der Wert,
                                                     # desto mehr wird ausge-
                                                     # geben, bei 0 wird
                                                     # nichts ausgegeben.
              version       => '0.0.39',             # Programmversion
              version_datum => '17.01.2005',         # Programmdatum

              mw            => undef,                # Hauptfenster-Widget
              canvas        => undef,                # Canvas-Widget
              menu          => undef,                # Menu-Widget
              menufont      => '{Arial} 8 {normal}', # Schriftart des Menu
              rueck_btn     => undef,                # Rckgngig-Button
              wieder_btn    => undef,                # Wiederholen-Button

              #gitterzeigen  => 0,                    # Hilfs-Gitter anzeigen?
              #gitterweite   => 25,                   # Weite des Gitters
              #rastimgitter  => 0,                    # Im Gitter einrasten?
              #gitterfarbe   => '#C0C0C0',            # Farbe des Gitters
              #JETZT in Graph2D !!!

              zeige_e_nr    => 0,                    # Eckennummern anzeigen
              zeige_k_nr    => 0,                    # Kantennummern anzeigen

              #dateiname     => '',                   # Dateiname der gerade
              #                                       # bearbeiten Datei
              #                                       # (inklusive Pfad)
              #JETZT in Graph2D !!!
              pfad          => cwd(),                # Pfad zum Laden von
                                                     # Dateien.
                                                     # (Klappt nicht unter Win
                                                     # 98, aber unter Win 2k?!)

              statuszeile   => '',                   # Anzuzeigender Text in
                                                     # der Statuszeile

              linientyp     => 'e_ins',              # Mit den Checkbuttons
                                                     # verbunden

              startecke     => -1,                   # Startecke der Kante, die
                                                     # gerade gezeichnet wird.
              zx            => 0,                    # x-Koordinate und
              zy            => 0,                    # y-Koordinate des
                                                     # Zeichnenanfangs

              historie      => new Graph::GED::Historie(),

              changed       => ' ',                  # Wurde der Graph seit dem
                                                     # Speichern gendert?
                                                     # ' ' : nicht gendert
                                                     # '*' : gendert

              bild          => '',                   # "$FindBin::RealBin/res/dkarte.gif",

             );

my $Graph  = new Graph::GED::Graph2D;
$Graph->kantenform('k_ins');


my @GitterX     = ();      # x-Koordinaten von senkrechten Gitterlinien
my @GitterY     = ();      # y-Koordinaten von waagerechten Gitterlinien


#------------------------------------------------------------------------------
# Prototypen:
#------------------------------------------------------------------------------
# Oberflche:
sub tk_start ();
sub add_file_menu ();
sub bind_start ();
sub zeichnen_anfang ($$$);
sub aendere_item ($$$);
sub zeichnen_ende ($$$);
sub refresh ();
sub clear (;$);
sub zeichne_ecke ($);
sub zeichne_kante ($);
sub lade_datei ();
sub speichere_datei ();
sub speichere_datei_als ();
sub export_nach_latex ();
sub export_nach_postscript ();
sub dump_eingeben ();
sub zeige_daten_an ($);
sub zeige_daten_an_dumper ();
sub hilfe ();
sub graph_erkl ();
sub history ();
sub todo ();
sub perldoc_ged_pl ();
sub perldoc_graph_ged_basis_pm ();
sub perldoc_graph_ged_ecke_pm ();
sub perldoc_graph_ged_kante_pm ();
sub perldoc_graph_ged_graph_pm ();
sub perldoc_graph_ged_graph2d_pm ();
sub perldoc_graph_ged_geom_pm ();
sub perldoc_graph_ged_historie_pm ();
sub perldoc_graph_ged_fhist_pm ();
sub perldoc_graph_ged_tktools_pm ();
sub ueber_ged ();
sub canvas_groesse ();
sub waehle_eckenfarbe ();
sub waehle_kantenfarbe ();
sub historie_schalter_aktualisieren ($$);
sub context_menu ($$$$$);
sub tk_cm_groessenwahl ($$$);
sub tk_cm_kantenformwahl ($$$);
sub tk_cm_kantenstilwahl ($$$);
sub tk_cm_kantendicke ($$$);
sub tk_cm_titelwahl ($$$$$);
sub tk_cm_titelausrichtung ($$$);
sub tk_cm_wertwahl ($$$$$);
sub tk_cm_wertausrichtung ($$$);
sub tk_beenden ();

# Logik:
sub beenden ();
sub struktur_einfuegen ($);
sub raste_ein ($$);
sub erzeuge_liniengraphen ();
sub rueckgaengig ();
sub wiederholung ();
sub aenderung ();
sub do_all_the_tests ();
sub clone_test ();
sub historie_test ();

# Daten:
sub dump_data ();
sub save ($);
sub load ($);
sub neue_datei ();
sub add ($);
sub clear_context_menu ();


#------------------------------------------------------------------------------
# Programmstart:
#------------------------------------------------------------------------------
#historie_test();
#clone_test();
$Graph->r2d();
print "Pfad = '$Konfig{pfad}'\n" if $Konfig{debug} > 1;
#do_all_the_tests();

Graph::GED::FHist::load();
tk_start();

$Konfig{statuszeile} = 'Willkommen bei GED';

$Konfig{mw}->after(100,
                    sub {
                         ($Graph::GED::Graph2D::MinX,
                          $Graph::GED::Graph2D::MinY,
                          $Graph::GED::Graph2D::MaxX,
                          $Graph::GED::Graph2D::MaxY) = canvas_groesse();
                    }
                  );
MainLoop();
exit;


###############################################################################
###############################################################################
####                                                                       ####
####                            TTTTT   K   K                              ####
####                              T     K  K                               ####
####                              T     KKK                                ####
####                              T     K  K                               ####
####                              T     K   K                              ####
####                                                                       ####
###############################################################################
###############################################################################


sub tk_start () {
    #--------------------------------------------------------------------------
    # sub          : t k _ s t a r t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Startet den Oberflchenkram.
    #                Dies ist in eine Funktion ausgelagert worden, um den glo-
    #                balen Namensbereich nicht zu berfrachten.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 31.01.2003 - CD - Erstellt
    # 0.0.2  - 11.02.2003 - CD - Men umgestellt nach durchforsten der perldoc.
    #                            toplevel menu mit cascading untermens.
    # 0.0.3  - 18.03.2003 - CD - Neue Linientypenkrzel verwendet, lokale Vari-
    #                            able $linientyptext eingefhrt.
    # 0.0.4  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.5  - 06.02.2004 - CD - Neues Men "Ansicht".
    # 0.0.6  - 06.02.2004 - CD - Button zum Eckenmergen hinzugefgt.
    # 0.0.7  - 07.02.2004 - CD - Bindung frs Neuzeichnen der Zeichenflche.
    #                            Ecken- und Kantenfarbe bers Ansichtsmen
    #                            whlbar.
    # 0.0.8  - 09.02.2004 - CD - Erstes Bearbeiten Men auskommentiert.
    #                            Umstellungen auf die Verwendung von Graph2D.
    #                            Eckenradiuswahl ins Men verlagert.
    #                            Versionsnummer im Titel mit anzeigen.
    # 0.0.9  - 12.02.2004 - CD - Rckgngig- und Wiederholenbuttons eingebaut.
    # 0.0.10 - 30.12.2004 - CD - Bindung von context_menu() an die rechte
    #                            Maustaste.
    # 0.0.11 - 31.12.2004 - CD - Neuen Meneintrag fr Dumper.
    # 0.0.12 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    #                            Perldoc anzeigen im Hilfemen.
    #                            Gitterfarbe ist nun whlbar.
    #                            Balloon-Help hinzugefgt.
    # 0.0.13 - 14.01.2005 - CD - Button fr: Alle Kanten an Cursorposition
    #                                        lschen.
    #                            Meneintrge zum Anzeigen der Ecken- und
    #                            Kantennummern.
    #                            Neuer Meneintrag zum Anzeigen des Graphen im
    #                            GED2-Format.
    # 0.0.14 - 16.01.2005 - CD - Anzeigen im GED3-Format.
    #                            Buttons zum Verschieben des Graphens.
    # 0.0.15 - 17.01.2005 - CD - Button zum Zentrieren und Skalieren und
    #                            verkleinern / vergrern.
    #--------------------------------------------------------------------------

    print "tk_start\n" if $Konfig{debug};

    my $linientyptext = 'Ecke einfgen';

    #--------------------------------------------------------------------------
    # TK-Hauptfenster:
    #--------------------------------------------------------------------------
    my $mw = MainWindow->new();
    $mw->title('G - E - D   ' . $Konfig{version});

    my $icon = $mw->Photo(-file   => "$FindBin::RealBin/res/ged.bmp",
                          -height => 32,
                          -width  => 32,
                         );
    $mw->Icon(-image => $icon);


    #--------------------------------------------------------------------------
    # Status-Bar:
    #--------------------------------------------------------------------------
    my $stat_f = $mw->Frame(
       #-relief      => 'ridge',
        -borderwidth => 2,
    );
    my $stat_l = $stat_f->Label(
        -relief       => 'groove', # ridge
        -textvariable => \$Konfig{statuszeile},
        -borderwidth  => 2,
       #-background   => '#d7d7d7',
       #-state        => 'disabled',
        -anchor       => 'w',
    )->pack(
        -side         => 'left',
        -expand       => 1,
        -fill         => 'x',
    );
    $stat_f->Label(
        -relief       => 'groove', # ridge
        -textvariable => \$Konfig{changed},
        -borderwidth  => 2,
       #-background   => '#d7d7d7',
       #-state        => 'disabled',
        -width        => 1,
    )->pack(
        -side         => 'left',
        -expand       => 0,
        -fill         => 'x',
    );

    #--------------------------------------------------------------------------
    # Men:
    #--------------------------------------------------------------------------
    my $mf = $mw->Frame(#-relief      => 'ridge',
                        -borderwidth => 2,
                       );#


    # Menu-Bar:
    $Konfig{menu} = $mf->Menu(-type => 'menubar',
                              -font => $Konfig{menufont},
                              );

    $mw->configure(-menu => $Konfig{menu});

    # Eintrge in der Menu-Bar:
    $Konfig{menu}->cascade(-label     => 'Datei',
                           -underline => 0,
                           -font      => $Konfig{menufont},
                          );
    $Konfig{menu}->cascade(-label     => 'Bearbeiten',
                           -underline => 0,
                           -font      => $Konfig{menufont},
                          );
    $Konfig{menu}->cascade(-label     => 'Ansicht',
                           -underline => 0,
                           -font      => $Konfig{menufont},
                          );
    $Konfig{menu}->cascade(-label     => 'Manipulationen',
                           -underline => 0,
                           -font      => $Konfig{menufont},
                          );
    $Konfig{menu}->cascade(-label     => 'Interna',
                           -underline => 0,
                           -font      => $Konfig{menufont},
                          );
    $Konfig{menu}->separator(); # Rest rechtsbndig
    $Konfig{menu}->cascade(-label     => 'Hilfe',
                           -underline => 0,
                           -font      => $Konfig{menufont},
                          );

    # Dateimen:
    add_file_menu();


    # Bearbeitenmen:
    my $menu_bearbeiten = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               );
    $menu_bearbeiten->cascade(-label     => 'Kreise',
                              -underline => 0,
                              -font      => $Konfig{menufont},
                             );
    $menu_bearbeiten->cascade(-label     => 'Vollstndige Graphen',
                              -underline => 0,
                              -font      => $Konfig{menufont},
                             );
    $menu_bearbeiten->command(-label     => "Mittelpunkt einfgen",
                              -command  => [ \&struktur_einfuegen, 'M' ],
                              -font     => $Konfig{menufont},
                             );
    my $menu_bearbeiten_kreise =
    $menu_bearbeiten
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command" => "Dreieck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C3' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Viereck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C4' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Fnfeck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C5' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Sechseck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C6' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Siebeneck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C7' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Achteck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C8' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Neuneck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C9' ],
                                -font     => $Konfig{menufont},
                               ],
                               '-',
                               ["command" => "Zwanzigeck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C20' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Dreisigeck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C30' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Vierzigeck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C40' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Fnfzigeck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C50' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Neununneunzigeck einfgen",
                                -command  => [ \&struktur_einfuegen, 'C99' ],
                                -font     => $Konfig{menufont},
                               ],
                             ],
              );
    $menu_bearbeiten->entryconfigure('Kreise',
                                     -menu => $menu_bearbeiten_kreise,
                                    );
    my $menu_bearbeiten_vgraphen = $menu_bearbeiten
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command" => "K_3 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K3' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_4 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K4' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_5 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K5' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_6 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K6' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_7 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K7' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_8 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K8' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_9 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K9' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_10 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K10' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_11 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K11' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_12 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K12' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_13 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K13' ],
                                -font     => $Konfig{menufont},
                               ],
                               '-',
                               ["command" => "K_20 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K20' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_30 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K30' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_40 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K40' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_50 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K50' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_99 einfgen",
                                -command  => [ \&struktur_einfuegen, 'K99' ],
                                -font     => $Konfig{menufont},
                               ],
                             ],
              );
    $menu_bearbeiten->entryconfigure('Vollstndige Graphen',
                                     -menu => $menu_bearbeiten_vgraphen,
                                     );

    $Konfig{menu}->entryconfigure('Bearbeiten', -menu => $menu_bearbeiten);

    # Ansichtmen:
    my $menu_ansicht = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ['checkbutton' => "Gitternetz anzeigen",
                                -variable     => \$Graph->{gitterzeigen}, #!!!
                                -command      => \&refresh,
                                -underline    => 0,
                                -font         => $Konfig{menufont},
                               ],
                               [qw(cascade Gitterweite -tearoff 1 -underline 6
                                   -menuitems),
                                [map ['radiobutton' => $_,
                                      -variable => \$Graph->{gitterweite}, #!!!
                                      -command  => \&refresh,
                                     ],
                                     qw(5 10 15 20 25 30 35 40 45 50
                                        60 70 80 100),
                                ],
                               ],
                               ['checkbutton' => "Am Gitternetz einrasten",
                                -variable     => \$Graph->{rastimgitter}, #!!!
                                -command      => \&refresh,
                                -underline    => 17,
                                -font         => $Konfig{menufont},
                               ],
                               ['command'     => "Gitterfarbe whlen (kleine Auswahl)",
                                -command      => sub {
                                                     my ($x, $y) = $Konfig{mw}->geometry()
                                                         =~ /\+(\d+)\+(\d+)$/;
                                                     Graph::GED::TkTools::farbwahl(
                                                         $Konfig{mw},
                                                         $x+140, $y+155,
                                                         sub {
                                                             aenderung();
                                                             $Graph->gitterfarbe($_[0]);
                                                             refresh();
                                                         },
                                                     )
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ['command'     => "Gitterfarbe whlen (groe Auswahl)",
                                -command      => sub {
                                                     aenderung();
                                                     my $farbe = $Konfig{mw}->
                                                         chooseColor(-title        => 'Gitterfarbe whlen',
                                                                     -initialcolor => $Graph->hintergrund(),
                                                                    );
                                                     $Graph->gitterfarbe($farbe) if $farbe;
                                                     refresh();
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               '-',
                               ['command'     => "Anzeige erneuern",
                                -command      => \&refresh,
                                -accelerator  => 'F5',
                                -font         => $Konfig{menufont},
                                -underline    => 10,
                               ],
                               '-',
                               ['checkbutton' => "Eckennummern anzeigen",
                                -variable     => \$Konfig{zeige_e_nr},
                                -command      => \&refresh,
                               #-underline    => 17,
                                -font         => $Konfig{menufont},
                               ],
                               ['checkbutton' => "Kantennummern anzeigen",
                                -variable     => \$Konfig{zeige_k_nr},
                                -command      => \&refresh,
                               #-underline    => 17,
                                -font         => $Konfig{menufont},
                               ],
                               '-',
                               ['command'     => "Hintergrundfarbe whlen (kleine Auswahl)",
                                -command      => sub {
                                                     my ($x, $y) = $Konfig{mw}->geometry()
                                                         =~ /\+(\d+)\+(\d+)$/;
                                                     Graph::GED::TkTools::farbwahl(
                                                         $Konfig{mw},
                                                         $x+140, $y+155,
                                                         sub {
                                                             aenderung();
                                                             $Graph->hintergrund($_[0]);
                                                             refresh();
                                                         },
                                                     )
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ['command'     => "Hintergrundfarbe whlen (groe Auswahl)",
                                -command      => sub {
                                                     aenderung();
                                                     my $farbe = $Konfig{mw}->
                                                         chooseColor(-title        => 'Hintergrundfarbe whlen',
                                                                     -initialcolor => $Graph->hintergrund(),
                                                                    );
                                                     $Graph->hintergrund($farbe) if $farbe;
                                                     refresh();
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ["command"    => "Hintergrundbild festlegen",
                                -command     => \&lade_hintergrundbild,
                                #-accelerator => 'Ctrl-L',
                                -font        => $Konfig{menufont},
                               ],
                               '-',
                               ['command'     => "Default Eckenfarbe whlen (kleine Auswahl)",
                                -command      => sub {
                                                     my ($x, $y) = $Konfig{mw}->geometry()
                                                         =~ /\+(\d+)\+(\d+)$/;
                                                     Graph::GED::TkTools::farbwahl(
                                                         $Konfig{mw},
                                                         $x+140, $y+165,
                                                         sub {
                                                             aenderung();
                                                             $Graph->eckenfarbe($_[0]);
                                                             refresh();
                                                         },
                                                     )
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ['command'     => "Default Kantenfarbe whlen (kleine Auswahl)",
                                -command      => sub {
                                                     my ($x, $y) = $Konfig{mw}->geometry()
                                                         =~ /\+(\d+)\+(\d+)$/;
                                                     Graph::GED::TkTools::farbwahl(
                                                         $Konfig{mw},
                                                         $x+140, $y+175,
                                                         sub {
                                                             aenderung();
                                                             $Graph->kantenfarbe($_[0]);
                                                             refresh();
                                                         },
                                                     )
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               '-',
                               ['command'     => "Default Eckenfarbe whlen (groe Farbauswahl)",
                                -command      => \&waehle_eckenfarbe,
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ['command'     => "Default Kantenfarbe whlen (groe Farbauswahl)",
                                -command      => \&waehle_kantenfarbe,
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               '-',
                               [cascade    => 'Default Eckenradius',
                                -tearoff   => 0,
                                -underline => 6,
                                -menuitems =>
                                [map ['radiobutton' => $_,
                                      -variable => \$Graph->{radius},
                                      # UNSCHOEN da an Getter / Setter vorbei!
                                      -command  => \&refresh,
                                     ],
                                     qw(3 4 5 6 7 8 9 10 12 14 16 18 20 25 30
                                        35 40 50 75 100),
                                ],
                               ],
                               [cascade    => 'Default Kantendicke',
                                -tearoff   => 0,
                                -underline => 6,
                                -menuitems =>
                                [map ['radiobutton' => $_,
                                      -variable => \$Graph->{kantendicke},
                                      # UNSCHOEN da an Getter / Setter vorbei!
                                      -command  => \&refresh,
                                     ],
                                     qw/1 2 3 4 5 6 7 8 9 10 11 12 14 16 17 18
                                        19 20 25 30 35 40 50 75 100/,
                                ],
                               ],
                             ],
              );
    $Konfig{menu}->entryconfigure('Ansicht', -menu => $menu_ansicht);

    # Manipulatorenmen:
    my $menu_manip = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command"    => "In Liniengraphen umwandeln",
                                -command     => \&erzeuge_liniengraphen,
                                -underline   => 3,
                                -font        => $Konfig{menufont},
                               ],
                               #["command"    => 'Dump eingeben',
                               # -command     => \&dump_eingeben,
                               # -font        => $Konfig{menufont},
                               #],
                             ],
              );
    $Konfig{menu}->entryconfigure('Manipulationen', -menu => $menu_manip);

    # Internamen:
    my $menu_interna = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command"    => "interne Daten anzeigen (GED-Format)",
                                -command     => [ \&zeige_daten_an, '.ged'],
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "interne Daten anzeigen (GED2-Format)",
                                -command     => [ \&zeige_daten_an, '.ged2'],
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "interne Daten anzeigen (GED3-Format)",
                                -command     => [ \&zeige_daten_an, '.ged3'],
                                -accelerator => 'Ctrl-I',
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "interne Struktur anzeigen (Dumper)",
                                -command     => \&zeige_daten_an_dumper,
                               #-accelerator => 'Ctrl-I',
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => 'Dump eingeben (GED-Format)',
                                -command     => \&dump_eingeben,
                                -font        => $Konfig{menufont},
                               ],
                             ],
              );
    $Konfig{menu}->entryconfigure('Interna', -menu => $menu_interna);

    # Hilfemen:
    my $menu_hilfe = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command"    => "Hilfe",
                                -command     => \&hilfe,
                                -accelerator => 'F1',
                                -font        => $Konfig{menufont},
                               ],
                               '-',
                               ["command"    => "Was sind Graphen?",
                                -command     => \&graph_erkl,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "History",
                                -command     => \&history,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "Todo",
                                -command     => \&todo,
                                -font        => $Konfig{menufont},
                               ],
                               '-',
                               ["command"    => "perldoc ged.pl",
                                -command     => \&perldoc_ged_pl,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/Basis.pm",
                                -command     => \&perldoc_graph_ged_basis_pm,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/Ecke.pm",
                                -command     => \&perldoc_graph_ged_ecke_pm,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/Kante.pm",
                                -command     => \&perldoc_graph_ged_kante_pm,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/Graph.pm",
                                -command     => \&perldoc_graph_ged_graph_pm,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/Graph2D.pm",
                                -command     => \&perldoc_graph_ged_graph2d_pm,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/Geom.pm",
                                -command     => \&perldoc_graph_ged_geom_pm,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/Historie.pm",
                                -command     => \&perldoc_graph_ged_historie_pm,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/FHist.pm",
                                -command     => \&perldoc_graph_ged_fhist_pm,
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "perldoc Graph/GED/TkTools.pm",
                                -command     => \&perldoc_graph_ged_tktools_pm,
                                -font        => $Konfig{menufont},
                               ],
                               '-',
                               ["command"    => "ber GED",
                                -command     => \&ueber_ged,
                                -font        => $Konfig{menufont},
                               ],
                             ],
              );
    $Konfig{menu}->entryconfigure('Hilfe', -menu => $menu_hilfe);

    #--------------------------------------------------------------------------
    # Linker Animationsrahmen:
    #--------------------------------------------------------------------------
    my $flinks = $mw->Frame(
        -relief      => 'groove',
        -borderwidth => 2,
    );
    $flinks->Label(
        -text => 'Bild',
    )->pack();
    my $skala = $flinks->Scale(
        -from     => 1,
        -to       => 50,
        -label    => '',
        -orient   => 'vertical',
        -variable => \$Graph->{radius}, ###UNSCHOEN !!!
        -command  =>
            sub {
                $Graph->r2d();
                refresh();
            },
    )->pack();

    #--------------------------------------------------------------------------
    # Zeichenflche:
    #--------------------------------------------------------------------------
    my $graf_lw = $mw->Canvas(-background => 'white',
                              -state      => 'disabled',
                              -width      => 80,
                              -height     => 25,
                              -relief     => 'flat',
                              -cursor     => 'crosshair',
                             );

    $Konfig{canvas} = $graf_lw;
    bind_start(); # <----- $Konfig{canvas}

    #--------------------------------------------------------------------------
    # Schalter:
    #--------------------------------------------------------------------------
    my $f = $mw->Frame(-relief      => 'groove',
                       -borderwidth => 2,
                      );


    my $clr_b = $f->Button(-text           => 'Alles lschen',
                           -command        => \&clear,
                          )
                    ->pack(#-expand => 1,
                           -fill   => 'x',
                           -side   => 'top',
                          );

    my $fmove = $f->Frame()
                   ->pack(-fill   => 'x',
                          -side   => 'top',
                         );
    my @movebuttons;
    push @movebuttons,
    $fmove->Button(-text    => 'nach links',
                   -command => sub {aenderung();
                                    $Graph->nach_links();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/nach_links.gif"),
                  #-relief  => 'flat',
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @movebuttons,
    $fmove->Button(-text    => 'nach oben',
                   -command => sub {aenderung();
                                    $Graph->nach_oben();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/nach_oben.gif"),
                  #-relief  => 'flat',
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @movebuttons,
    $fmove->Button(-text    => 'nach unten',
                   -command => sub {aenderung();
                                    $Graph->nach_unten();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/nach_unten.gif"),
                  #-relief  => 'flat',
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @movebuttons,
    $fmove->Button(-text    => 'nach rechts',
                   -command => sub {aenderung();
                                    $Graph->nach_rechts();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/nach_rechts.gif"),
                  #-relief  => 'flat',
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );

    my $fskal = $f->Frame()
                   ->pack(-fill   => 'x',
                          -side   => 'top',
                         );
    my @skalbuttons;
    push @skalbuttons,
    $fskal->Button(-text    => 'zentrieren',
                   -command => sub {aenderung();
                                    $Graph->zentrieren();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/zentrieren.gif"),
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @skalbuttons,
    $fskal->Button(-text    => 'zentrieren plus',
                   -command => sub {aenderung();
                                    $Graph->zentrieren_skalieren();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/zentrieren_plus.gif"),
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @skalbuttons,
    $fskal->Button(-text    => 'verkleinern',
                   -command => sub {aenderung();
                                    $Graph->kleiner();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/kleiner.gif"),
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @skalbuttons,
    $fskal->Button(-text    => 'vergroessern',
                   -command => sub {aenderung();
                                    $Graph->groesser();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/groesser.gif"),
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );

    #my $skala = $f->Scale(-from     => 1,
    #                      -to       => 50,
    #                      -label    => 'Eckenradius',
    #                      -orient   => 'horizontal',
    #                      -variable => \$Graph->{radius}, ###UNSCHOEN !!!
    #                      -command  => sub {$Graph->r2d();
    #                                        refresh();
    #                                       },
    #                     )
    #               ->pack();

    my $quit_b = $f->Button(-text           => 'Beenden',
                            -command        => \&beenden,
                           )
                     ->pack(#-expand => 1,
                            -fill   => 'x',
                            -side   => 'bottom',
                           );
    my $refr_b = $f->Button(-text           => 'Neu zeichnen',
                            -command        => \&refresh,
                           )
                     ->pack(#-expand => 1,
                            -fill   => 'x',
                            -side   => 'bottom',
                           );

    #--------------------------------------------------------------------------
    # Bilder-Buttons:
    #--------------------------------------------------------------------------
    my $fb = $mw->Frame();

    # Dateibutton-Frame:
    my $fdb = $fb->Frame(-relief      => 'groove',
                         -borderwidth => 2,
                        )
                  ->pack(-side        => 'left',
                         -padx        => 2,
                        );
    my @dbilder = ($mw->Photo(-file => "$FindBin::RealBin/res/datei_neu.gif"),
                   $mw->Photo(-file => "$FindBin::RealBin/res/datei_oeffnen.gif"),
                   $mw->Photo(-file => "$FindBin::RealBin/res/datei_speichern.gif"),
                   $mw->Photo(-file => "$FindBin::RealBin/res/datei_speichern_unter.gif"),
                  );
    my @dbefehle = (\&neue_datei,
                    \&lade_datei,
                    \&speichere_datei,
                    \&speichere_datei_als,
                  );
    my @dateibuttons;
    for my $i (0..$#dbilder) {
        push @dateibuttons,
        $fdb->Button(-text    => 'text',
                     -command => $dbefehle[$i],
                     -image   => $dbilder[$i],
                     -relief  => 'flat',
                    )
              ->pack(-side => 'left');
    }

    # Rckgngig-/Wiederholenbutton-Frame:
    my $frw = $fb->Frame(-relief      => 'groove',
                         -borderwidth => 2,
                        )
                  ->pack(-side        => 'left',
                         -padx        => 2,
                        );
    $Konfig{rueck_btn} = $frw->
        Button(-text    => 'text',
               -command => \&rueckgaengig,
               -image   => $mw->Photo(-file => "$FindBin::RealBin/res/rueckgaengig.gif"),
               -relief  => 'flat',
              )
        ->pack(-side => 'left');
    $Konfig{wieder_btn} = $frw->
        Button(-text    => 'text',
               -command => \&wiederholung,
               -image   => $mw->Photo(-file => "$FindBin::RealBin/res/wiederholung.gif"),
               -relief  => 'flat',
              )
        ->pack(-side => 'left');
    $Konfig{rueck_btn}->configure(-state => 'disabled');
    $Konfig{wieder_btn}->configure(-state => 'disabled');

    # Editbutton-Frame:
    my $feb1 = $fb->Frame(-relief      => 'groove',   # erzeugen
                          -borderwidth => 2,
                         )
                   ->pack(-side        => 'left',
                          -padx        => 2,
                         );
    my $feb2 = $fb->Frame(-relief      => 'groove',   # modifizieren
                          -borderwidth => 2,
                         )
                   ->pack(-side => 'left',
                          -padx        => 2,
                         );
    my $feb3 = $fb->Frame(-relief      => 'groove',  # lschen
                          -borderwidth => 2,
                         )
                   ->pack(-side => 'left',
                          -padx        => 2,
                         );
    my @bilder = ($mw->Photo(-file => "$FindBin::RealBin/res/ecke_neu.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/ecke_weg.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/ecke_mergen.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/ecke_verschieben.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/ecke_verschieben2.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante_loeschen.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kanten_loeschen.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante_hin.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante_zurueck.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante_beide.gif"),
                 );
    my @frames = ($feb1, $feb3, $feb3, $feb2, $feb2, $feb1, $feb3, $feb3, $feb1, $feb1, $feb1);
    my @buttons = ();
    my @befehle = (sub {$Konfig{linientyp}     = 'e_ins';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                            #$_->configure(-relief => 'flat');
                        }
                        $buttons[0]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Ecken zum Lschen da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'e_del';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[1]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Ecken zum Mergen da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'e_mer';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        $_->configure(-relief => 'raised') for @buttons;
                        $buttons[2]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Ecken zum Verschieben da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'e_mv1';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[3]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Ecken zum Verschieben da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'e_mv2';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[4]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() < 2) {
                            print "\a";
                            $Konfig{statuszeile} = "Zu wenig Ecken vorhanden!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_ins';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[5]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->m() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Kanten zum Lschen da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_del';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[6]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->m() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Kanten zum Lschen da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_dla';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[7]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() < 2) {
                            print "\a";
                            $Konfig{statuszeile} = "Zu wenig Ecken vorhanden!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_inh';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[8]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() < 2) {
                            print "\a";
                            $Konfig{statuszeile} = "Zu wenig Ecken vorhanden!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_inz';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[9]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() < 2) {
                            print "\a";
                            $Konfig{statuszeile} = "Zu wenig Ecken vorhanden!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_inb';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[10]->configure(-relief => 'sunken');
                       },
                  );
    for my $i (0..$#bilder) {
        push @buttons, $frames[$i]->Button(-text    => 'text',
                                           -command => $befehle[$i],
                                           -image   => $bilder[$i],
                                           -relief  => 'raised',
                                           #-relief  => 'flat',
                                          )
                                    ->pack(-side => 'left');
    }
    $buttons[0]->configure(-relief => 'sunken');

    # Editbutton-Erklrtext-Frame:
    my $febt = $fb->Frame(-relief      => 'groove',
                          -borderwidth => 2,
                         )
                   ->pack(-side   => 'left',
                          -padx   => 2,
                          -ipady  => 4,
                          -expand => 1,
                          -fill   => 'x',
                         );
    $febt->Label(-textvariable => \$linientyptext,
                 -foreground   => 'blue',
                )
          ->pack(-side => 'left');

    #--------------------------------------------------------------------------
    # Elemente packen:
    #--------------------------------------------------------------------------
    $stat_f->pack(
        -side   => 'bottom',
       #-expand => 1,
        -fill   => 'x',
       #-anchor => 'w',
    );
    $mf->pack(
        -side   => 'top',
       #-expand => 1,
        -fill   => 'x',
        -anchor => 'n',
    );
    $fb->pack(
        -side   => 'top',
       #-expand => 1,
        -fill   => 'x',
    );
    #$flinks->pack(
    #    -side   => 'left',
    #   #-expand => 1,
    #    -fill   => 'both',
    #);
    
    $graf_lw->pack(
        -side   => 'left',
        -expand => 1,
        -fill   => 'both',
    );
    $f->pack(
        -side   => 'left',
       #-expand => 1,
        -fill   => 'y',
    );

    #--------------------------------------------------------------------------
    # Balloon-Widget erzeugen:
    #--------------------------------------------------------------------------
    {
        my $b = $mw->Balloon(-statusbar => $stat_l);
        $b->attach(
            $dateibuttons[0],
            -state => 'status',
            -msg   => "Der aktuelle Graph wird gelscht und ein neuer leerer Graph erzeugt.",
        );
        $b->attach(
            $dateibuttons[1],
            -state => 'status',
            -msg   => "ffnet eine Graphendatei und ldt den Graphen in den Editor.",
        );
        $b->attach(
            $dateibuttons[2],
            -state => 'status',
            -msg   => "Speichert den Graphen.",
        );
        $b->attach(
            $dateibuttons[3],
            -state => 'status',
            -msg   => "Speichert den Graphen unter einem anderen Namen oder in einem anderen Format.",
        );

        $b->attach(
            $Konfig{rueck_btn},
            -state => 'status',
            -msg   => "Letzte nderung rckgngig machen.",
        );
        $b->attach(
            $Konfig{wieder_btn},
            -state => 'status',
            -msg   => "Rckgngig gemachte nderung wiederholen.",
        );

        $b->attach(
            $buttons[0],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfgen neuer Ecken.",
        );
        $b->attach(
            $buttons[1],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Lschen von Ecken.",
        );
        $b->attach(
            $buttons[2],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Mergen neuer Ecken.",
        );
        $b->attach(
            $buttons[3],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Verschieben neuer Ecken (Pfeilanzeige).",
        );
        $b->attach(
            $buttons[4],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Verschieben neuer Ecken (Anzeige aller verschobenen Kanten).",
        );
        $b->attach(
            $buttons[5],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfgen neuer normaler Kanten.",
        );
        $b->attach(
            $buttons[6],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Lschen von Kanten.",
        );
        $b->attach(
            $buttons[7],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Lschen von allen Kanten in der Nhe.",
        );
        $b->attach(
            $buttons[8],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfgen neuer Kanten hin.",
        );
        $b->attach(
            $buttons[9],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfgen neuer Kanten zurck.",
        );
        $b->attach(
            $buttons[10],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfgen neuer Kanten hin und zurck.",
        );

#        $b->attach(
#            $mb,
#            -msg => 'Press and hold this button to see the menu.'
#        );
#        $b->attach(
#            $menu,
#           #-state           => 'status',
#            -balloonposition => 'mouse',
#            -msg             => [
#                'Use this to tear off the menu.',
#                 'This is a checkbox entry.',
#                 'cascade', # Cascade entry (ignored by Balloon)
#                 'separator', # Separator: never active so no message will be displayed for this entry.
#                 'This is a command entry - it will close this window.',
#            ],
#        );
        $b->attach(
            $clr_b,
            -state      => 'status',
            -balloonmsg => "Der aktuelle\nGraph wird\ngelscht",
            -statusmsg  => "Der aktuelle Graph wird gelscht",
        );
        $b->attach(
            $quit_b,
            -state      => 'status',
            -balloonmsg => "Beendet\ndas\nProgramm",
            -statusmsg  => "Beendet das Programm",
        );
        $b->attach(
            $refr_b,
            -state      => 'status',
            -balloonmsg => "Zeichnet den\nGraphen neu",
            -statusmsg  => "Zeichnet den Graphen neu",
        );


        $b->attach(
            $movebuttons[0],
            -state => 'status',
            -msg   => "Bewegt den Graphen um die Gitterweite nach links.",
        );
        $b->attach(
            $movebuttons[1],
            -state => 'status',
            -msg   => "Bewegt den Graphen um die Gitterweite nach oben.",
        );
        $b->attach(
            $movebuttons[2],
            -state => 'status',
            -msg   => "Bewegt den Graphen um die Gitterweite nach unten.",
        );
        $b->attach(
            $movebuttons[3],
            -state => 'status',
            -msg   => "Bewegt den Graphen um die Gitterweite nach rechts.",
        );
        $b->attach(
            $skalbuttons[0],
            -state => 'status',
            -msg   => "Zentriert den Graphen.",
        );
        $b->attach(
            $skalbuttons[1],
            -state => 'status',
            -msg   => "Zentriert und skaliert den Graphen.",
        );
        $b->attach(
            $skalbuttons[2],
            -state => 'status',
            -msg   => "Verkleinert den Graphen.",
        );
        $b->attach(
            $skalbuttons[3],
            -state => 'status',
            -msg   => "Vergrert den Graphen.",
        );

        # destroy the balloons when I am destroyed:
        #  - Balloon.pm adds bindings to all widgets which we now want to remove if we can.
        #$Konfig{balloon} = $b;
        #$mw->OnDestroy( sub { $Konfig{balloon}->destroy() } );
    }


    #--------------------------------------------------------------------------
    # Restliches:
    #--------------------------------------------------------------------------
    $mw->bind('<Control-b>', \&beenden);
    $mw->bind('<Escape>',
        sub {
            $graf_lw->focus();
            clear_context_menu();
        }
    );

    $mw->bind('<F5>',        sub {$refr_b->invoke()});

    $mw->bind('<Control-n>', \&neue_datei);
    $mw->bind('<Control-l>', \&lade_datei);
    $mw->bind('<Control-s>', \&speichere_datei);
    $mw->bind('<Control-i>', [\&zeige_daten_an, '.ged3']);
    $mw->bind('<F1>',        \&hilfe);

    $mw->bind('<Alt-d>',     sub {$Konfig{menu}->postcascade('Datei');});
    $mw->bind('<Alt-b>',     sub {$Konfig{menu}->postcascade('Bearbeiten');});
    $mw->bind('<Alt-i>',     sub {$Konfig{menu}->postcascade('Interna');});
    $mw->bind('<Alt-h>',     sub {$Konfig{menu}->postcascade('Hilfe');});


    $Konfig{canvas}->Tk::bind("<Button-3>", [\&context_menu, Ev('x'), Ev('y'), Ev('X'), Ev('Y')]);

    # Zum Neuzeichnen fr das Gitter:
    $Konfig{canvas}->CanvasBind('<Configure>', \&refresh);

    $mw->geometry('600x500'); # 600x500

    $Konfig{mw} = $mw;
} # sub tk_start


sub add_file_menu () {
    #--------------------------------------------------------------------------
    # sub          : a d d _ f i l e _ m e n u
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt das Dateimen hinzu.
    #                Ausgelagert, um aktuelle bearbeitete Dateien anzuzeigen.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 09.02.2004 - CD - Ansicht erneuern auskommentiert (ist jetzt im
    #                           Men Ansicht.
    # 0.0.4 - 10.02.2004 - CD - Module in Namensraum Graph::GED:: verschoben.
    # 0.0.5 - 03.01.2005 - CD - Inhalt lschen ist dazugekommen
    # 0.0.6 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    #--------------------------------------------------------------------------

    print "add_file_menue\n" if $Konfig{debug};


    my $menu_datei =
    $Konfig{menu}->Menu(-font      => $Konfig{menufont},
                -tearoff   => 0,
                -menuitems => [ ["command"    => "Neu",
                                 -command     => \&neue_datei,
                                 -accelerator => 'Ctrl-N',
                                 -font        => $Konfig{menufont},
                                 -underline   => 0,
                                ],
                                ["command"    => "Laden",
                                 -command     => \&lade_datei,
                                 -accelerator => 'Ctrl-L',
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Speichern",
                                 -command     => \&speichere_datei,
                                 -accelerator => 'Ctrl-S',
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Speichern unter",
                                 -command     => \&speichere_datei_als,
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Inhalt lschen",
                                 -command     => \&clear,
                                 -font        => $Konfig{menufont},
                                ],
                                '-',
                                ["command"    => "Export nach LaTeX",
                                 -command     => \&export_nach_latex,
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Export nach Postscript",
                                 -command     => \&export_nach_postscript,
                                 -font        => $Konfig{menufont},
                                ],
                                #'-',
                                #["command"    => "Anzeige erneuern",
                                # -command     => \&refresh,
                                # -accelerator => 'F5',
                                # -font        => $Konfig{menufont},
                                #],
                              ],
               );
    $menu_datei->separator();
    for (reverse Graph::GED::FHist::array()) {
        $menu_datei->command(-label       => $_,
                             -command     =>
                              [sub {$Graph->dateiname($_[0]);
                                    clear();
                                    load($_[0]);
                                    refresh();
                                    $Konfig{mw}->title('G - E - D   -   ('
                                                       . $Graph->dateiname()
                                                       . ')');
                                   }, $_
                              ],
                             #-accelerator => 'Ctrl-B', #'ESC',
                             -font        => $Konfig{menufont},
                            );
    }
    $menu_datei->separator();
    $menu_datei->command(
                         -label       => "Beenden",
                         -command     => \&beenden,
                         -accelerator => 'Ctrl-B', #'ESC',
                         -font        => $Konfig{menufont},
                        );
    $Konfig{menu}->entryconfigure('Datei', -menu => $menu_datei);
} # sub add_file_menu


sub bind_start () {
    #--------------------------------------------------------------------------
    # sub          : b i n d _ s t a r t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Bindet <Button-1> an zeichen_anfang(), falls es noch keine
    #                Bindung gibt.
    #                Wenn es eine <Motion>-Bindung gibt, dann muss man dem Be-
    #                nutzer erlauben, das Zeichnen des Elements zu beenden,
    #                bevor <Button-1> erneut gebunden wird.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 31.01.2003 - CD - Erstellt
    # 0.0.2 - 04.02.2003 - CD - Canvas-Widget muss nun nicht mehr bergeben
    #                           werden.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #--------------------------------------------------------------------------

    my @bindings  = $Konfig{canvas}->Tk::bind("<Motion>");
    return if scalar @bindings;

    $Konfig{canvas}->Tk::bind("<Button-1>", [\&zeichnen_anfang, Ev('x'), Ev('y')]);
} # sub bind_start


sub zeichnen_anfang ($$$) {
    #--------------------------------------------------------------------------
    # sub          : z e i c h n e n _ a n f a n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Wird aufgerufen, wenn der Benutzer mit der Maus auf die
    #                Arbeitsflche klickt und noch kein Zeichenvorgang statt-
    #                findet.
    #                Abhngig vom ausgewhlten "Linientyp" (kann auch etwa
    #                'Ecke' sein), wird das Zeichnen der Linie begonnen (oder
    #                - wie im Fall von 'Ecke' - das Objekt gezeichnet, wenn es
    #                keiner zwei Mausklicks bedarf).
    # Parameter    : Canvas-Widget, x, y
    #                Auerdem wird die globale Variable '$Konfig{linientyp}'
    #                ausgewertet.
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 31.01.2003 - CD - Erstellt
    # 0.0.2  - 03.02.2003 - CD - Eckenkoordinaten speichern
    #                            Kantenkoordinaten auf Ecken setzen und nur in
    #                            der Nhe von Ecken zu Zeichnen beginnen!
    #                            Eckenlschen eingebaut.
    # 0.0.3  - 05.02.2003 - CD - 'Ecke verschieben' eingebaut.
    #                            'Ecke verschieben 2' eingebaut.
    # 0.0.4  - 10.02.2003 - CD - An neue Linientypenbezeichungen angepasst.
    # 0.0.5  - 18.03.2003 - CD - Neue Linientypenkrzel verwendet. Die Lang-
    #                            formen sind im Hash %Linientypen ber die
    #                            Krzel als Schlssel zugnglich.
    # 0.0.6  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.7  - 06.02.2004 - CD - Beim Eckensetzen ggf. am Gitter einrasten.
    # 0.0.8  - 06.02.2004 - CD - Eckenmergen eingebaut.
    #                            Ecken/Kantenfunktionalitten ausgelagert.
    # 0.0.9  - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.10 - 10.02.2004 - CD - Umstellung auf die Eckenklasse.
    # 0.0.11 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    # 0.0.12 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    # 0.0.13 - 03.01.2005 - CD - aenderung() bei Verschiebungen schon zu Beginn
    #                            aufrufen.
    # 0.0.14 - 14.01.2005 - CD - Kanten lschen.
    # 0.0.15 - 17.01.2005 - CD - Kontextmens lschen, falls vorhanden.
    #--------------------------------------------------------------------------

    my $w = shift; # widget
    my $x = shift; # x-Koordinate
    my $y = shift; # y-Koordinate

    print "zeichnen_anfang x='$x', y='$y', Linientyp='$Konfig{linientyp}'\n"
          if $Konfig{debug};

    #--------------------------------------------------------------------------
    # Vorarbeiten:
    #--------------------------------------------------------------------------

    clear_context_menu(); # ggf. noch offenes Kontextmen entfernen

    # Im Fall von Linientypen Startkoordinaten auf die nchste Ecke setzen:
    if ($Konfig{linientyp} eq 'k_ins' or
        $Konfig{linientyp} eq 'k_inh' or
        $Konfig{linientyp} eq 'k_inz' or
        $Konfig{linientyp} eq 'k_inb' or
        $Konfig{linientyp} eq 'e_mv1' or
        $Konfig{linientyp} eq 'e_mv2'  )
    {
        my $e = $Graph->meint_ecke($x, $y);
        if ($e > -1 and
            ($Graph->n() > 1 or # Linienzeichnen macht erst ab 2 Ecken Sinn!
             $Graph->n() == 1 and $Konfig{linientyp} =~ /^e_mv[12]$/
            )
           )
        { 
            $Konfig{zx} = $Graph->ecke($e)->x();
            $Konfig{zy} = $Graph->ecke($e)->y();
            $Konfig{startecke} = $e;
            $Graph->kantenform($Konfig{linientyp});
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie mssen an einer Ecke anfangen!';
            return;
        }
        aenderung() if $Konfig{linientyp} eq 'e_mv1'
                    or $Konfig{linientyp} eq 'e_mv2';
    }
    else {
        $Konfig{zx} = $x;
        $Konfig{zy} = $y;
    }


    #--------------------------------------------------------------------------
    # Operationen, die in aendere_item() und zeichnen_ende weiter behandelt
    # werden:
    #--------------------------------------------------------------------------

    #-----------
    if ($Konfig{linientyp} eq 'k_ins') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width => 1,
                       -tags  => 'zeichnemichjetzt',
                       -fill  => $Graph->kantenfarbe(),
                      );
    }
    #----------->
    elsif ($Konfig{linientyp} eq 'k_inh') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width      => 1,
                       -tags       => 'zeichnemichjetzt',
                       -arrow      => 'last',
                       -arrowshape => [5, 7, 3],
                       -fill       => $Graph->kantenfarbe(),
                      );
    }
    # <----------
    elsif ($Konfig{linientyp} eq 'k_inz') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width      => 1,
                       -tags       => 'zeichnemichjetzt',
                       -arrow      => 'first',
                       -arrowshape => [5, 7, 3],
                       -fill       => $Graph->kantenfarbe(),
                      );
    }
    # <---------->
    elsif ($Konfig{linientyp} eq 'k_inb') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width      => 1,
                       -tags       => 'zeichnemichjetzt',
                       -arrow      => 'both',
                       -arrowshape => [5, 7, 3],
                       -fill       => $Graph->kantenfarbe(),
                      );
    }
    # * --->
    elsif ($Konfig{linientyp} eq 'e_mv1') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width      => 1,
                       -tags       => 'zeichnemichjetzt',
                       -width      => 1 + 2 * $Graph->radius(),
                       -arrow      => 'last',
                       -arrowshape => [
                                       5 + $Graph->radius(),
                                       7 + $Graph->radius(),
                                       3 + $Graph->radius(),
                                      ],
                       -fill       => $Graph->eckenfarbe(),
                      );
    }



    #--------------------------------------------------------------------------
    # Operationen, die hier bereits abgeschlossen werden knnen:
    #--------------------------------------------------------------------------

    # *
    if ($Konfig{linientyp} eq 'e_ins') {
        # falls am Gitter eingerastet werden soll, Koordinaten einrasten:
        if ($Graph->rastimgitter()) {
            ($Konfig{zx}, $Konfig{zy}) = raste_ein($Konfig{zx}, $Konfig{zy});
        }

        if ($Graph->ecke_zu_nah($Konfig{zx}, $Konfig{zy})) {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind zu dicht an einer anderen Ecke!';
            return;
        }
        else {
            aenderung();
            $Graph->ecke_einfuegen($Konfig{zx}, $Konfig{zy});
            zeichne_ecke($Graph->n()-1);
            $w->Tk::bind("<Motion>",  "");
            bind_start();
        }
    }
    # -*
    elsif ($Konfig{linientyp} eq 'e_del') {
        my $e = $Graph->meint_ecke($x, $y);
        if ($e > -1) {
            aenderung();
            $Graph->ecke_loeschen($e);
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind nicht dicht genug an einer Ecke!';
            return;
        }
    }

    # -* merge
    elsif ($Konfig{linientyp} eq 'e_mer') {
        my $e = $Graph->meint_ecke($x, $y);
        if ($e > -1) {
            aenderung();
            $Graph->ecke_mergen($e);
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind nicht dicht genug an einer Ecke!';
            return;
        }
    }
    # - ---------
    elsif ($Konfig{linientyp} eq 'k_del') {
        my $k = $Graph->meint_kante($x, $y);
        if ($k > -1) {
            aenderung();
            $Graph->kante_loeschen($k);
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind nicht dicht genug an einer Kante oder '.
                           'zu dicht an zweien!';
            return;
        }
    }
    elsif ($Konfig{linientyp} eq 'k_dla') {
        my @k = $Graph->meint_kante($x, $y);
        if (@k > 0) {
            aenderung();
            $Graph->kante_loeschen($_) for reverse sort {$a <=> $b} @k;
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind nicht dicht genug an einer Kante!';
            return;
        }
    }
    else {
        # Im Fall von "echten Linien" (also solchen Objekten, die eine
        # zweite Benutzerangabe zum Beenden des Zeichnens bentigen) wird
        # <Motion> an aendere_item() und <Button-1> an zeichnen_ende()
        # gebunden:
        $w->Tk::bind("<Motion>",   [\&aendere_item,  Ev('x'), Ev('y')]);
        $w->Tk::bind("<Button-1>", [\&zeichnen_ende, Ev('x'), Ev('y')]);
    }
} # sub zeichnen_anfang


sub aendere_item ($$$) {
    #--------------------------------------------------------------------------
    # sub          : a e n d e r e _ i t e m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Diese Funktion ist an <Motion> gebunden, wenn ein
    #                Zeichenvorgang luft.
    # Parameter    : Canvas-Widget, x, y
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 31.01.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2003 - CD - 'Ecke verschieben 2' eingebaut.
    # 0.0.3 - 10.02.2003 - CD - An neue Linientypenbezeichungen angepasst.
    # 0.0.4 - 18.03.2003 - CD - Neue Linientypenkrzel verwendet.
    # 0.0.5 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.6 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.7 - 10.02.2004 - CD - Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    my $w = shift; # widget
    my $x = shift; # x-Koordinate
    my $y = shift; # y-Koordinate

    print "aendere_item x='$x', y='$y'\n" if $Konfig{debug} > 1;

    if ($Konfig{linientyp} eq 'e_mv2') {
        my $e = $Konfig{startecke}; # hier steckt die Ecke drin!
        $Graph->ecke($e)->x($x);
        $Graph->ecke($e)->y($y);                                             #;
        refresh();
    }
    else {
        $w->coords('zeichnemichjetzt', $Konfig{zx}, $Konfig{zy}, $x, $y);
        # Hier wird auf den Tag 'zeichnemichjetzt' zugegriffen, der in
        # zeichnen_anfang() vergeben wurde.
    }
} # sub aendere_item


sub zeichnen_ende ($$$) {
    #--------------------------------------------------------------------------
    # sub          : z e i c h n e n _ e n d e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Diese Funktion wird aufgerufen, wenn der Benutzer beim
    #                Zeichnen eines Objektes das zweite Mal klickt.
    #                Damit wird der Zeichenvorgang beendet, die Bindungen von
    #                <Motion> aufgehoben und die Bindung von <Button-1> durch
    #                den Aufruf von bind_start() wieder auf zeichnen_anfang()
    #                gesetzt.
    # Parameter    : Canvas-Widget, x, y
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 31.01.2003 - CD - Erstellt
    # 0.0.2  - 03.02.2003 - CD - Ecke bestimmen, falls keine da, wird das Be-
    #                            enden verweigert.
    #                            Kantendaten speichern.
    # 0.0.3  - 05.02.2003 - CD - 'Ecke verschieben' eingebaut.
    #                            'Ecke verschieben 2' eingebaut.
    #                            Schleifen verhindert.
    # 0.0.4  - 10.02.2003 - CD - An neue Linientypenbezeichungen angepasst.
    # 0.0.5  - 18.03.2003 - CD - Neue Linientypenkrzel verwendet.
    #                            Bei Ecke Schnell verschieben nicht mehr auf
    #                            zu nah zur Ausgangsecke getestet.
    # 0.0.6  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.7  - 06.02.2004 - CD - Beim Eckenverschieben ggf. am Gitter ein-
    #                            rasten.
    # 0.0.8  - 06.02.2004 - CD - Ecken/Kantenfunktionalitten ausgelagert.
    # 0.0.9  - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.10 - 10.02.2004 - CD - Umstellung auf die Eckenklasse.
    # 0.0.11 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    # 0.0.12 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    # 0.0.13 - 03.01.2005 - CD - aenderung() bei Verschiebungen schon zu Beginn
    #                            aufrufen (und hier nicht mehr).
    #--------------------------------------------------------------------------

    my $w = shift; # widget
    my $x = shift; # x-Koordinate
    my $y = shift; # y-Koordinate

    print "zeichnen_ende x='$x', y='$y'\n" if $Konfig{debug};

    if ($Konfig{linientyp} eq 'e_mv1') {
        my $e = $Konfig{startecke}; # hier steckt die Ecke drin!

        # falls am Gitter eingerastet werden soll, Koordinaten einrasten:
        if ($Graph->rastimgitter()) {
            ($x, $y) = raste_ein($x, $y);
        }

        if ($Graph->ecke_zu_nah_ohne($x, $y, $e)) {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind zu dicht an einer anderen Ecke!';
            return;
        }

        else {
            #my $e = $Konfig{startecke}; # hier steckt die Ecke drin!
            print "Ecke $e wird verschoben, von (",
                   $Graph->ecke($e)->x(), ", ", $Graph->ecke($e)->y(),
                   ") auf ($x, $y).\n" if $Konfig{debug};
            $w->coords('zeichnemichjetzt', $Konfig{zx}, $Konfig{zy}, $x, $y);
            $w->delete('zeichnemichjetzt'); # dtag
            #aenderung();
            $Graph->ecke($e)->x($x);
            $Graph->ecke($e)->y($y);
            refresh();
        }
    }
    elsif ($Konfig{linientyp} eq 'e_mv2') {
        my $e = $Konfig{startecke}; # hier steckt die Ecke drin!

        # falls am Gitter eingerastet werden soll, Koordinaten einrasten:
        if ($Graph->rastimgitter()) {
            ($x, $y) = raste_ein($x, $y);
        }

        if ($Graph->ecke_zu_nah_ohne($x, $y, $e)) {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind zu dicht an einer anderen Ecke!';
            return;
        }
        else {
            print "Ecke $e wird verschoben, von (",
                   $Graph->ecke($e)->x(), ", ", $Graph->ecke($e)->y(),
                   ") auf ($x, $y).\n" if $Konfig{debug};
            #aenderung();
            $Graph->ecke($e)->x($x);
            $Graph->ecke($e)->y($y);
            refresh();
        }
    }
    else { # Kanten
        my $e = $Graph->meint_ecke($x, $y);
        if ($e > -1) {
            if ($e == $Konfig{startecke}) {
                print "\a";
                $Konfig{statuszeile} = 'Schleifen sind im Moment nicht mglich!';
                return;
            }
            $x = $Graph->ecke($e)->x();
            $y = $Graph->ecke($e)->y();
            aenderung();
            $Graph->kante_einfuegen($Konfig{startecke}, $e);
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind zu nicht dicht genug an einer Ecke!';
            return;
        }

        $w->coords('zeichnemichjetzt', $Konfig{zx}, $Konfig{zy}, $x, $y);
        $w->dtag('zeichnemichjetzt');
        # Hier wird auf den Tag 'zeichnemichjetzt' zugegriffen, der in
        # zeichnen_anfang() vergeben wurde.
        # Auerdem wird der Tag jetzt von diesem Objekt entfernt, sonst wrde
        # es beim Zeichenen des nchsten Objekts mit verndert.

        $Konfig{zx} = $Graph->ecke($e)->x();
        $Konfig{zy} = $Graph->ecke($e)->y();
        $Konfig{startecke} = $e;
    }

    $w->Tk::bind("<Motion>",  "");
    bind_start();
    $Konfig{statuszeile} = ''; # ok :)
} # sub zeichnen_ende


sub refresh () {
    #--------------------------------------------------------------------------
    # sub          : r e f r e s h
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Alles neu zeichnen.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.3 - 06.02.2004 - CD - Falls gewnscht Gitternetz einzeichnen.
    #                           Gitterlinienkoordinaten in @GitterX und
    #                           @GitterY merken.
    # 0.0.4 - 06.02.2004 - CD - canvas_groesse() aufgerufen.
    # 0.0.5 - 07.02.2004 - CD - canvas_groesse() aufgerufen und die Daten in
    #                           den Klassenvariablen $MinX, $MinY, $MaxX,
    #                           $MaxY der Klasse Graph2D gespeichert.
    # 0.0.6 - 08.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.7 - 09.02.2004 - CD - Gitterkoordinaten auch berechnen, wenn nur
    #                           eingerastet werden soll.
    # 0.0.8 - 02.01.2005 - CD - Hintergrundfarbe der Canvas setzen.
    # 0.0.9 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    #                           Variavle Gitterfarbe auch bei vertikalen Linien
    #                           verwenden (dort war noch die Farbe hartcodierte
    #                           eingetragen).
    #--------------------------------------------------------------------------

    print "refresh\n" if $Konfig{debug};

    #--------------------------------------------------------------------------
    # Aktuelle Gre der Zeichenflche ermitteln:
    #--------------------------------------------------------------------------
    ($Graph::GED::Graph2D::MinX, $Graph::GED::Graph2D::MinY,
     $Graph::GED::Graph2D::MaxX, $Graph::GED::Graph2D::MaxY) = canvas_groesse();

    #--------------------------------------------------------------------------
    # Zeichenflche leeren und Hintergrundfarbe setzen:
    #--------------------------------------------------------------------------
    $Konfig{canvas}->delete('all');
    $Konfig{canvas}->configure(-background => $Graph->hintergrund());
    if (length $Graph->hintergrundbild() and -f $Graph->hintergrundbild()) {
        $Konfig{canvas}->createImage(
        #    ($Graph::GED::Graph2D::MaxX-$Graph::GED::Graph2D::MinX)/2,
        #    ($Graph::GED::Graph2D::MaxY-$Graph::GED::Graph2D::MinY)/2,
        #    -anchor => 'center',
            0,
            0,
            -anchor => 'nw',
            -image  => $Konfig{mw}->Photo(-file => $Graph->hintergrundbild()),
        );
    }

    #--------------------------------------------------------------------------
    # Gitternetz einzeichnen:
    #--------------------------------------------------------------------------
    if ($Graph->gitterzeigen() or $Graph->rastimgitter()) {
        @GitterX = (); # ev. vorhandene
        @GitterY = (); # alte Eintrge lschen

        for (
             my $x =  $Graph->gitterweite();
                $x <= $Graph::GED::Graph2D::MaxX;
                $x += $Graph->gitterweite()
            )
        {
            $Konfig{canvas}->createLine($x, $Graph::GED::Graph2D::MinY,
                                        $x, $Graph::GED::Graph2D::MaxY,
                                        -width => 1,
                                        -fill  => $Graph->gitterfarbe(),
                                       )
                                       if $Graph->gitterzeigen();
            push @GitterX, $x;
        }

        for (
             my $y =  $Graph->gitterweite();
                $y <= $Graph::GED::Graph2D::MaxY;
                $y += $Graph->gitterweite()
            )
        {
            $Konfig{canvas}->createLine($Graph::GED::Graph2D::MinX, $y,
                                        $Graph::GED::Graph2D::MaxX, $y,
                                        -width => 1,
                                        -fill  => $Graph->gitterfarbe(),
                                       )
                                       if $Graph->gitterzeigen();
            push @GitterY, $y;
        }
    }

    #--------------------------------------------------------------------------
    # Graphen neu einzeichnen:
    #--------------------------------------------------------------------------
    zeichne_ecke($_)  for (0..$Graph->n()-1);
    zeichne_kante($_) for (0..$Graph->m()-1);
} # sub refresh


sub clear (;$) {
    #--------------------------------------------------------------------------
    # sub          : c l e a r
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Alles lschen
    # Parameter    : bind, siehe Kommentar vom 05.02.2003 (optional)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 04.02.2003 - CD - Canvas-Widget muss nun nicht mehr bergeben
    #                           werden.
    # 0.0.3 - 05.02.2003 - CD - Parameter $bind hinzugefgt. Bei 0 wird
    #                           bind_start nicht aufgerufen, d.h. der Aufrufer
    #                           kmmert sich selbst um die Bindung. Bei ande-
    #                           ren Werten oder ohne Parameter wird bind_start
    #                           aufgerufen.
    #                           Bentigt wird dies, um das Malen von Ecken nach
    #                           Doppelklick auf Namen der Datei beim Laden zu
    #                           verhindern.
    # 0.0.4 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.5 - 09.02.2004 - CD - Umstellung auf Graphenklassen.
    # 0.0.6 - 01.01.2005 - CD - Gitter nach lschen wieder anzeigen.
    #--------------------------------------------------------------------------

    my $bind = shift;

    $bind = 1 unless defined $bind;

    print "clear - bind = '$bind'\n" if $Konfig{debug};

    #--------------------------------------------------------------------------
    # Graphendarstellung lschen:
    #--------------------------------------------------------------------------
    $Konfig{canvas}->delete('all');

    #--------------------------------------------------------------------------
    # Graphendaten lschen:
    #--------------------------------------------------------------------------
    $Graph->clear();

    #--------------------------------------------------------------------------
    # Gitter anzeigen, falls gewnscht:
    #--------------------------------------------------------------------------
    refresh();

    #--------------------------------------------------------------------------
    # Bindung aufheben, falls man gerade am Zeichnen ist:
    #--------------------------------------------------------------------------
    $Konfig{canvas}->Tk::bind("<Motion>",  "");

    #--------------------------------------------------------------------------
    # Bindungen wieder herstellen, falls gewnscht:
    #--------------------------------------------------------------------------
    bind_start() if $bind;

} # sub clear


sub zeichne_ecke ($) {
    #--------------------------------------------------------------------------
    # sub          : z e i c h n e _ e c k e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zeichnet die angegebene Ecke ein.
    # Parameter    : Eckennummer 0 .. $Graph->n()-1
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 03.02.2003 - CD - Erstellt
    # 0.0.2  - 04.02.2003 - CD - Programmabbruch bei undefinierten Koordinaten
    #                            eingebaut.
    # 0.0.3  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.4  - 07.02.2004 - CD - Verwendung des Farbwertes $Graph{eckenfarbe}
    #                            statt hardvercodetem Rot.
    # 0.0.5  - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.6  - 10.02.2004 - CD - Umstellung auf die Eckenklasse.
    # 0.0.7  - 30.02.2004 - CD - Beachtung einzelner Farbwerte der Ecken.
    #                            Beachtung einzelner Gren der Ecken.
    # 0.0.8  - 01.01.2005 - CD - Titel und Gewicht mit ausgeben.
    # 0.0.9  - 03.01.2005 - CD - Anchor-Berechnung vereinfacht.
    #                            Titel- und Gewichtfarbe bercksichtigen.
    # 0.0.10 - 14.01.2005 - CD - Eckennummern auf Wunsch anzeigen.
    #--------------------------------------------------------------------------

    my $e = shift;

    my $ecke = $Graph->ecke($e);
    my $x    = $ecke->x();
    my $y    = $ecke->y();

    die "Koordinaten von Ecke $e sind undefiniert!" unless defined $x and
                                                           defined $y;

    print "zeichne_ecke, e = '$e', (N = '",
          $Graph->n(), "')\n",
          "    (", $x, ", ", $y, ")\n",
          "    Radius = '", $Graph->radius(),
          "', Delta = '", $Graph->delta(), "'\n"
          if $Konfig{debug} > 1;

    my $farbe  = $Graph->ecke($e)->farbe()  || $Graph->eckenfarbe();
    my $radius = $Graph->ecke($e)->radius() || $Graph->radius();

    $Konfig{canvas}->
        createOval($x-$radius, $y-$radius,
                   $x+$radius, $y+$radius,
                   -width   => 1,
                   -fill    => $farbe,
                   -outline => $farbe,
                  );

    if (length $ecke->titel()) {
        my $delta = 1 + 2 * ($ecke->radius() || $Graph->radius());
        $delta = int($delta / sqrt 2) if length $ecke->titelpos() > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $ecke->titelpos() =~ /N/;
        $ty += $delta if $ecke->titelpos() =~ /S/;
        $tx -= $delta if $ecke->titelpos() =~ /W/;
        $tx += $delta if $ecke->titelpos() =~ /O/;
        (my $anchor = $ecke->titelpos()) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $ecke->titel(),
                       -anchor => $anchor,
                       -fill   => $ecke->titelfarbe(),
                      );
    }
    if (defined $ecke->gewicht()) {
        my $delta = 1 + 2 * ($ecke->radius() || $Graph->radius());
        $delta = int($delta / sqrt 2) if length $ecke->gewichtpos() > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $ecke->gewichtpos() =~ /N/;
        $ty += $delta if $ecke->gewichtpos() =~ /S/;
        $tx -= $delta if $ecke->gewichtpos() =~ /W/;
        $tx += $delta if $ecke->gewichtpos() =~ /O/;
        (my $anchor = $ecke->gewichtpos()) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $ecke->gewicht(),
                       -anchor => $anchor,
                       -fill   => $ecke->gewichtfarbe(),
                      );
    }
    if ($Konfig{zeige_e_nr}) {
        my $pos = 'O';
        my $delta = 1 + 2 * ($ecke->radius() || $Graph->radius());
        $delta = int($delta / sqrt 2) if length $pos > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $pos =~ /N/;
        $ty += $delta if $pos =~ /S/;
        $tx -= $delta if $pos =~ /W/;
        $tx += $delta if $pos =~ /O/;
        (my $anchor = $pos) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $e,
                       -anchor => $anchor,
                       -fill   => $farbe,
                      );
    }
} # sub zeichne_ecke


sub zeichne_kante ($) {
    #--------------------------------------------------------------------------
    # sub          : z e i c h n e _ k a n t e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zeichnet die angegebene Kante ein.
    # Parameter    : Kantennummer 0 .. $Graph{m}-1
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 03.02.2003 - CD - Erstellt
    # 0.0.2  - 04.02.2003 - CD - Programmabbruch bei undefinierten Koordinaten
    #                            eingebaut.
    # 0.0.3  - 11.02.2003 - CD - Angepasst an neue Linientypennamen.
    # 0.0.4  - 18.03.2003 - CD - Neue Linientypenkrzel verwendet.
    # 0.0.5  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.6  - 07.02.2004 - CD - Verwendung des Farbwertes $Graph{kantenfarbe}
    #                            statt hardvercodetem Schwarz.
    # 0.0.7  - 09.02.2004 - CD - Arrowshape abhngig vom Radius.
    # 0.0.8  - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.9  - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                            Umstellung auf die Eckenklasse.
    # 0.0.10 - 12.02.2004 - CD - Neue Hilfsmethoden x_von_a etc. verwendet.
    # 0.0.11 - 30.02.2004 - CD - Beachtung einzelner Farbwerte der Ecken.
    # 0.0.12 - 01.01.2005 - CD - Titel und Gewicht mit ausgeben.
    # 0.0.13 - 01.01.2005 - CD - Kantendicke bercksichtigen.
    #                            Kanten werden nicht mehr von den
    #                            Mittelpunkten, sondern von den Eckenrndern
    #                            aus gezogen (Koordinatenkorrektur).
    #                            Der Linienstil (Normal / Gepunkted / Ge-
    #                            strichelt) wird jetzt untersttzt.
    # 0.0.14 - 03.01.2005 - CD - Anchor-Berechnung vereinfacht.
    #                            Titel- und Gewichtfarbe bercksichtigen.
    # 0.0.15 - 14.01.2005 - CD - Kantennummern auf Wunsch anzeigen.
    #--------------------------------------------------------------------------

    my $k = shift;

    my $kante = $Graph->kante($k);
    my $form  = $kante->form();

    die "Ecken von Kante $k sind undefiniert!" unless defined $form;

    my $xa   = $Graph->x_von_a_index($k);
    my $ya   = $Graph->y_von_a_index($k);
    my $xe   = $Graph->x_von_e_index($k);
    my $ye   = $Graph->y_von_e_index($k);

    die "Koordinaten der Anfangsecke sind undefiniert!" unless defined $xa and
                                                               defined $ya;
    die "Koordinaten der Endecke sind undefiniert!"     unless defined $xe and
                                                               defined $ye;

    print "zeichne_kante, k = '$k', (M = '",
          $Graph->m(), "')\n",
          "    ",   $kante->a(),
          " --- ",  $kante->e(),
          ",    (", $xa,
          ", ",     $ya,
          " -> ",   $xe,
          ", ",     $ye,
          ")\n"
          if $Konfig{debug} > 1;

    my $arrow = 'none';

    print "Kante $k von Ecke ", $kante->a(),
          " nach Ecke ", $kante->e(),
          ", ($xa,$ya) -> ($xe, $ye)\n"
          if $Konfig{debug} > 1;

    $arrow = 'last'  if $form eq 'k_inh';
    $arrow = 'first' if $form eq 'k_inz';
    $arrow = 'both'  if $form eq 'k_inb';

    ###########################################################################
    # Koordinatenkorrektur um die Kanten nur zwischen den Ecken zu ziehen und
    # nicht von Mittelpunkt zu Mittelpunkt:
    ###########################################################################
    my $ra   = $Graph->ecke($kante->a())->radius() || $Graph->radius();
    my $re   = $Graph->ecke($kante->e())->radius() || $Graph->radius();
    my ($xa_, $ya_, $xe_, $ye_) = ($xa, $ya, $xe, $ye);
    ($xa_, $xe_) = ($xe_, $xa_) if $xe_ < $xa_;
    ($ya_, $ye_) = ($ye_, $ya_) if $ye_ < $ya_;
    my $phi = $xe_ == $xa_ ? PI/2 : atan(($ye_-$ya_)/($xe_-$xa_));
    my $rxa = $ra * cos($phi);
    my $rya = $ra * sin($phi);
    my $rxe = $re * cos($phi);
    my $rye = $re * sin($phi);
    if ($xa == $xe) {                  # Y-Achse
        if ($ya <= $ye) {
            ($xa_, $ya_, $xe_, $ye_) = ($xa, int($ya + $rya),
                                        $xe, int($ye - $rye));
        }
        else {
            ($xa_, $ya_, $xe_, $ye_) = ($xa, int($ya - $rya),
                                        $xe, int($ye + $rye));
        }
    }
    elsif ($ya == $ye) {               # X-Achse
        if ($xa <= $xe) {
            ($xa_, $ya_, $xe_, $ye_) = (int($xa + $rxa), $ya,
                                        int($xe - $rxe), $ye);
        }
        else {
            ($xa_, $ya_, $xe_, $ye_) = (int($xa - $rxa), $ya,
                                        int($xe + $rxe), $ye);
        }
    }
    elsif ($xe >= $xa && $ye >= $ya) { # Q1
        ($xa_, $ya_, $xe_, $ye_) = (int($xa + $rxa), int($ya + $rya),
                                    int($xe - $rxe), int($ye - $rye));
    }
    elsif ($xe <= $xa && $ye <= $ya) { # Q2
        ($xa_, $ya_, $xe_, $ye_) = (int($xa - $rxa), int($ya - $rya),
                                    int($xe + $rxe), int($ye + $rye));
    }
    elsif ($xe <= $xa && $ye >= $ya) { # Q3
        ($xa_, $ya_, $xe_, $ye_) = (int($xa - $rxa), int($ya + $rya),
                                    int($xe + $rxe), int($ye - $rye));
    }
    elsif ($xe >= $xa && $ye <= $ya) { # Q4
        ($xa_, $ya_, $xe_, $ye_) = (int($xa + $rxa), int($ya - $rya),
                                    int($xe - $rxe), int($ye + $rye));
    }
    else {
        print "FEHLER in zeichne_kante()!\n";
    }
    ($xa, $ya, $xe, $ye) = ($xa_, $ya_, $xe_, $ye_);
    ###########################################################################


    my %dash = ( normal      => [1],
                 gestrichelt => '-',
                 gepunktet   => '.',
               );
    my $farbe = $kante->farbe() || $Graph->kantenfarbe();
    my $dicke = $kante->dicke() || $Graph->kantendicke();
    if ($arrow eq 'none') {
        $Konfig{canvas}->
            createLine($xa, $ya, $xe, $ye,
                       -width => $dicke,
                       -fill  => $farbe,
                       -dash  => $dash{$kante->stil()},
                      );
    }
    else {
        $Konfig{canvas}->
            createLine($xa, $ya, $xe, $ye,
                       -width      => $dicke,
                       -arrow      => $arrow,
                      #-arrowshape => [5, 7, 3],
                       -arrowshape => [
                                       2*$Graph->radius()-1 + $dicke-1,
                                       2*$Graph->radius()+1 + $dicke-1,
                                       $Graph->radius() + $dicke-1,
                                      ],
                       -fill       => $farbe,
                       -dash       => $dash{$kante->stil()},
                      );
    }


    my $x = int(($xa + $xe)/2); # Mittelpunkt der Kante
    my $y = int(($ya + $ye)/2); # berechnen...
    if (length $kante->titel()) {
        my $delta = 1 + $Graph->radius();
        $delta = int($delta / sqrt 2) if length $kante->titelpos() > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $kante->titelpos() =~ /N/;
        $ty += $delta if $kante->titelpos() =~ /S/;
        $tx -= $delta if $kante->titelpos() =~ /W/;
        $tx += $delta if $kante->titelpos() =~ /O/;
        (my $anchor = $kante->titelpos()) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $kante->titel(),
                       -anchor => $anchor,
                       -fill   => $kante->titelfarbe(),
                      );
    }
    if (defined $kante->gewicht()) {
        my $delta = 1 + $Graph->radius();
        $delta = int($delta / sqrt 2) if length $kante->gewichtpos() > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $kante->gewichtpos() =~ /N/;
        $ty += $delta if $kante->gewichtpos() =~ /S/;
        $tx -= $delta if $kante->gewichtpos() =~ /W/;
        $tx += $delta if $kante->gewichtpos() =~ /O/;
        (my $anchor = $kante->gewichtpos()) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $kante->gewicht(),
                       -anchor => $anchor,
                       -fill   => $kante->gewichtfarbe(),
                      );
    }
    if ($Konfig{zeige_k_nr}) {
        $Konfig{canvas}->
            createText($x, $y,
                       -text   => $k,
                       -fill   => $farbe,
                      );
    }
} # sub zeichne_kante


sub lade_datei () {
    #--------------------------------------------------------------------------
    # sub          : l a d e _ d a t e i
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ein Dialog zum Laden einer Graphendatei.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 13.02.2003 - CD - Endlich wird der Klick nach Doppelklick nicht
    #                           mehr in dem Canvas ausgewertet.
    #                           Dank Tk::after / Ishka von www.perl.de :)
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 01.01.2005 - CD - Ldt ged und ged2.
    # 0.0.5 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.6 - 16.01.2005 - CD - Ldt ged3 (default).
    #--------------------------------------------------------------------------

    print "lade_datei\n" if $Konfig{debug};

    #
    # Bindungen zu <Motion> und <Button-1> lsen:
    # (sonst setztz man mit dem Click auf OK ev eine Ecke, falls darunter das
    # Canvas-Widget ist.)
    #
    #my @bindings  = $Konfig{canvas}->Tk::bind("<Motion>");
    #print "Bindungen: '", @bindings, "'\n";
    #my @bindings2  = $Konfig{canvas}->Tk::bind("<Button-1>");
    #print "Bindungen2: '", @bindings2, "'\n";
    $Konfig{canvas}->Tk::bind("<Motion>",    "");
    $Konfig{canvas}->Tk::bind("<Button-1>",  "");
    #$Konfig{canvas}->Tk::bind("<Button-1>",  sub {print "JAUJAUJAU\n"});
    #@bindings  = $Konfig{canvas}->Tk::bind("<Motion>");
    #print "Bindungen: '", @bindings, "'\n";
    #@bindings2  = $Konfig{canvas}->Tk::bind("<Button-1>");
    #print "Bindungen2: '", @bindings2, "'\n";

    my $typen = [
        ['Alle Graphen Dateien',          ['.ged', '.ged2', '.ged3']],
        ['Graphen Dateien (ged3-Format)', '.ged3'                   ],
        ['Graphen Dateien (ged2-Format)', '.ged2'                   ],
        ['Graphen Dateien (ged-Format)',  '.ged'                    ],
        ['Alle Dateien',                  '*'                       ],
    ];
    my $dn = $Konfig{mw}->getOpenFile(-filetypes  => $typen,
                              -initialdir => $Konfig{pfad},
                             );

    if (defined $dn and $dn ne "") {
        $Graph->dateiname($dn);
        print "Dateiname = '$dn'\n" if $Konfig{debug};
        $Konfig{mw}->title('G - E - D   ' . $Konfig{version} . "   -   ($dn)");
        clear(0);
        load($dn);
        refresh();
    }

    #
    # Bindungen wieder herstellen:
    #
    #@bindings  = $Konfig{canvas}->Tk::bind("<Motion>");
    #print "Bindungen: '", @bindings, "'\n";
    #@bindings2  = $Konfig{canvas}->Tk::bind("<Button-1>");
    #print "Bindungen2: '", @bindings2, "'\n";
    $Konfig{mw}->after(1000, \&bind_start);
} # sub lade_datei


sub speichere_datei () {
    #--------------------------------------------------------------------------
    # sub          : s p e i c h e r e _ d a t e i
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ein Dialog zum Speichern einer Graphendatei.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2003 - CD - Doppelten Aufruf von save im Fall von Speichern
    #                           mit leerem Dateinamen verhindert.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    #--------------------------------------------------------------------------

    print "speichere_datei, Dateiname ist '", $Graph->dateiname(), "'\n"
          if $Konfig{debug};

    if ($Graph->dateiname() eq '') {
        speichere_datei_als();
    }
    else {
        save($Graph->dateiname());
    }
} # sub speichere_datei


sub speichere_datei_als () {
    #--------------------------------------------------------------------------
    # sub          : s p e i c h e r e _ d a t e i _ a l s
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ein Men zum Speichern einer Graphendatei unter einem an-
    #                deren Namen.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 11.02.2003 - CD - -initialfile rausgenommen wegen Problemen in
    #                           tk.pm
    # 0.0.3 - 17.02.2003 - CD - Dateinamen in Historie eingetragen.
    #                           Rumpfnamen als Vorschlag (-initialfile) ber-
    #                           geben.
    # 0.0.4 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.5 - 01.01.2005 - CD - Speichert in den Formaten ged und ged2
    #                           (default).
    # 0.0.6 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.7 - 16.01.2005 - CD - Neues Format ged3 ist default.
    # 0.0.8 - 17.01.2005 - CD - Punkt in Dateinamen einfgen, falls er fehlt
    #                           (dies ist unter Linux der Fall).
    #--------------------------------------------------------------------------

    print "speichere_datei_als, Dateiname ist '", $Graph->dateiname(), "'\n"
        if $Konfig{debug};

    my $name = $Graph->dateiname();
    $name =~ s~.*/([^/]*)\.ged[23]?$~$1~g; #~

    my $typen = [
        ['Graphen Dateien (ged3-Format)', '.ged3'],
        ['Graphen Dateien (ged2-Format)', '.ged2'],
        ['Graphen Dateien (ged-Format)',  '.ged' ],
        ['Alle Dateien',                  '*'    ],
    ];
    my $dn = $Konfig{mw}->getSaveFile(-filetypes        => $typen,
                              -defaultextension => '.ged3',
                              -initialfile      => $name,
#                              -initialdir       => $Konfig{pfad},
# Entfllt, da die Datei mit absolutem Pfad angegeben wird...
                             );

    if (defined $dn and $dn ne "") {
        #if ($dateiname =~ m~[^.]ged\d?$~) {
        #    $dateiname =~ s~([^.])(ged\d?$)~$1.$2~;
        #}
        $Graph->dateiname($dn);
        print "Dateiname = '$dn'\n" if $Konfig{debug};
        $Konfig{mw}->title('G - E - D   ' . $Konfig{version} . "   -   ($dn)");
        speichere_datei();
        add($dn);
    }
} # sub speichere_datei_als


sub lade_hintergrundbild () {
    #--------------------------------------------------------------------------
    # sub          : l a d e _ h i n t e r g r u n d b i l d
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ein Dialog zum Laden des Hintergrundbildes.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 14.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "lade_hintergrundbild\n" if $Konfig{debug};

    #
    # Bindungen zu <Motion> und <Button-1> lsen:
    # (sonst setztz man mit dem Click auf OK ev eine Ecke, falls darunter das
    # Canvas-Widget ist.)
    #
    $Konfig{canvas}->Tk::bind("<Motion>",    "");
    $Konfig{canvas}->Tk::bind("<Button-1>",  "");

    my $typen = [
        ['GIF- und JPG-Dateien',   ['gif', '.jpg', '.jpeg']],
        ['JPG-Dateien',            ['.jpg', '.jpeg']       ],
        ['GIF-Dateien',            '.ged'                  ],
        ['Alle Dateien',           '*'                     ],
    ];
    my $dn = $Konfig{mw}->getOpenFile(-filetypes  => $typen,
                              -initialdir => $Konfig{pfad},
                             );

    if (defined $dn and $dn ne "") {
        aenderung();
        $Graph->hintergrundbild($dn);
        refresh();
    }

    #
    # Bindungen wieder herstellen:
    #
    $Konfig{mw}->after(1000, \&bind_start);

} # sub lade_hintergrundbild


sub export_nach_latex () {
    #--------------------------------------------------------------------------
    # sub          : e x p o r t _ n a c h _ l a t e x
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ein Men zum Speichern einer Graphendatei im LaTeX-Format.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 13.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    #                           toLaTeX nach Graph2D verschoben.
    #--------------------------------------------------------------------------

    print "export_nach_latex, Dateiname ist '", $Graph->dateiname(), "'\n"
        if $Konfig{debug};

    ( my $name = $Graph->dateiname() ) =~ s/\.ged$/.tex/;

    my $typen = [
        ['LaTeX-Dateien',           '.tex'],
        ['Interne Graphen Dateien', '.ged'],
        ['Alle Dateien',            '*'   ],
    ];
    my $dn = $Konfig{mw}->
                getSaveFile(-title            => "Graph nach LaTeX exportieren",
                            -filetypes        => $typen,
                            -defaultextension => 'tex',
#                             -initialfile      => $name,
# macht leider auch Probleme
#                             -initialfile      => $Graph->dateiname(),
# Dann knallts in tk.pm ...
#                             -initialdir       => $Konfig{pfad},
# Entfllt, da die Datei mit absolutem Pfad angegeben wird...
                             );

    if (defined $dn and $dn ne "") {
        print "LaTeX-Dateiname = '$dn'\n" if $Konfig{debug} > 1;
        $Graph->toLaTeX($dn);
        $Konfig{statuszeile} = "Datei als '$name' nach LaTeX exportiert.";
    }

} # sub export_nach_latex


sub export_nach_postscript () {
    #--------------------------------------------------------------------------
    # sub          : e x p o r t _ n a c h _ p o s t s c r i p t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ein Men zum Speichern einer Graphendatei im PS-Format.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 13.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    #--------------------------------------------------------------------------

    print "export_nach_postscript, Dateiname ist '", $Graph->dateiname(), "'\n"
        if $Konfig{debug};

    ( my $name = $Graph->dateiname() ) =~ s/\.ged$/.ps/;

    my $typen = [
        ['Postscript-Dateien',      '.ps'],
        ['Interne Graphen Dateien', '.ged'],
        ['Alle Dateien',            '*'   ],
    ];
    my $dn = $Konfig{mw}->
                getSaveFile(-title            => "Graph nach Postscript " .
                                                 "exportieren",
                            -filetypes        => $typen,
                            -defaultextension => 'ps',
#                            -initialfile      => $name,
# macht leider auch Probleme
#                            -initialfile      => $Graph->dateiname(),
# Dann knallts in tk.pm ...
#                            -initialdir       => $Konfig{pfad},
# Entfllt, da die Datei mit absolutem Pfad angegeben wird...
                             );

    if (defined $dn and $dn ne "") {
        print "Postscript-Dateiname = '$dn'\n" if $Konfig{debug} > 1;
        $Konfig{canvas}->postscript(-file => $dn);

        $Konfig{statuszeile} = "Datei als '$name' nach Postscript exportiert.";
    }
} # sub export_nach_postscript


sub dump_eingeben () {
    #--------------------------------------------------------------------------
    # sub          : d u m p _ e i n g e b e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ermglicht es, eine interne Datei / einen Dump per Hand
    #                einzugeben.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 12.03.2003 - CD - Funktionalitt eingebaut
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.4 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.5 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    # 0.0.6 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    #--------------------------------------------------------------------------

    print "dump_eingeben\n" if $Konfig{debug};

    sub load_dump {
        print "load_dump\n" if $Konfig{debug};

        my $text = shift;

        $Graph->clear();

        # Linientypenhash aufbauen:
        my %typen;
        for (keys %Linientypen) {
            $typen{$Linientypen{$_}} = $_;
        }


        my @lines = split /\n/, $text;

        my $hx = 0;
        my $hy = 0;
        my $he = 0;
        my $ha = 0;
        my $hf = 0;

        my $l = ''; # aktuelle Zeile
        my $z = 0;  # Zeilennummer

        # Kommentarkopf berlesen:
        do {
            $l = shift @lines;
            ++$z;
        } while '#' eq substr $l, 0, 1;

        # Eckenzahl einlesen:
        return "Fehler in Zeile $z - N fehlt!" unless defined $l;
        (my $n) = $l =~ /^\s*(\d+)\s*$/;
        return "Fehler in Zeile $z - N fehlt!" unless defined $1;


        # Eckenkoordinaten einlesen:
        for (0..$n-1) {
            $l = shift @lines;
            ++$z;
            return "Fehler in Zeile $z - Eckenkoordinate fehlt!"
                unless defined $l;
            $l =~ /^\s*(\d+)\s*;\s*(\d+)\s*$/;
            return "Fehler in Zeile $z - Eckenkoordinate fehlt!"
                unless defined $1 and defined $2;
            $Graph->ecke_einfuegen($1, $2);
        }


        # Kantenzahl einlesen:
        $l = shift @lines;
        ++$z;
        return "Fehler in Zeile $z - M fehlt!" unless defined $l;
        chomp $l;
        (my $m) = $l =~ /^\s*(\d+)\s*$/;

        return "Fehler in Zeile $z - M fehlt!" unless defined $1;

        # Eckenkoordinaten einlesen:
        for (0..$m-1) {
            $l = shift @lines;
            ++$z;
            return "Fehler in Zeile $z - Kantendaten fehlen!"
                unless defined $l;
            $l =~ /^\s*(\d+)\s*;\s*(\d+)\s*;\s*(.*)\s*$/;
            return "Fehler in Zeile $z - Kantendaten fehlen!"
                unless defined $1 and defined $2 and defined $3;
            $Graph->kante_einfuegen($1, $2, exists $typen{$3}?$typen{$3}:$3);
        }

        return '';
    } # sub load_dump


    #
    # Neues Toplevel-Fenster zur Texteingabe erzeugen:
    #
    my $tl = $Konfig{mw}->Toplevel();
    $tl->title('Interne Daten');
    $tl->Label(
        -text   => 'Interne Datenstruktur im GED-Format:',
    )->pack(
        -anchor => 'w',
    );
    my $txt = $tl->Scrolled("Text",
                        -scrollbars => 'osoe',
                        -background => '#d7d7d7',
                        -foreground => 'blue',
                        -width      => 40,
                        -height     => 25,
                        -relief     => 'flat',
                        -font       => '{Courier New} 10 {normal}',
                        -wrap       => 'none',
                       )
                ->pack();
    $tl->Button(-text    => 'Dump laden',
                -command => sub {aenderung();
                                 neue_datei();
                                 my $rtext = load_dump($txt->get('1.0', 'end'));
                                 if ($rtext eq '') {
                                     $Graph->dateiname('');
                                     $Konfig{mw}->title('G - E - D   ' . $Konfig{version});
                                     $Konfig{statuszeile} = 'Dump erfolgreich eingelesen.';
                                     refresh();
                                     $tl->destroy();
                                 }
                                 else {
                                     print "\a";
                                     $Konfig{statuszeile} = $rtext;
                                 }
                                },
               )
         ->pack(-expand => '1',
                -fill   => 'x',
                -side   => 'left',
               );


    #
    # Rest: grab verhindert das Zurckschalten auf $WM per Klick, focus sorgt
    #       dafr, dass $tl und in diesem $txt auch wirklich aktiv ist:
    #
    $tl->grab();
    $txt->focus();
} # sub dump_eingeben


sub zeige_daten_an ($) {
    #--------------------------------------------------------------------------
    # sub          : z e i g e _ d a t e n _ a n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zeigt die interne Datenstruktur in einem Fenster an.
    # Parameter    : Format ('.ged', '.ged2' ...)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.3 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.4 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.5 - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.6 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    # 0.0.7 - 16.01.2005 - CD - Die Daten werden jetzt in eine temporre Datei
    #                           gespeichert.
    #--------------------------------------------------------------------------

    my ($w, $format) = @_;
    $format = $w unless defined $format; # Mal aufruf via bind mal direkt.

    print "zeige_daten_an (Format '$format')\n" if $Konfig{debug};

    #
    # Temporren Dateinamen ermitteln:
    # 
    my ($fh, $fn) = File::Temp::tempfile('tempXXXXX',
                                         SUFFIX => $format,
                                         DIR    => "$FindBin::RealBin/data",
                                        );
    close $fh; # Sonst funktioniert das unlink nicht.

    #
    # Graphen unter diesem Namen speichern:
    # 
    my $changed  = $Konfig{changed};
    my $dateiname = $Graph->dateiname();

    $Graph->save($fn);
    $Konfig{changed} = $changed;
    $Graph->dateiname($dateiname);

    #
    # Geschriebene Datei einlesen:
    #
    open (F, $fn) or die $!;
    my $text;
    {local $/; $text = <F>;}
    close F or warn $!;

    #
    # Temporre Datei lschen:
    # 
    unlink $fn;

    #
    # Text ausgeben:
    #
    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         "Interne Daten im '$format'-Format",
                                         'Interne Datenstruktur:',
                                         '{Courier New} 10 {normal}',
                                         $text,
                                         0,
                                        );
} # sub zeige_daten_an


sub zeige_daten_an_dumper () {
    #--------------------------------------------------------------------------
    # sub          : z e i g e _ d a t e n _ a n _ d u m p e r
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zeigt die interne Datenstruktur in einem Fenster an.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 31.12.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "zeige_daten_an_dumper\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'Interne Daten (Dumper)',
                                         'Interne Datenstruktur gedumpt:',
                                         '{Courier New} 10 {normal}',
                                         Dumper($Graph),
                                         0,
                                        );
} # sub zeige_daten_an_dumper


sub hilfe () {
    #--------------------------------------------------------------------------
    # sub          : h i l f e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zeigt einen Hilfstext an.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2003 - CD - Neues Toplevel-Fenster zur Anzeige spendiert.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.5 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    #--------------------------------------------------------------------------

    print "hilfe\n" if $Konfig{debug};

    my $text = <<"HILFETEXT";
     GED   V$Konfig{version} vom $Konfig{version_datum}
     ~~~

...
HILFETEXT

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'Hilfe zu GED',
                                         'Hilfe:',
                                         '{Courier New} 10 {normal}',
                                         $text,
                                         0,
                                        );
} # sub hilfe


sub graph_erkl () {
    #--------------------------------------------------------------------------
    # sub          : g r a p h _ e r k l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Gibt eine Erklrung zu Graphen aus.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2003 - CD - Neues Toplevel-Fenster zur Anzeige spendiert.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.5 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    #--------------------------------------------------------------------------

    print "graph_erkl\n" if $Konfig{debug};

    my $text = <<'GRAPHERKL';
G - E - D ist ein grafischer Editor fr Graphen aus der Graphentheorie.

Graphen bestehen aus einer Menge von Ecken N = {e_1, e_2, ..., e_n} und einer
Menge von Kanten M = {k_1, k_2, ..., k_m}, welche jeweils zwei Ecken
miteinander verbinden.

Die Kanten kann man sich deshalb jeweils als ein Paar von Ecken vorstellen,
wenn die Kante k1 die Ecke e1 mit der Ecke e3 verbindet, kann man schreiben:
k1 = (e1, e3).
GRAPHERKL

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'Was sind Graphen?',
                                         'Was sind Graphen?',
                                         '{Courier New} 10 {normal}',
                                         $text,
                                         0,
                                        );
} # sub graph_erkl


sub history () {
    #--------------------------------------------------------------------------
    # sub          : h i s t o r y
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zeigt die History an.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 13.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.4 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    #--------------------------------------------------------------------------

    print "history\n" if $Konfig{debug};

    #
    # History-Text:
    #
    my $text = <<'HISTORYTEXT';
Version 0.0.1 - 31.01.2003 (6.882 Byte - 195 Zeilen - ged_2003_01_31.pl)
~~~~~~~~~~~~~
Aufbauend auf dem Beispiel "Ein Zeichenprogramm" auf Seite 213/214 im Buch
"Einfhrung in Perl / Tk" von Nancy Walsh entsteht das Grundgerst.

Man kann Kreise setzten (Ecken) und beliebige Striche ziehen.

Funktionen:
    tk_start
    bind_start
    zeichnen_anfang
    aendere_item
    zeichnen_ende


Version 0.0.2 - 03.02.2003 (25.370 Byte - 678 Zeilen - ged_2003_02_03.pl)
~~~~~~~~~~~~~
- Ideensammlung relativ am Anfang der Datei als Kommentare eingefgt.
- $Canvas eingefhrt.
- weitere globale Variablen: $N, $Radius, $Delta, @EckenX, @EckenY, @KantenA,
  @KantenE, @Kantenform, $M, $KantenA, $Kantenform, $Zx, $Zy.
- Clear-Button rechts oben eingefgt.
- Quit- und Refresh-Button eingefgt.
- ESC an Programmende und F5 an refresh gebunden.
- Zeichenanfangsecke ($KantenA) und Kantenform ($Kantenform) merken.
- Eckenkoordinaten beim Zeichnen in den Arrays ablegen und $N hochzhlen.
- Eckenlschen eingebaut.
- Kanten drfen nur noch in der Nhe von Ecken beginnen und enden und rasten
  dann auf diesen ein.

Neue Funktionen:
    ecke_zu_nah
    meint_ecke
    beenden
    r2d
    refresh
    clear
    zeichne_ecke
    zeichne_kante

Version 0.0.3 - 04.02.2003 (29.915 Byte - 800 Zeilen - ged_2003_02_04_a.pl)
~~~~~~~~~~~~~
- Men eingefgt.
- Skalaelement zur Grennderung des Radius der Eckendarstellung eingefgt.
- Nach Eckenlschen hhere Eckennummern korrigiert.
- Ausgabe aus beenden nach save verschoben, in beenden save aufgerufen.
- Debug-Startausgaben mit Funktionsnamen in jede Funktion eingebaut.
- In clear Motion-Bindung aufheben - Zeichenvorgang wird damit unterbrochen.
- Kantenzeichnen korrigiert.

Neue Funktionen:
    save
    ueber_ged (leerer Rumpf)
    neue_datei (leerer Rumpf)
    lade_datei (leerer Rumpf)
    speichere_datei (leerer Rumpf)
    speichere_datei_als (leerer Rumpf)


Version 0.0.4 - 04.02.2003 (48.289 Byte - 1253 Zeilen - ged_2003_02_04_b.pl)
~~~~~~~~~~~~~
- $Debug und $Dateiname als globale Variablen eingefhrt.
- $MW eingefhrt.
- Hilfemen um Graphenerklrung und "ber GED" erweitert.
- Acceleratoren im Men eingebaut (Tastenkrzelanzeige).
- Dummy-Bearbeitenmen und Internamen eingebaut.
- Einige Tastaturbindungen vorgenommen.
- bind_start erhlt jetzt das Canvas nicht mehr als Parameter
- Data::Dumper zu Debug-Zwecken verwendet.
- Einige Ausgaben von $Debug abhngig gemacht.
- beenden ruft nicht mehr save auf.
- clear erhlt jetzt das Canvas nicht mehr als Parameter und setzt die globalen
  Ecken / Kantenarrays auf leere Listen nachdem sie auf undef gesetzt wurden.
- zeichne_ecke bricht jetzt bei falschem Aufruf mit Fehler ab.
- zeichne_kante bricht jetzt bei falschem Aufruf mit Fehler ab.
- save schreibt in eine Datei ($Dateiname) und nicht nur auf den Bildschirm.
- ueber_ged, neue_datei, lade_datei, speichere_datei mit Leben gefllt,
  waren vorher leere Blcke.

Neue Funktionen:
    dump_data
    load
    graph_erkl
    zeige_daten_an
    hilfe
    dump_eingeben (noch ohne Funktionalitt)


Version 0.0.5 - 04.02.2003 (49.134 Byte - 1283 Zeilen - ged_2003_02_04_c.pl)
~~~~~~~~~~~~~
- Cwd zur Ermittlung des Arbeitsverzeichnisses verwendet. (-> $Pfad)
- in load Laden der Eckenzahl korrigiert und das Einlesen der Kanten eingebaut.
- $Pfad per -initialdir in speichere_datei_als verwendet.


Version 0.0.6 - 05.02.2003 (65.573 Byte - 1724 Zeilen - ged_2003_02_05.pl)
~~~~~~~~~~~~~
- $Debug wird als erste Variable definiert.
- Neue Methode "Ecke verschieben 2" (Linientyp).
- Linientyp "Linie" heit jetzt "Kante".
- Neue Methode "Kante lschen" (Linientyp).
- Statuszeilenausgaben an verschiedenen Stellen eingebaut.
- Speichern _unter_ statt Speichern _als_.
- ndern der Radiusgre fr die Eckenkreise fhrt jetzt ein automatisches
  refresh durch, d.h. man sieht die nderung gleich auf dem Bildschirm.
- Methoden zum Ecken verschieben (2 Stck) und Kanten lschen realisiert.
- Schleifen verhindert, d.h. Kanten drfen nur noch zwischen verschiedenen
  Ecken verlaufen.
- meint_ecke durch last beschleunigt.
- Parameter in clear() eingebaut, Bindungen werden nur auf Wunsch wieder
  hergestellt.
- Viele Ausgaben abhngig von $Debug gemacht (verschiedene Level).
- Titelanzeige bei neue_datei und lade_datei gendert.
- Doppelten Aufruf von Save in speichere_datei verhindert.
- -initialdir in speichere_datei_als ausgebaut, wird mit absolutem Pfad
  angegeben und fhrt dann zu Fehlern.
- graph_erkl mit neuem Toplevel-Window versehen.
- Hilfe mit neuem Toplevel-Window versehen.

Neue Funktionen:
    ecke_zu_nah_ohne
    meint_kante (aber noch nicht funktionstchtig...)


Version 0.0.7 - 08.02.2003 (71.673 Byte - 1863 Zeilen - ged_2003_02_08.pl)
~~~~~~~~~~~~~
- Linientyp "Ecke" in "Ecke einfgen" gendert.
- Icon fr GED erzeugt und dieses in tk_start dem MainWindow zugewiesen.
- Arbeitswerkzeugwahl nun ber Editbuttons am oberen Rand und nicht mehr ber
  Radioboxes am rechten Rand.


Version 0.0.8 - 10.02.2003 (88.312 Byte - 2249 Zeilen - ged_2003_02_10.pl)
~~~~~~~~~~~~~
- Linientypennamen gendert.
- Durchgngigen Menufont in allen Menelementen verwendet.
- Editbuttons in drei Gruppen (erzeugen, modifizieren und lschen) aufgeteilt
  und optisch voneinander abgesetzt.
- Tests eingebaut, so dass man keine Kante einfgen kann, wenn nur eine Ecke da
  ist etc.
- Kantenlschen: Kantenzahl runterzhlen und refresh eingebaut.
- Neues Icon auch bei "ber GED" verwendet.
- meint_kante mit der richtigen Funktion befllt.

Neue Funktionen:
    abstand_punkt_linie
    punkt_abstand
    floor
    do_all_the_tests


Version 0.0.9 - 11.02.2003 (95.174 Byte - 2401 Zeilen - ged_2003_02_11.pl)
~~~~~~~~~~~~~
- Gesamtes Mensystem umgestellt nach ausfhrlichem Schmkern in den perldocs
  (menu, ...), dies wurde in obigem Buch nicht vernnftig erklrt, jetzt hat
  das Programm ein richtig "normales" Men, in das man mit Alt-Buchstabe, Alt,
  F10 wechseln und in dem man mit den Cursortasten gewohnt navigieren kann.
- Linientypennamen gendert (Nacharbeiten).
- -initialfile rausgenommen in speichere_datei_als.


Version 0.0.10 - 12.02.2003 (98.898 Byte - 2492 Zeilen - ged_2003_02_12_a.pl)
~~~~~~~~~~~~~~
- Im Men Bearbeiten kann man nun Kreise und vollstndige Graphen einfgen
  (C3, C4, C5, C6, C7, C8, C9, K3, K4, K5, K6, K7).

Neue Funktionen:
    struktur_einfuegen


Version 0.0.11 - 12.02.2003 (107.338 Byte - 2669 Zeilen - ged_2003_02_12_b.pl)
~~~~~~~~~~~~~~
- Funtion todo eingefgt, ToDo-Texte sind als HERE-Dokument in diese Funktion
  gewandert.
- Grob geschtzte Versionsnummer $Version (5) und Datum $Datum eingefgt.
  Version und Datum werden nun bei "ber GED" angezeigt.
- Grafische Elemente (Icons etc.) ins Unterverzeichnis res verschoben.
- Weitere Kn's (K8 bis K13, K20, K30, K40, K50, K99), sowie Cn's (C20, C30,
  C40, C50, C99) sowie das Einfgen des Mittelpunktes in das Bearbeitenmen
  aufgenommen.
- ESC beendet nun nicht mehr das Programm, sondern Ctrl-B. Entsprechend wird
  ESC in den Diversen Funktionen mit eigenen Toplevel-Windows hinterher auch
  nicht wieder an $MW gebunden.
- Fehlermeldung in meint_kante ausgeben, wenn es uneindeutig ist.
- struktur_einfuegen erweitert / verbessert, Koordinatenberechnung ist nun
  richtig. Schleifeninvariante Variablen vor der Schleife deklariert und
  initialisiert.

Neue Funktionen:
    todo


Version 0.0.12 - 13.02.2003 (138.257 Byte - 3395 Zeilen - ged_2003_02_13.pl)
~~~~~~~~~~~~~~
- Speichern im LaTeX-Modus eingebaut
- Speichern im Postscript-Modus eingebaut
- Beim Doppelklick im Laden-Men wird nun dieser nicht mehr im Canvas ausgewer-
  tet, dank tK::after.
- Der ToDo-Text ist wieder an den Dateianfang gewandert, in eine globale
  Variable. (Here-Dokument)
- Diese History angelegt und teilweise rekonstruiert, Versionsnummern
  rckwirkend vergeben / korrigiert.
- Men "Bearbeiten 2" mit kascadierenden Submens als Alternative zu Men
  Bearbeiten eingefgt.
- Men Hilfe ist jetzt wieder rechtsbndig (durch letzten Seperator der
  Menubar!).
- In do_all_tests Zeilen umgebrochen, so dass sie in die ersten 80 Zeichen pro
  Zeile passen.

Neue Funktionen:
    history
    toLaTeX
    export_nach_latex
    export_nach_postscript


Version 0.0.13 vom 17.02.2003
~~~~~~~~~~~~~~
> 136.747 Byte - 3.405 Zeilen -      ged_2003_02_17.pl
>   5.816 Byte -   167 Zeilen - GEDfhist_2003_02_17.pm
> -------         ----
> 142.563 Byte - 3.572 Zeilen

- Verwaltung der zuletzt bearbeiteten Dateien in GEDfhist.pm / ged.pl eingebaut.
- Haupt-Men in globaler Variable $Menu zugnglich gemacht (um die vernderten
  bearbeiteten Dateien ins Men eintragen zu knnen).
  Dito $Menuefont, aber beide nur im Block mit tk_start und add_file_menu.
- Mainteil ans Ende verlegt, sonst sind die Blockvariablen nicht initialisiert.

Neue Funktionen:
    GEDfhist::load
    GEDfhist::save
    GEDfhist::add
    GEDfhist::array
    GEDfhist::min
    GEDfhist::max
    add
    add_file_menu


Version 0.0.14 vom 12.03.2003
~~~~~~~~~~~~~~
> 140.598 Byte - 3.519 Zeilen -      ged_2003_03_12.pl
>   5.816 Byte -   167 Zeilen - GEDfhist_2003_02_17.pm
> -------        -----
> 146.414 Byte - 3.686 Zeilen

- Funktion dump_eingeben mit Leben gefllt.

Neue Funktionen:
    dump_eingeben (war bisher nur ein Rumpf)
    load_dump (local in dump_eingeben)


Version 0.0.15 vom 18.03.2003
~~~~~~~~~~~~~~
> 142.919 Byte - 3.581 Zeilen -      ged_2003_03_18.pl
>   5.816 Byte -   167 Zeilen - GEDfhist_2003_02_17.pm
> -------        -----
> 148.735 Byte - 3.748 Zeilen

- Linientypen werden nun ber ein Hash definiert, das ein Krzel als Schlssel
  und den schnen Langtext als Wert hat. Gespeichert word nur der Schlssel,
  beim Laden werden Werte in Schlssel bertragen, um alte Dateien lesen zu
  knnen.
- Beim schnellen Eckenverschieben wird nun nicht mehr auf berschneidung mit
  der Ausgangsecke getestet.


Version 0.0.16 vom 05.02.2004
~~~~~~~~~~~~~~
> 151.988 Byte - 3.658 Zeilen -      ged_2004_02_05.pl
>   5.816 Byte -   167 Zeilen - GEDfhist_2003_02_17.pm
> -------        -----
> 157.804 Byte - 3.825 Zeilen

- Viele globale Variablen in Hashes %Konfig und %Graph zusammengefasst.


Version 0.0.17 vom 06.02.2004
~~~~~~~~~~~~~~
> 158.920 Byte - 3.840 Zeilen -      ged_2004_02_06.pl
>   5.816 Byte -   167 Zeilen - GEDfhist_2003_02_17.pm
> -------        -----
> 164.736 Byte - 4.007 Zeilen

- Einfgen von groben Kommentarblcken zur Abtrennung der Funktionen in
  Oberflche, Logik und Daten (Dateien), Sortieren der Funktionen.
- Verschieben der Kommentarblcke unter den Anfang der Sub's, damit man ganze
  Funktionen im Editor bersichtlich zusammenfalten kann.
- Es kann optional ein Gitterzetz in variabler Weite eingeblendet werden.
  (Geplant ist das Optionale Einrasten beim Setzen und Verschieben von Ecken
  auf den Ratserpunkten, die Variable und der Menpunkt sind auch schon da,
  die Funktionalitt fehlt aber noch.)

Neue Funktionen:
    tk_beenden


Version 0.0.18 vom 06.02.2004
~~~~~~~~~~~~~~
> 163.121 Byte - 3.982 Zeilen -    ged_2004_02_06_b.pl
>   5.554 Byte -   159 Zeilen - GEDfhist_2004_02_06.pm
> -------        -----
> 168.675 Byte - 4.141 Zeilen

- Einrastfunktion am Gitter eingebaut.
- Alle Funktionen mit Prototypen versehen und diese zu Beginn des Programms
  genannt.

Neue Funktionen:
    rast_ein


Version 0.0.19 vom 06.02.2004
~~~~~~~~~~~~~~
> 167.786 Byte - 4.108 Zeilen -    ged_2004_02_06_c.pl
>   5.554 Byte -   159 Zeilen - GEDfhist_2004_02_06.pm
> -------        -----
> 173.340 Byte - 4.267 Zeilen

- Schnittstelle von load und save gendert, nun muss der Dateiname als Para-
  meter angegeben werden.
- Neues Men "Manipulatoren" mit dem Punkt "In Liniengraphen umwandeln".
- Umwandlung des Graphen in seinen Liniengraphen

Neue Funktionen:
    erzeuge_liniengraphen


Version 0.0.20 vom 06.02.2004
~~~~~~~~~~~~~~
> 179.181 Byte - 4.418 Zeilen -    ged_2004_02_06_d.pl
>   5.554 Byte -   159 Zeilen - GEDfhist_2004_02_06.pm
> -------        -----
> 184.735 Byte - 4.577 Zeilen

- Funktion zur Ermittlung der Canvasgre vorbereitet und in refresh() und
  struktur_einfuegen() eingebaut. Bisher wird aber nur ein fest eingetragenes
  Gren-Quadrupel zurckgegeben.
- Verschiedenen Funktionen rund um Kanten und Ecken hinzugefgt und an anderen
  Stellen aufgerufen. (Verlagerung von Komplexitt).
- Eckenmergen realisiert.
- Todo-Texte aufgerumt.

Neue Funktionen:
    canvas_groesse
    ecke_einfuegen
    ecke_loeschen
    ecke_mergen
    kante_existiert
    kante_einfuegen
    kante_loeschen


Version 0.0.21 vom 07.02.2004
~~~~~~~~~~~~~~
> 182.814 Byte - 4.489 Zeilen -      ged_2004_02_07.pl
>   5.554 Byte -   159 Zeilen - GEDfhist_2004_02_06.pm
> -------        -----
> 188.368 Byte - 4.648 Zeilen

- Geometry zur Grenermittlung in canvas_groesse() verwendet.
- Refresh wird jetzt per Event bei Grennderungen des Fensters aufgerufen,
  so dass das Gitter und auch die eingefgten Graphen an die aktuelle
  Fenstergre angepasst werden.
- Ecken und Kantenfarbe ist nun nicht mehr fest rot/schwarz, sondern
  variabel $Graph{eckenfarbe} bzw. $Graph{kantenfarbe}.
- Ecken- und Kantenfarbe ber Men Ansicht whlbar.
- Waitcursor bei Liniengraphenberechnung eingeschaltet.


Version 0.0.22 vom 08.02.2004
~~~~~~~~~~~~~~
> 179.294 Byte - 4.374 Zeilen -      ged_2004_02_08.pl
>   5.584 Byte -   162 Zeilen - GEDfhist_2004_02_08.pm
> -------        -----
> 184.878 Byte - 4.536 Zeilen

- mit "use Tk::ErrorDialog;" werden jetzt grafische Fehlermeldungen erzeugt.
- anzeige_fenster() zum einfacheren Anzeigen der Daten in zeige_daten_an(),
  hilfe(), graph_erkl(), history() und todo() verwendet.

Neue Funktionen:
    anzeige_fenster


Version 0.0.23 vom 08.02.2004
~~~~~~~~~~~~~~
> 181.451 Byte - 4.425 Zeilen -    ged_2004_02_08_b.pl
>   5.584 Byte -   162 Zeilen - GEDfhist_2004_02_08.pm
> -------        -----
> 187.035 Byte - 4.587 Zeilen

- mit "use Tk::ErrorDialog;" werden jetzt grafische Fehlermeldungen erzeugt.
- anzeige_fenster() zum einfacheren Anzeigen der Daten in zeige_daten_an(),
  hilfe(), graph_erkl(), history() und todo() verwendet.
- Bug der nicht mehr funktionierenden Linienarten auer normal behoben (es
  fehlte ein Komma vor einem weiteren eingefgten Parameter [-fill]
  von createLine() hinter -arrowshape).


Version 0.0.24 vom 09.02.2004
~~~~~~~~~~~~~~
> 156.599 Byte - 3.713 Zeilen -      ged_2004_02_09.pl
>   5.584 Byte -   162 Zeilen - GEDfhist_2004_02_08.pm
>   9.064 Byte -   273 Zeilen -  GEDgeom_2004_02_08.pm
>  21.437 Byte -   528 Zeilen -    Graph_2004_02_09.pm
>  31.765 Byte -   776 Zeilen -  Graph2D_2004_02_09.pm
> -------        -----
> 224.449 Byte - 5.452 Zeilen

- linientyp, startecke, zx und zy sind jetzt nicht mehr in %Graph, sondern in
  %Konfig, da diese Informationen inhaltlich nicht zum Graphen, sondern zum
  Zeichnen desselben whrend des Linienziehens dienen.
- Gitterfarbe jetzt ber %Konfig einstellbar, nicht mehr in der Funktion hart
  verdrahtet.
- Funktionalitt ins neue Modul GEDgeom.pm ausgelagert (abstand_punkt_linie,
  punkt_abstand [und floor]).
- Erstes Bearbeiten Men auskommentiert.
- Menpunkt Ansicht erneuern aus dem Dateimen entfernt (ist auch im
  Ansichtsmen gewesen)
- Neue Klasen Graph und Graph2D, in die viel Funktionalitt ausgelagert wurde.

Ausgelagerte Funktionen (gelscht):
    ecke_zu_nah
    ecke_zu_nah_ohne
    meint_ecke
    meint_kante
    r2d
    ecke_einfuegen
    ecke_loeschen
    ecke_mergen
    kante_existiert
    kante_einfuegen
    kante_loeschen

Ausgelagerte Funktionalitten (entkernt):
    clear
    struktur_einfuegen
    erzeuge_liniengraphen

Neue Funktionen:
    waehle_eckenfarbe
    waehle_kantenfarbe


Funktionen in GEDgeom.pm:
    abstand_punkt_linie
    punkt_abstand
    floor

Methoden in Graph:
    new
    _property
    n
    m
    kantenform
    KantenA
    KantenE
    Kantenform
    ecke_einfuegen
    ecke_loeschen
    ecke_mergen
    kante_existiert
    kante_einfuegen
    kante_loeschen
    clear

Methoden in Graph2D:
    new
    radius
    delta
    eckenfarbe
    kantenfarbe
    EckenX
    EckenY
    ecke_einfuegen
    ecke_loeschen
    ecke_mergen
    clear
    ecke_zu_nah
    ecke_zu_nah_ohne
    meint_ecke
    meint_kante
    r2d
    struktur_einfuegen
    erzeuge_liniengraphen


Version 0.0.25 vom 09.02.2004
~~~~~~~~~~~~~~
> 161.976 Byte - 3.822 Zeilen -     ged_2004_02_09_b.pl
>   5.875 Byte -   177 Zeilen -  GEDfhist_2004_02_09.pm
>   9.358 Byte -   291 Zeilen -   GEDgeom_2004_02_09.pm
>  21.707 Byte -   545 Zeilen -   Graph_2004_02_09_b.pm
>  32.073 Byte -   795 Zeilen - Graph2D_2004_02_09_b.pm
> -------        -----
> 230.989 Byte - 5.630 Zeilen

- Bug beseitigt der Auftrat, wenn man im Gitterraster einrasten mchte ohne
  es anzuzeigen.
- Eckenradiuseinstellung von der Oberflche ins Men verlagert (abreibar)
- Gitterweitenmen abreibar gemacht
- Fehler behoben: Wurde im Farbwahlmen ESC gedrckt, so kommt keine Farbe
  zurck und die dargestellten Dinge (Ecken bzw. Kanten) verschwanden.
- POD angefangen mit Titel und Copyright, um die Sourcen zum Download anbieten
  zu knnen. (Sie sind damit unter der Doppellizenz GPL / artistic, siehe
  perldoc perlgpl bzw. perldoc perlartistic.)


Version 0.0.26 vom 10.02.2004
~~~~~~~~~~~~~~
> 163.838 Byte - 3.884 Zeilen -      ged_2004_02_10.pl
>   5.875 Byte -   177 Zeilen - GEDfhist_2004_02_09.pm
>   9.358 Byte -   291 Zeilen -  GEDgeom_2004_02_09.pm
>  25.986 Byte -   676 Zeilen -    Graph_2004_02_10.pm
>  37.248 Byte -   941 Zeilen -  Graph2D_2004_02_10.pm
>   7.283 Byte -   173 Zeilen -     Ecke_2004_02_10.pm
>   8.403 Byte -   199 Zeilen -    Kante_2004_02_10.pm
> -------        -----
> 257.991 Byte - 6.343 Zeilen

- Klassen fr Ecken und Kanten eingefhrt und die Graphenklassen um Hilfs-
  funktionen erweitert, so dass die interne Datenstruktur vor ged.pl total
  gekapselt ist.

Neue Methoden in Graph:
    _kanten
    setze_kanten
    loesche_kanten
    kante
    entnehme_kante
    fuege_kante_hinzu

Neue Methoden in Graph2D:
    new
    _ecken
    setze_ecken
    loesche_ecken
    ecke
    entnehme_ecke
    fuege_ecke_hinzu

Methoden in Ecke.pm:
    new
    _property
    x
    y

Methoden in Kante.pm:
    new
    _property
    a
    e
    form
    inzidiert_mit_ecke

Version 0.0.27 vom 10.02.2004
~~~~~~~~~~~~~~
> 164.922 Byte - 3.905 Zeilen -     ged_2004_02_10_b.pl
>   5.884 Byte -   177 Zeilen -     FHist_2004_02_10.pm
>   9.367 Byte -   291 Zeilen -      Geom_2004_02_10.pm
>  26.238 Byte -   676 Zeilen -   Graph_2004_02_10_b.pm
>  37.453 Byte -   941 Zeilen - Graph2D_2004_02_10_b.pm
>   7.295 Byte -   173 Zeilen -    Ecke_2004_02_10_b.pm
>   8.417 Byte -   199 Zeilen -   Kante_2004_02_10_b.pm
> -------        -----
> 259.576 Byte - 6.363 Zeilen

- Alle Module in den Namensraum Graph::GED:: verschoben.
- Tippfehler bei Load "for," statt "form" verbessert.

Version 0.0.28 vom 10.02.2004
~~~~~~~~~~~~~~
> 167.508 Byte - 3.905 Zeilen -     ged_2004_02_10_c.pl
>   5.884 Byte -   177 Zeilen -     FHist_2004_02_10.pm
>   9.367 Byte -   291 Zeilen -      Geom_2004_02_10.pm
>  25.045 Byte -   651 Zeilen -   Graph_2004_02_10_c.pm
>  40.159 Byte - 1.000 Zeilen - Graph2D_2004_02_10_c.pm
>   4.019 Byte -   111 Zeilen -    Ecke_2004_02_10_c.pm
>   5.149 Byte -   139 Zeilen -   Kante_2004_02_10_c.pm
>   8.480 Byte -   200 Zeilen -     Basis_2004_02_10.pm
>   4.656 Byte -   123 Zeilen -  Historie_2004_02_10.pm
> -------        -----
> 270.267 Byte - 6.653 Zeilen

- Basisklasse hinzugefgt, von der alle Klassen abgeleitet werden.
  Dadurch entfllt _property in allen anderen Klassen und alle new-
  Methoden werden einfacher.
  Die Methode clone muss von allen Klassen, die Kontainer in ihrem Hash
  haben, vie Graph, Graph2D berschrieben werden, damit
  nicht die Referenzen auf die Kontainer, sondern die Inhalte kopiert
  werden.
  Clonefunktionalitt noch nicht fertig, daher auskommentiert.
- Rumpfklasse Graph::GED::Historie fr Zurcknahme und Wiederholung
  angelegt (noch ohne Inhalt und ohne Verwendung. Bentig das clonen.
- Einige Fehler ausgemerzt, wo "Graphen::GED::" noch fehlte, etwa beim
  Ermitteln der Canvasausdehnung (eingefgte Kreise und vollstndige
  Graphen landeten dadurch in der oberen linken Ecke).

Methoden in Basis.pm:
    new
    _property
    clone


Version 0.0.29 vom 11.02.2004
~~~~~~~~~~~~~~
>  10.118 Byte -   253 Zeilen -    Graph::GED::Basis.pm
>   4.502 Byte -   139 Zeilen -     Graph::GED::Ecke.pm
>   5.884 Byte -   177 Zeilen -    Graph::GED::FHist.pm
>   9.367 Byte -   291 Zeilen -     Graph::GED::Geom.pm
>  27.736 Byte -   713 Zeilen -    Graph::GED::Graph.pm
>  40.250 Byte - 1.003 Zeilen -  Graph::GED::Graph2D.pm
>   4.656 Byte -   123 Zeilen - Graph::GED::Historie.pm
>   5.149 Byte -   139 Zeilen -    Graph::GED::Kante.pm
> 170.704 Byte - 4.042 Zeilen -                  ged.pl
> -------        -----
> 278.366 Byte - 6.880 Zeilen

- Clonefunktionalitt fertig gestellt.
- POD bei einigen Klassen weiter gefllt.

Neue Funktionen:
    clone_test
    Graph::GED::Graph2D::clone
    Graph::GED::Graph::clone
    Graph::GED::Basis::clone


Version 0.0.30 vom 12.02.2004
~~~~~~~~~~~~~~
>  10.131 Byte -   255 Zeilen -    Graph::GED::Basis.pm
>   4.502 Byte -   139 Zeilen -     Graph::GED::Ecke.pm
>   6.250 Byte -   181 Zeilen -    Graph::GED::FHist.pm
>   9.372 Byte -   291 Zeilen -     Graph::GED::Geom.pm
>  27.772 Byte -   714 Zeilen -    Graph::GED::Graph.pm
>  42.383 Byte - 1.033 Zeilen -  Graph::GED::Graph2D.pm
>   4.656 Byte -   123 Zeilen - Graph::GED::Historie.pm
>   5.149 Byte -   139 Zeilen -    Graph::GED::Kante.pm
>   8.102 Byte -   194 Zeilen -  Graph::GED::TkTools.pm
> 162.796 Byte - 3.888 Zeilen -                  ged.pl
> -------        -----
> 281.113 Byte - 6.957 Zeilen

- Kapselung der interenen Kanten/Ecken zusammenhnge durch vier Hilfsfunktionen
  in Graph::GED::Graph2D (x_von_a, y_von_a, x_von_e und y_von_e), welche die
  Koordinaten der Ecken ausgeben, die mit der Kante inzidieren.
  Dann muss man spter nur hier ndern, falls man doch mal die Ecken direkt in
  die Kanten aufnehmen will.
- use FindBin; use lib "$FindBin::RealBin"; verwendet, damit man das Programm
  (ohne Installation) nicht nur aus dem ged-Verzeichnis aus starten kann.
- Neues Modul Graph::GED::TkTools;
  Darein anzeige_fenster ausgelagert und als optionalen Parameter eine Wahl
  eingefhrt, ob der erzeugte "Dialog" modal sein soll.
- Alle Hilfefenster unmodal gemacht.
- Stellen ausgemerzt, an denen trotz debug auf 0 noch etwas ausgegeben wurde.
  (Hoffentlich alle.)

Neue Funktionen / Methoden:
    Graph::GED::Graph2D::x_von_a
    Graph::GED::Graph2D::y_von_a
    Graph::GED::Graph2D::x_von_e
    Graph::GED::Graph2D::y_von_e
    Graph::GED::Graph2D::x_von_a_index
    Graph::GED::Graph2D::y_von_a_index
    Graph::GED::Graph2D::x_von_e_index
    Graph::GED::Graph2D::y_von_e_index
    Graph::GED::TkTools::anzeige_fenster


Version 0.0.31 beta
~~~~~~~~~~~~~~~~~~~

Ich habe meinen Grapheneditor um eine Historienfunktion
erweitert. Diese Funktionalitt ist in einer extra
Klasse (Historie.pm, Graph::GED::Historie) enthalten,
die Klasse kann alle mglichen (von Graph::GED::Basis
abgeleiteten) Objekte speichern und verwalten.

Somit ist sie spter universell einsetzbar, wenn alle
Fehler behoben sind. Die Ableitung von Graph::GED::Basis
ist deshalb ntig, weil die verwalteten Objekte eine clone-
und eine clear-Methode haben mssen, beides wird u.a. von
Graph::GED::Basis zur Verfgung gestellt.

Diese beiden Methoden kmmern sich um das saubere ko-
pieren bzw. Lschen der verwalteten Objekte. Dies wird
dann wichtig, wenn die Objekte nicht nur "flache Daten"
enthalten, sondern anonyme Arrays oder Hashes.

(Genaugenommen prfe ich auch nicht auf die Ableitung von
Graph::GED::Basis, sondern nur, ob die bergebenen
Objekte die Methoden clone und clear beherrschen (mit can).)

Die Historie ist eine State-Machine (wem das nichts sagt:
es gibt verschiedene Zustnde und definierte bergnge
von Zustand zu Zustand und die Maschine "wei", was sie
in welchem Zustand bei welcher Anforderung machen soll).

Ich unterscheide folgende Zustnde:
    NORMAL
    NORMAL_ENDE
    NACH_RUECK
    NACH_RUECK_ANFANG
    NACH_WIEDER
    NACH_WIEDER_ENDE

Folgende "Anforderungen" bzw. von auen zugngliche
Methoden gibt es (neben new, clone und max):
    objekt_eintragen()
    rueckgaengig()
    wiederholung()

Zum Testen gibt es im Moment nur drei Speicherpltze in der
Historie, das werde ich natrlich spter auch hochsetzen.

(Die Kommentarbeschreibung der Aktionen in den verschiedenen
Stati stimmt noch nicht, das werde ich nachbessern, wenn alle
Fehler behoben sind.)

Das Historienobjekt in ged.pl selbst ist in %Konfig
unter dem Schlssel historie enthalten.
Es wird am Anfang mit new Graph::GED::Historie angelegt.

Das meiste funktioniert schon, aber mir ist noch ein Fehler
aufgefallen, den ich aber nicht genau lokalisieren und auch
nicht wiederholen konnte.

In der Betaversion ist in Graph/GED/Historie.pm der Debug-Level
auf 2, es gibt eine Extraausgabe in
    [tt]historie_schalter_aktualisieren()[/tt]
in ged.pl und die Fehlerausgaben in Tk-Fenster sind momentan
ausgeschaltet. Das wird spter gendert.


Neue Funktionen / Methoden:
    historie_test
    rueckgaengig
    wiederholung
    aenderung
    historie_schalter_aktualisieren
    Graph::GED::Historie::_hist_eintrag
    Graph::GED::Historie::_hist_loesch_ersten_eintrag
    Graph::GED::Historie::_hist_loesch_eintrag
    Graph::GED::Historie::clone
    Graph::GED::Historie::objekt_eintragen
    Graph::GED::Historie::rueckgaengig
    Graph::GED::Historie::wiederholung
    Graph::GED::Historie::geaendert


noch zu tun nach Abschlu beta-Test:
    Kommentartabelle in Historie an Realitt anpassen
    Windows-Fehlermeldung wieder anschalten in ged.pl
    meldung in historie_schalter_aktualisieren ausstellen
    Debug in Historie auf 0
    beta aus Versionsnummer raus


Version 0.0.31 vom 08.12.2004
~~~~~~~~~~~~~~
>  12.060 Byte -   302 Zeilen -    Graph::GED::Basis.pm
>   4.514 Byte -   139 Zeilen -     Graph::GED::Ecke.pm
>   6.250 Byte -   181 Zeilen -    Graph::GED::FHist.pm
>   9.372 Byte -   291 Zeilen -     Graph::GED::Geom.pm
>  27.018 Byte -   697 Zeilen -    Graph::GED::Graph.pm
>  42.883 Byte - 1.042 Zeilen -  Graph::GED::Graph2D.pm
>  40.401 Byte - 1.001 Zeilen - Graph::GED::Historie.pm
>   5.149 Byte -   139 Zeilen -    Graph::GED::Kante.pm
>   8.102 Byte -   194 Zeilen -  Graph::GED::TkTools.pm
> 179.335 Byte - 4.297 Zeilen -                  ged.pl
> -------        -----
> 335.084 Byte - 8.283 Zeilen

- Funktion Graph::GED::Historie::_hist_loesch_ersten_eintrag repariert.
- Funktion Graph::GED::Historie::_konsistent repariert.

Die Historie funktioniert noch nicht einwandfrei, aber weitgehend, so dass sie
fr das Programm freigeschaltet wird.

noch zu tun:
    Kommentartabelle in Historie an Realitt anpassen


Version 0.0.32 vom 01.01.2005
~~~~~~~~~~~~~~
>  12.060 Byte -   302 Zeilen -    Graph::GED::Basis.pm
>   6.950 Byte -   198 Zeilen -     Graph::GED::Ecke.pm
>   6.364 Byte -   185 Zeilen -    Graph::GED::FHist.pm
>   9.434 Byte -   289 Zeilen -     Graph::GED::Geom.pm
>  27.022 Byte -   697 Zeilen -    Graph::GED::Graph.pm
>  60.143 Byte - 1.567 Zeilen -  Graph::GED::Graph2D.pm
>  40.356 Byte -   999 Zeilen - Graph::GED::Historie.pm
>   8.041 Byte -   229 Zeilen -    Graph::GED::Kante.pm
>   8.175 Byte -   193 Zeilen -  Graph::GED::TkTools.pm
> 209.437 Byte - 5.011 Zeilen -                  ged.pl
> -------        -----
> 387.982 Byte - 9.670 Zeilen

- Neues Graphenformat 'ged2' eingefhrt: Die Eckeneigenschaften Farbe, Radius,
  Beschriftung, Wert, Ausrichtung der Beschriftung und Ausrichtung des Wertes
  und die Kanteneigenschaften Farbe, Stil, Beschriftung, Wert, Ausrichtung der
  Beschriftung und Ausrichtung des Wertes werden mit abgespeichert und geladen.
  Auerdem noch die Standardwerte fr Eckenfarbe, Radius und Kantenfarbe.
  (Beim Laden und Speichern stehen nun die Formate ged und ged2 zur Verfgung.)
- Kontextmen fr Ecken und Kanten hinzugefgt. Auer den neuen obigen
  Eigenschaften kann man darber auch Ecken mergen, lschen, am Gitter
  ausrichten und Kanten lschen und Kantenformen und -stile ndern.
- Ausgabe des Graphen als Dump im internen Men.
- ToDo-Liste berarbeitet.
- Gitter wird nach Lschen angezeigt, falls dies vorher der Fall war.
- Funktionalitt aus save und load nach Graph2D ausgelagert.
- Neue Attribute und Getter / Setter fr obige Eigentschaften in
  Graph::GED::Ecke.pm und Graph::GED::Kante.pm.
- Dokumentationen (pod).

Neue Funktionen / Methoden:
    zeige_daten_an_dumper
    context_menu
    tk_cm_farbwahl_klein
    tk_cm_groessenwahl
    tk_cm_kantenformwahl
    tk_cm_kantenstilwahl
    tk_cm_titelwahl
    tk_cm_titelausrichtung
    tk_cm_wertwahl
    tk_cm_wertausrichtung
    Graph::GED::Ecke::farbe
    Graph::GED::Ecke::radius
    Graph::GED::Ecke::titel
    Graph::GED::Ecke::gewicht
    Graph::GED::Ecke::titelpos
    Graph::GED::Ecke::gewichtpos
    Graph::GED::Kante::farbe
    Graph::GED::Kante::stil
    Graph::GED::Kante::titel
    Graph::GED::Kante::gewicht
    Graph::GED::Kante::titelpos
    Graph::GED::Kante::gewichtpos
    Graph::GED::Graph2D::save_ged
    Graph::GED::Graph2D::load_ged
    Graph::GED::Graph2D::save_ged2
    Graph::GED::Graph2D::load_ged2


Version 0.0.33 vom 02.01.2005
~~~~~~~~~~~~~~
>  12.060 Byte -    302 Zeilen -    Graph::GED::Basis.pm
>   6.950 Byte -    198 Zeilen -     Graph::GED::Ecke.pm
>   6.364 Byte -    185 Zeilen -    Graph::GED::FHist.pm
>   9.434 Byte -    289 Zeilen -     Graph::GED::Geom.pm
>  27.022 Byte -    697 Zeilen -    Graph::GED::Graph.pm
>  61.731 Byte -  1.600 Zeilen -  Graph::GED::Graph2D.pm
>  40.356 Byte -    999 Zeilen - Graph::GED::Historie.pm
>   8.407 Byte -    237 Zeilen -    Graph::GED::Kante.pm
>  11.350 Byte -    281 Zeilen -  Graph::GED::TkTools.pm
> 220.957 Byte -  5.214 Zeilen -                  ged.pl
> -------        ------
> 404.631 Byte - 10.002 Zeilen

- Kantendicke eingefhrt. Das ged2-Format wurde um die Dicke ergnzt.
  Vorbergehend wird beim Einlesen von ged2 auf die Dicke nicht geprft, so
  dass auch mit Version 0.0.32 erstellten ged2-Dateien eingelesen werden
  knnen.
- Kantenstil (normal, gestrichelt odeer gepunktelt) dargestellt.
- Die Kanten werden nicht mehr von Eckenmitte zu Eckenmitte gezogen, sondern
  von Eckenrand zu Eckenrand.
- Refresh nach Kanten ziehen fr Zeichnung von Eckenrand zu Eckenrand
- Bercksichtigung individueller Eckenradien in Graph2D::meint_ecke().
  Neue Berechnung von delta aus dem radius.
- tk_cm_farbwahl_klein() durch Graph::GED::TkTools::farbwahl() ersetzt.
  Dadurch fllt diese Funktion weg.
- Kommentarpflege in Graph::GED::TkTools (pod).
- Hintergrundfarbe wird jetzt im Graphen mit gespeichert und lsst sich via
  Men und via Kontextmen whlen.

Neue Funktionen / Methoden:
    tk_cm_kantendicke
    Graph::GED::Kante::dicke
    Graph::GED::TkTools::farbwahl
    Graph::GED::Graph2D::hintergrund

Entfernte Funktionen / Methoden:
    tk_cm_farbwahl_klein


Version 0.0.34 vom 03.01.2005
~~~~~~~~~~~~~~
>  12.060 Byte -    302 Zeilen -    Graph::GED::Basis.pm
>   7.918 Byte -    218 Zeilen -     Graph::GED::Ecke.pm
>   6.364 Byte -    185 Zeilen -    Graph::GED::FHist.pm
>   9.434 Byte -    289 Zeilen -     Graph::GED::Geom.pm
>  27.022 Byte -    697 Zeilen -    Graph::GED::Graph.pm
>  67.667 Byte -  1.727 Zeilen -  Graph::GED::Graph2D.pm
>  40.356 Byte -    999 Zeilen - Graph::GED::Historie.pm
>   9.427 Byte -    257 Zeilen -    Graph::GED::Kante.pm
>  12.203 Byte -    294 Zeilen -  Graph::GED::TkTools.pm
> 256.160 Byte -  6.010 Zeilen -                  ged.pl
> -------        ------
> 448.611 Byte - 10.978 Zeilen 

- Ankerberechnungen beim Zeichnen von Ecken und Kanten vereinfacht.
- Planung fr neues Speicherformat GED3 (siehe ToDo-Liste).
- Gitterwerte liegen jetzt in $Graph und werden im ged2-Format auch mit
  abgespeichert und geladen.
- Alle Perdocs knnen ber das Hilfe-Men gesichtet werden.
- Anzeigefensterfunktion in Graph::GED::TkTools um Bindungen erweitert,
  insbesondere kann nun per Mausrad gescrollt werden.
- Gitterfarbe ist nun whlbar.
- nderungsanzeige rechts unten neben der Statusbar ('*' => gendert).
  Beim Beenden wird jetzt nachgefragt, falls nicht gesicherte nderungen
  vorliegen.
- Inhalt Lschen im Dateimen.
- Fr Ecken- und Kantenbeschreibungen und -gewichte sind jetzt Farben whlbar.
  Diese Farben werden auch im ged2-Format mit abgespeichert.
- Die Gittereinstellungen sind nun auch per Kontextmen mglich.
  Defaulteckenfarbe und -kantenfarbe lsst sich jetzt auch per Kontextmen
  einstellen.
- Tooltip-Hilfe fr die Bedienelemente wird in der Statuszeile angezeigt.

Neue Funktionen / Methoden:
    Graph::GED::Graph2D::radius2delta
    perldoc_graph_ged_basis_pm
    perldoc_graph_ged_ecke_pm
    perldoc_graph_ged_kante_pm
    perldoc_graph_ged_graph_pm
    perldoc_graph_ged_graph2d_pm
    perldoc_graph_ged_geom_pm
    perldoc_graph_ged_historie_pm
    perldoc_graph_ged_fhist_pm
    perldoc_graph_ged_tktools_pm


Version 0.0.35 vom 14.01.2005
~~~~~~~~~~~~~~
>  12.060 Byte -    302 Zeilen -    Graph::GED::Basis.pm
>   7.918 Byte -    218 Zeilen -     Graph::GED::Ecke.pm
>   6.364 Byte -    185 Zeilen -    Graph::GED::FHist.pm
>   9.434 Byte -    289 Zeilen -     Graph::GED::Geom.pm
>  27.022 Byte -    697 Zeilen -    Graph::GED::Graph.pm
>  69.733 Byte -  1.776 Zeilen -  Graph::GED::Graph2D.pm
>  40.356 Byte -    999 Zeilen - Graph::GED::Historie.pm
>   9.427 Byte -    257 Zeilen -    Graph::GED::Kante.pm
>  12.203 Byte -    294 Zeilen -  Graph::GED::TkTools.pm
> 269.067 Byte -  6.301 Zeilen -                  ged.pl
> -------        ------
> 463.584 Byte - 11.318 Zeilen

- aenderung() vor dem Beginn von Ecken-Verschiebungen aufgerufen statt am Ende.
- Ecken- und Kantennummern werden auf Wunsch angezeigt.
- Neuer Button zum Lschen aller getroffenen Kanten.
- Anzeige der internen Struktur im GED2-Format.
- Alle Pfade zu den Bilderdateien um "$FindBin::RealBin/" vorweg ergnzt, so
  dass man ged nicht aus dem ged-Verzeichnis heraus aufrufen muss.
- Hintergrundbild lsst sich laden, speichern, auswhlen (ber Men und
  Kontextmen).

Neue Funktionen / Methoden:
    zeige_daten_an2
    Graph::GED::Graph2D::hintergrundbild


Version 0.0.36 vom 16.01.2005
~~~~~~~~~~~~~~
>  12.060 Byte -    302 Zeilen -    Graph::GED::Basis.pm
>   7.918 Byte -    218 Zeilen -     Graph::GED::Ecke.pm
>   6.364 Byte -    185 Zeilen -    Graph::GED::FHist.pm
>   9.434 Byte -    289 Zeilen -     Graph::GED::Geom.pm
>  27.022 Byte -    697 Zeilen -    Graph::GED::Graph.pm
>  78.770 Byte -  1.998 Zeilen -  Graph::GED::Graph2D.pm
>  40.356 Byte -    999 Zeilen - Graph::GED::Historie.pm
>   9.643 Byte -    260 Zeilen -    Graph::GED::Kante.pm
>  12.203 Byte -    294 Zeilen -  Graph::GED::TkTools.pm
> 265.375 Byte -  6.209 Zeilen -                  ged.pl
> -------        ------
> 469.145 Byte - 11.451 Zeilen

- Defaultkantendicke ist jetzt nicht mehr 1, sondern wird im Graphen mit
  abgespeichert und ist ber das Men whlbar.
- Menbeschriftung im Ansichtsmen um "Default " ergnzt (Eckenfarbe,
  Kantenfarbe...).
- Dateiname wird jetzt in $Graph gehalten und mit abgespeichert.
  Der Grund ist, dass dies bei der Animation ntig ist.
  Sollte die Datei gendert werden, so wird der flasche gespeicherte Name beim
  Laden durch den richtigen berschrieben.
- Anzeige der Daten in ged-Formaten erfolgt jetzt ber Speicherung in einer
  temporren Datei.
- Graph2d nach Graph2D umbenannt im Dateinamen und Klasse einheitlich.
- toLaTeX nach Graph2D verschoben.

Neue Funktionen / Methoden:
    Graph::GED::Graph2D::kantendicke
    Graph::GED::Graph2D::dateiname
    Graph::GED::Graph2D::save
    Graph::GED::Graph2D::load
    Graph::GED::Graph2D::toLaTeX

Entfernte Funktionen / Methoden:
    zeige_daten_an2
    toLaTeX


Version 0.0.37 vom 16.01.2005
~~~~~~~~~~~~~~
>  12.060 Byte -    302 Zeilen -    Graph::GED::Basis.pm
>   7.918 Byte -    218 Zeilen -     Graph::GED::Ecke.pm
>   6.364 Byte -    185 Zeilen -    Graph::GED::FHist.pm
>   9.434 Byte -    289 Zeilen -     Graph::GED::Geom.pm
>  27.022 Byte -    697 Zeilen -    Graph::GED::Graph.pm
>  96.490 Byte -  2.336 Zeilen -  Graph::GED::Graph2D.pm
>  40.356 Byte -    999 Zeilen - Graph::GED::Historie.pm
>   9.643 Byte -    260 Zeilen -    Graph::GED::Kante.pm
>  12.203 Byte -    294 Zeilen -  Graph::GED::TkTools.pm
> 221.184 Byte -  5.094 Zeilen -                  ged.pl
> -------        ------
> 442.674 Byte - 10.674 Zeilen

- Neues Graphenformat 'ged3' eingefhrt.
- Buttons zum Verschieben des Graphen eingebaut.

Neue Funktionen / Methoden:
    Graph::GED::Graph2D::save_ged3
    Graph::GED::Graph2D::load_ged3
    nach_links
    nach_rechts
    nach_oben
    nach_unten


Version 0.0.38 vom 17.01.2005
~~~~~~~~~~~~~~
>  12.060 Byte -    302 Zeilen -    Graph::GED::Basis.pm
>   7.918 Byte -    218 Zeilen -     Graph::GED::Ecke.pm
>   6.364 Byte -    185 Zeilen -    Graph::GED::FHist.pm
>   9.434 Byte -    289 Zeilen -     Graph::GED::Geom.pm
>  27.022 Byte -    697 Zeilen -    Graph::GED::Graph.pm
> 109.627 Byte -  2.704 Zeilen -  Graph::GED::Graph2D.pm
>  40.356 Byte -    999 Zeilen - Graph::GED::Historie.pm
>   9.643 Byte -    260 Zeilen -    Graph::GED::Kante.pm
>  12.203 Byte -    294 Zeilen -  Graph::GED::TkTools.pm
> 271.774 Byte -  6.364 Zeilen -                  ged.pl
> -------        ------
> 506.401 Byte - 12.312 Zeilen

- Verschiebebuttons mit transparentem Hintergrund, so dass sie grau und nicht
  mehr wei erscheinen.
- nach_links etc. nach Graph2D verschoben.
- Buttons und Funktionalitt fr Zentrieren und Zentrieren/Skalieren eingebaut.
- Balloon-Texte fr Bewegungs- und Skalierungsbuttons hinzugefgt.
- Verschiebung erfolgt jetzt um Gitterweite.

Neue Funktionen / Methoden:
    Graph::GED::Graph2D::zentrieren
    Graph::GED::Graph2D::zentrieren_skalieren
    Graph::GED::Graph2D::kleiner
    Graph::GED::Graph2D::groesser
    Graph::GED::Graph2D::nach_links
    Graph::GED::Graph2D::nach_rechts
    Graph::GED::Graph2D::nach_oben
    Graph::GED::Graph2D::nach_unten

Entfernte Funktionen / Methoden:
    nach_links
    nach_rechts
    nach_oben
    nach_unten


Version 0.0.39 vom 17.01.2005
~~~~~~~~~~~~~~
>  12.060 Byte -    302 Zeilen -    Graph::GED::Basis.pm
>   7.918 Byte -    218 Zeilen -     Graph::GED::Ecke.pm
>   6.364 Byte -    185 Zeilen -    Graph::GED::FHist.pm
>   9.434 Byte -    289 Zeilen -     Graph::GED::Geom.pm
>  27.022 Byte -    697 Zeilen -    Graph::GED::Graph.pm
> 110.038 Byte -  2.715 Zeilen -  Graph::GED::Graph2D.pm
>  40.356 Byte -    999 Zeilen - Graph::GED::Historie.pm
>   9.643 Byte -    260 Zeilen -    Graph::GED::Kante.pm
>  12.203 Byte -    294 Zeilen -  Graph::GED::TkTools.pm
> 276.125 Byte -  6.487 Zeilen -                  ged.pl
> -------        ------
> 511.163 Byte - 12.446 Zeilen

- Contextmens werden jetzt auch unter Linux gelscht.
- Verbesserung Zentrieren+Skalieren fr den Fall xmin=xmax und ymin=ymax.
- Extension verbessert beim Speichern unter Linux.

Neue Funktionen / Methoden:
    clear_context_menu
HISTORYTEXT


    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'History',
                                         'History:',
                                         '{Courier New} 10 {normal}',
                                         $text,
                                         0,
                                        );
} # sub history


sub todo () {
    #--------------------------------------------------------------------------
    # sub          : t o d o
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : ToDo-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2003 - CD - Erstellt
    # 0.0.2 - 13.02.2003 - CD - Text in globale Variable verschoben, damit die
    #                           ToDo-Liste wieder am Anfang dieser Datei steht.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.5 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'Ideen',
                                         'Ideen:',
                                         '{Courier New} 10 {normal}',
                                         $Todotext,
                                         0,
                                        );
} # sub todo


sub perldoc_ged_pl () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g e d _ p l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/ged.pl),
        0,
    );
} # sub perldoc_ged_pl


sub perldoc_graph_ged_basis_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ b a s i s _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/Basis.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub perldoc_graph_ged_ecke_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ e c k e _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/Ecke.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub perldoc_graph_ged_kante_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ k a n t e _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/Kante.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub perldoc_graph_ged_graph_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ g r a p h _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/Graph.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub perldoc_graph_ged_graph2d_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ g r a p h 2 d _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/Graph2D.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub perldoc_graph_ged_geom_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ g e o m _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/Geom.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub perldoc_graph_ged_historie_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ h i s t o r i e _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/Historie.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub perldoc_graph_ged_fhist_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ f h i s t _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/FHist.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub perldoc_graph_ged_tktools_pm () {
    #--------------------------------------------------------------------------
    # sub          : p e r l d o c _ g r a p h _ g e d _ t k t o o l s _ p m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/Graph/GED/TkTools.pm),
        0,
    );
} # sub perldoc_graph_ged_basis_pm


sub ueber_ged () {
    #--------------------------------------------------------------------------
    # sub          : u e b e r _ g e d
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Zeigt etwas Informationen ber GED an.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 10.02.2003 - CD - GED-Icon verwendet
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 09.02.2004 - CD - Hinweis auf die Webseite eingebaut.
    #--------------------------------------------------------------------------

    print "ueber_ged\n" if $Konfig{debug};

    my $icon = $Konfig{mw}->Photo(-file   => "$FindBin::RealBin/res/ged.bmp",
                          -height => 32,
                          -width  => 32,
                         );
    my $dialog = $Konfig{mw}->Dialog(-text           => <<'HERE2' . "Version $Konfig{version} / $Konfig{version_datum}",
G - E - D ist ein grafischer Editor fr Graphen aus der Graphentheorie.

GED bietet die Mglichkeit, Ecken und Kanten leicht mit der Maus auf der
Arbeitsflche zu positionieren und erlaubt es, den entstandenen Graphen in
unterschiedlichen Formaten abzuspeichern.

Autor: Christian Dhl (e.mail: christian@duehl.de)

Webseite: http://www.duehl.de/christian/perl/ged.html

HERE2
                                #-bitmap         => 'info',
                                 -image          => $icon,
                                 -title          => 'ber G-E-D',
                                 -default_button => 'OK',
                                 -buttons        => [ 'OK' ],
                                #-font           => '{Courier New} 10 {normal}',
                                 -font           => '{Times New Roman} 12 {normal}',
                                );
    $dialog->Show(-global);
} # sub ueber_ged


sub canvas_groesse () {
    #--------------------------------------------------------------------------
    # sub          : c a n v a s _ g r o e s s e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ermittelt die Koordinaten des sichtbaren Bereiches der
    #                Zeichenflche.
    # Parameter    : keine
    # Rckgabewert : (xmin, ymin, xmax, ymax)
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    # 0.0.2 - 07.02.2004 - CD - Geometry zur Grenermittlung verwendet.
    #--------------------------------------------------------------------------

    print "canvas_groesse\n" if $Konfig{debug};


    #my ($x, $y, $w, $h) = $Konfig{canvas}->bbox("all");
    #print "x=$x y=$y w=$w h=$h\n";
    #use Data::Dumper;
    #print Dumper($Konfig{canvas}->bbox("all")), "\n";

    my $geom           = $Konfig{canvas}->geometry();
    my ($wh, $px, $py) = split /\+/, $geom;
    my ($xx, $yy)      = split /x/, $wh;
    print "$geom ($xx x $yy + $px + $py)\n" if $Konfig{debug};


    return (0, 0, $xx, $yy);   #(0,0, 486, 430);
} # sub canvas_groesse


sub waehle_eckenfarbe () {
    #--------------------------------------------------------------------------
    # sub          : w a e h l e _ e c k e n f a r b e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Whlt die Eckenfarbe per Dialog.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $farbe = $Konfig{mw}->chooseColor(-title        => 'Eckenfarbe whlen',
                                         -initialcolor => $Graph->eckenfarbe(),
                                        );
    $Graph->eckenfarbe($farbe) if $farbe;
    refresh();
} # sub waehle_eckenfarbe


sub waehle_kantenfarbe () {
    #--------------------------------------------------------------------------
    # sub          : w a e h l e _ k a n t e n f a r b e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Whlt die Kantenfarbe per Dialog.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $farbe = $Konfig{mw}->chooseColor(-title        => 'Kantenfarbe whlen',
                                         -initialcolor => $Graph->kantenfarbe(),
                                        );
    $Graph->kantenfarbe($farbe) if $farbe;
    refresh();
} # sub waehle_kantenfarbe


sub historie_schalter_aktualisieren ($$) {
    #--------------------------------------------------------------------------
    # sub          : h i s t o r i e _ s c h a l t e r
    #                                             _ a k t u a l i s i e r e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Aktiviert oder deaktiviert die Rckgngig-/Wiederholen-
    #                Schalter.
    # Parameter    : 1) Anzahl Rckgngigmglichkeiten
    #                2) Anzahl Wiederholungsmglichkeiten
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $alt = shift;
    my $neu = shift;


    print "historie_schalter_aktualisieren($alt,$neu)\n" if $Konfig{debug};

    if ($alt) {
        $Konfig{rueck_btn}->configure(-state => 'normal');
    }
    else {
        $Konfig{rueck_btn}->configure(-state => 'disabled');
    }
    if ($neu) {
        $Konfig{wieder_btn}->configure(-state => 'normal');
    }
    else {
        $Konfig{wieder_btn}->configure(-state => 'disabled');
    }
} # sub historie_schalter_aktualisieren


sub context_menu ($$$$$) {
    #--------------------------------------------------------------------------
    # sub          : c o n t e x t _ m e n u
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Kontextmenue anzeigen.
    # Parameter    : 1) Canvas Widget
    #                2) x-Koordinate
    #                3) y-Koordinate
    #                4) absolute x-Koordinate
    #                5) absolute y-Koordinate
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 31.12.2004 - CD - Ausrichtungen fr Wert und Titel.
    # 0.0.3 - 01.01.2005 - CD - Ecke mergen, lschen, am Gitter ausrichten
    #                           Kante lschen.
    # 0.0.4 - 01.01.2005 - CD - Kantendicke.
    # 0.0.4 - 02.01.2005 - CD - tk_cm_farbwahl_klein() durch
    #                           Graph::GED::TkTools::farbwahl() ersetzt.
    #                           Ecken- und Kantenfarbe auch per kleiner
    #                           Farbauswahl whlbar.
    #                           Kontextmen abseits von Ecken und Kanten
    #                           eingefhrt (mit Wahl der Hintergrundfarbe).
    # 0.0.5 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    #                           Die Gittereinstellungen sind nun auch per
    #                           Kontextmen mglich.
    #                           Selbiges gilt fr die Defaultecken- und
    #                           kantenfarbe.
    # 0.0.6 - 14.01.2005 - CD - Hintergrundbild laden.
    # 0.0.7 - 16.01.2005 - CD - Default Kantendicke bestimmen.
    # 0.0.8 - 17.01.2005 - CD - Fr Probleme unter Linux: berflssige
    #                           Kontextmens zerstren.
    #--------------------------------------------------------------------------

    my ($canvas, $x, $y, $ax, $ay) = @_;
    print "context-Menue ($x, $y) - ($ax, $ay)\n" if $Konfig{debug};


    #--------------------------------------------------------------------------
    # Herausfinden, welches Element gemeint ist:
    #--------------------------------------------------------------------------
    my $ek = 0;                            # 0 nix, 1 Ecke, 2 Kante
    my $nr = $Graph->meint_ecke($x, $y);   # Nummer von Ecke oder Kante

    if ($nr > -1) {
        $ek = 1;
        #print "meint Ecke $nr\n";
    }
    else {
        $nr = $Graph->meint_kante($x, $y);
        if ($nr > -1) {
            #print "meint Kante $nr\n";
            $ek = 2;
        }
        else {
            $ek = 0;
            #print "Kein Element getroffen!\n";
        }
    }
    
    clear_context_menu();
    my $cmenu;
    if ($ek == 0) {
        $cmenu =
        $Konfig{mw}->Menu(
            -font      => $Konfig{menufont},
            -tearoff   => 0,
            -menuitems => [
                ["command"    => "Hintergrundfarbe ndern (kleine Farbauswahl)",
                 -command     => [ \&Graph::GED::TkTools::farbwahl,
                                   $Konfig{mw},
                                   $ax, $ay,
                                   sub {
                                       aenderung();
                                       $Graph->hintergrund($_[0]);
                                       refresh();
                                   },
                                 ],
                 -font        => $Konfig{menufont},
                ],
                ["command"    => "Hintergrundfarbe ndern (groe Farbauswahl)",
                 -command     => sub {
                                     my $c = $Konfig{mw}->chooseColor(-title
                                         => "Bitte die Farbe fr den "
                                            . "Hintergrund auswhlen:");
                                     aenderung();
                                     $Graph->hintergrund($c) if $c;
                                     refresh();
                                 },
                 -font        => $Konfig{menufont},
                ],
                ["command"    => "Hintergrundbild festlegen",
                 -command     => \&lade_hintergrundbild,
                 -font        => $Konfig{menufont},
                ],
                '-',
                ['checkbutton' => "Eckennummern anzeigen",
                 -variable     => \$Konfig{zeige_e_nr},
                 -command      => \&refresh,
                 -font         => $Konfig{menufont},
                ],
                ['checkbutton' => "Kantennummern anzeigen",
                 -variable     => \$Konfig{zeige_k_nr},
                 -command      => \&refresh,
                 -font         => $Konfig{menufont},
                 ],
                '-',
                ['checkbutton' => "Gitternetz anzeigen",
                 -variable     => \$Graph->{gitterzeigen}, #!!!
                 -command      => \&refresh,
                 -underline    => 0,
                 -font         => $Konfig{menufont},
                ],
                [qw(cascade Gitterweite -tearoff 1 -underline 6
                    -menuitems),
                 [map ['radiobutton' => $_,
                       -variable => \$Graph->{gitterweite}, #!!!
                       -command  => \&refresh,
                      ],
                      qw(5 10 15 20 25 30 35 40 45 50
                         60 70 80 100),
                 ],
                ],
                ['checkbutton' => "Am Gitternetz einrasten",
                 -variable     => \$Graph->{rastimgitter}, #!!!
                 -command      => \&refresh,
                 -underline    => 17,
                 -font         => $Konfig{menufont},
                ],
                ['command'     => "Gitterfarbe whlen (kleine Auswahl)",
                 -command      => sub {
                                      my ($x, $y) = $Konfig{mw}->geometry()
                                          =~ /\+(\d+)\+(\d+)$/;
                                      Graph::GED::TkTools::farbwahl(
                                          $Konfig{mw},
                                          $x+140, $y+155,
                                          sub {
                                              aenderung();
                                              $Graph->gitterfarbe($_[0]);
                                              refresh();
                                          },
                                      )
                                  },
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                ['command'     => "Gitterfarbe whlen (groe Auswahl)",
                 -command      => sub {
                                      aenderung();
                                      my $farbe = $Konfig{mw}->
                                          chooseColor(-title        => 'Gitterfarbe whlen',
                                                      -initialcolor => $Graph->hintergrund(),
                                                     );
                                      $Graph->gitterfarbe($farbe) if $farbe;
                                      refresh();
                                  },
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                '-',
                ['command'     => "Default Eckenfarbe whlen (kleine Auswahl)",
                 -command      => sub {
                                      my ($x, $y) = $Konfig{mw}->geometry()
                                          =~ /\+(\d+)\+(\d+)$/;
                                      Graph::GED::TkTools::farbwahl(
                                          $Konfig{mw},
                                          $x+140, $y+165,
                                          sub {
                                              aenderung();
                                              $Graph->eckenfarbe($_[0]);
                                              refresh();
                                          },
                                      )
                                  },
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                ['command'     => "Default Kantenfarbe whlen (kleine Auswahl)",
                 -command      => sub {
                                      my ($x, $y) = $Konfig{mw}->geometry()
                                          =~ /\+(\d+)\+(\d+)$/;
                                      Graph::GED::TkTools::farbwahl(
                                          $Konfig{mw},
                                          $x+140, $y+175,
                                          sub {
                                              aenderung();
                                              $Graph->kantenfarbe($_[0]);
                                              refresh();
                                          },
                                      )
                                  },
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                '-',
                ['command'     => "Default Eckenfarbe whlen (groe Farbauswahl)",
                 -command      => \&waehle_eckenfarbe,
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                ['command'     => "Default Kantenfarbe whlen (groe Farbauswahl)",
                 -command      => \&waehle_kantenfarbe,
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                '-',
                ["command"    => "Default Eckenradius ndern",
                 -command     => [ \&tk_cm_groessenwahl, $ax, $ay, $Graph ],
                 -font        => $Konfig{menufont},
                ],
                ["command"    => "Default Kantendicke ndern",
                 -command     => [ \&tk_cm_kantendicke, $ax, $ay, $Graph ] ,
                 -font        => $Konfig{menufont},
                ],
                          ],
        );
    }
    else {
        #print +($ek == 1 ? 'Ecke' : 'Kante'), " $nr ist gemeint\n";

        my $element = ($ek == 1 ? 'Ecke' : 'Kante');
        my $objekt  = ($ek == 1 ? $Graph->ecke($nr) : $Graph->kante($nr));

        $cmenu =
        $Konfig{mw}->Menu(-font      => $Konfig{menufont},
                          -tearoff   => 0,
                          -menuitems => [
                              ["command"    => "${element}nfarbe ndern (kleine Farbauswahl)",
                               -command     => [ \&Graph::GED::TkTools::farbwahl,
                                                 $Konfig{mw},
                                                 $ax, $ay,
                                                 sub {
                                                     aenderung();
                                                     $objekt->farbe($_[0]);
                                                     refresh();
                                                 },
                                               ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nfarbe ndern (groe Farbauswahl)",
                               -command     => sub {
                                                   my $c = $Konfig{mw}->chooseColor(-title
                                                       => "Bitte die Farbe fr $element $nr"
                                                          . " auswhlen:");
                                                   $objekt->farbe($c) if $c;
                                                   refresh();
                                               },
                               -font        => $Konfig{menufont},
                              ],
                              '-',
                              ($ek == 1
                              ? # nur fr Ecken:
                              ["command"    => "${element}ngre ndern",
                               -command     => [ \&tk_cm_groessenwahl, $ax, $ay, $objekt ],
                               -font        => $Konfig{menufont},
                              ]
                              : # nur fr Kanten:
                              (
                              ["command"    => "${element}nform ndern",
                               -command     => [ \&tk_cm_kantenformwahl, $ax, $ay, $objekt ] ,
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nstil ndern",
                               -command     => [ \&tk_cm_kantenstilwahl, $ax, $ay, $objekt ] ,
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}ndicke ndern",
                               -command     => [ \&tk_cm_kantendicke, $ax, $ay, $objekt ] ,
                               -font        => $Konfig{menufont},
                              ])),
                              '-',
                              ["command"    => "${element}nbeschriftung ndern",
                               -command     => [ \&tk_cm_titelwahl, $ax, $ay, $objekt, $element, $nr ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nbeschriftungsausrichtung ndern",
                               -command     => [ \&tk_cm_titelausrichtung, $ax, $ay, $objekt ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nbeschriftungsfarbe ndern (kleine Farbauswahl)",
                               -command     => [ \&Graph::GED::TkTools::farbwahl,
                                                 $Konfig{mw},
                                                 $ax, $ay,
                                                 sub {
                                                     aenderung();
                                                     $objekt->titelfarbe($_[0]);
                                                     refresh();
                                                 },
                                               ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nbeschriftungsfarbe ndern (groe Farbauswahl)",
                               -command     => sub {
                                                   my $c = $Konfig{mw}->chooseColor(-title
                                                       => "Bitte die Farbe fr $element $nr"
                                                          . " auswhlen:");
                                                   $objekt->titelfarbe($c) if $c;
                                                   refresh();
                                               },
                               -font        => $Konfig{menufont},
                              ],
                              '-',
                              ["command"    => "${element}nwert ndern",
                               -command     => [ \&tk_cm_wertwahl, $ax, $ay, $objekt, $element, $nr ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nwertausrichtung ndern",
                               -command     => [ \&tk_cm_wertausrichtung, $ax, $ay, $objekt ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nwertfarbe ndern (kleine Farbauswahl)",
                               -command     => [ \&Graph::GED::TkTools::farbwahl,
                                                 $Konfig{mw},
                                                 $ax, $ay,
                                                 sub {
                                                     aenderung();
                                                     $objekt->gewichtfarbe($_[0]);
                                                     refresh();
                                                 },
                                               ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nwertfarbe ndern (groe Farbauswahl)",
                               -command     => sub {
                                                   my $c = $Konfig{mw}->chooseColor(-title
                                                       => "Bitte die Farbe fr $element $nr"
                                                          . " auswhlen:");
                                                   $objekt->gewichtfarbe($c) if $c;
                                                   refresh();
                                               },
                               -font        => $Konfig{menufont},
                              ],
                              '-',
                              ($ek == 1 && $Graph->gitterzeigen() ?
                              ["command"    => "${element} in Gitter einrasten",
                               -command     => sub { aenderung();
                                                     my ($x, $y) = raste_ein($objekt->x(), $objekt->y());
                                                     $objekt->x($x);
                                                     $objekt->y($y);
                                                     refresh();
                                                 },
                               -font        => $Konfig{menufont},
                              ] : () ),
                              ($ek == 1 ? # nur fr Ecken:
                              ["command"    => "${element} mergen",
                               -command     => sub { aenderung();
                                                     $Graph->ecke_mergen($nr);
                                                     refresh();
                                                 },
                               -font        => $Konfig{menufont},
                              ] : () ),
                              ["command"    => "${element} lschen",
                               -command     => sub { aenderung();
                                                     $ek == 1 ? $Graph->ecke_loeschen($nr)
                                                              : $Graph->kante_loeschen($nr);
                                                     refresh();
                                                 },
                               -font        => $Konfig{menufont},
                              ],
                                        ],
                         );
    }
    $cmenu->post($ax, $ay);
    push @{ $Konfig{Contextmenu} }, $cmenu;
    $cmenu->bind(
        '<FocusOut>',
        sub {
            if (defined $Konfig{Contextmenu}) {
                $Konfig{Contextmenu}->destroy;
                $Konfig{Contextmenu} = undef;
            }
        }
    );
    #$cmenu->bind(
    #    '<Escape>',
    #    sub {
    #        if (defined $Konfig{Contextmenu}) {
    #            $Konfig{Contextmenu}->destroy;
    #            $Konfig{Contextmenu} = undef;
    #        }
    #    }
    #);
} # sub context_menu


sub tk_cm_groessenwahl ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ g r o e s s e n w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Grenauswahl fr das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Gre gesetzt werden soll (Ecke)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    aenderung();

    print "tk_cm_groessenwahl ($ax, $ay) Objekt $objekt\n"
        if $Konfig{debug};

    my @radien = (qw/  3   4   5   6
                       7   8   9  10
                      12  14  16  18
                      20  25  30  35
                      40  50  75 100/);
    my $index = 0;
    my $r = $objekt->radius() || $Graph->radius();
    for (0..$#radien) {
        if ($r == $radien[$_]) {
            $index = $_;
            last;
        }
    }
    my $radius = $radien[$index];

    my $menu = $Konfig{mw}->Menu(-tearoff => 0);

    for (@radien) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$radius,
            -command     => sub {
                                  $objekt->radius($radius);
                                  refresh();
                                },
            -columnbreak => 0,
        );
    }

    for (0..$#radien-1) {
        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 5;
    }

    $menu->post($ax, $ay);
} # sub tk_cm_groessenwahl


sub tk_cm_kantenformwahl ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ k a n t e n f o r m w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Formauswahl fr die Kante fr das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Gre gesetzt werden soll (Kante)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    aenderung();

    print "tk_cm_kantenformwahl ($ax, $ay) Objekt $objekt\n" if $Konfig{debug};

    my $f    = $Linientypen{ $objekt->form() }; # muss gesetzt sein!
    my $menu = $Konfig{mw}->Menu(-tearoff => 0);

    for (qw/k_ins k_inh k_inz k_inb/) {
        $menu->radiobutton(
            -label       => $Linientypen{$_},
            -variable    => \$f,
            -command     => sub {
                                  my $key;
                                  for (keys %Linientypen) {
                                      $key = $_ if $f eq $Linientypen{$_};
                                  }
                                  if ($key) {
                                      $objekt->form($key);
                                      refresh();
                                  }
                                  else {
                                      die "Interner Fehler bei Kantenformwahl";
                                  }
                                },
        );
    }

    $menu->post($ax, $ay);
} # sub tk_cm_kantenformwahl


sub tk_cm_kantenstilwahl ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ k a n t e n s t i l w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Stilauswahl fr die Kante fr das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Gre gesetzt werden soll (Kante)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    aenderung();

    print "tk_cm_kantenstilwahl ($ax, $ay) Objekt $objekt\n"
        if $Konfig{debug};

    my @stile = (qw/normal gestrichelt gepunktet/);
    my $s     = $objekt->stil();
    my $menu  = $Konfig{mw}->Menu(-tearoff => 0);

    for (@stile) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$s,
            -command     => sub {
                                  $objekt->stil($s);
                                  refresh();
                                },
        );
    }

    $menu->post($ax, $ay);
} # sub tk_cm_kantenstilwahl


sub tk_cm_kantendicke ($$$) {
    #--------------------------------------------------------------------------
    # sub          : tk_cm_kantendicke
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Wahl der Kantendicke fr das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Gre gesetzt werden soll (Ecke)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 16.01.2005 - CD - Auch fr Default-Kantendicke verwendbar (das
    #                           Objekt ist dann vom Typ $Graph2D).
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    aenderung();

    print "tk_cm_kantendicke ($ax, $ay) Objekt $objekt\n" if $Konfig{debug};

    my @dicken = (qw/  1   2   3   4   5
                       6   7   8   9  10
                      11  12  14  16  17
                      18  19  20  25  30
                      35  40  50  75 100/);
    my $d      = $objekt->isa('Graph::GED::Graph2D') ? $objekt->kantendicke()
                                         : $objekt->dicke();
    my $menu   = $Konfig{mw}->Menu(-tearoff => 0);

    for (@dicken) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$d,
            -command     => sub {
                                  if ($objekt->isa('Graph::GED::Graph2D')) {
                                      $objekt->kantendicke($d);
                                  }
                                  else {
                                      $objekt->dicke($d);
                                  }
                                  refresh();
                                },
            -columnbreak => 0,
        );
    }

    for (0..$#dicken) {
        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 5;
    }

    $menu->post($ax, $ay);

} # sub tk_cm_kantendicke


sub tk_cm_titelwahl ($$$$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ t i t e l w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Titelauswahl fr das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Gre gesetzt werden soll (Ecke)
    #                4) Ecke oder Kante
    #                5) Nummer des Objekts
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt, $element, $nr) = @_;

    aenderung();

    print "tk_cm_titelwahl ($ax, $ay) Objekt $objekt\n" if $Konfig{debug};

    my $t = $Konfig{mw}->Toplevel();
    $t->transient($Konfig{mw});
    $t->overrideredirect(1);
    $t->geometry("+$ax+$ay");
    #
    #$t->title("Bitte Beschreibung fr $element $nr eingeben");
    # braucht man nicht wegen overrideredirect(1)
    $t->label("Bitte Beschreibung fr $element $nr eingeben");

    my $tvar = $objekt->titel();
    my $txt = $t->Entry(-width        => 20,
                        -textvariable => \$tvar,
                       )
                 ->pack(-side         => 'left',
                        -fill         => 'x',
                        -expand       => 0,
                       );
    my $ok = $t->Button(-text        => 'OK',
                        -command     => sub {
                                             $objekt->titel($tvar);
                                             $t->destroy();
                                             $Konfig{mw}->focus();
                                             refresh();
                                            },
                        -default     => 'active',
                        -padx        => 15,
                       )
                 ->pack(-expand      => '0',
                        -fill        => 'none',
                        -side        => 'left',
                       );

    $t->bind('<Return>', sub{$ok->invoke()});

    $t->grab();
    $t->focus();
    $txt->focus();

} # sub tk_cm_titelwahl


sub tk_cm_titelausrichtung ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ t i t e l a u s r i c h t u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Titelausrichtung whlen fr das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Gre gesetzt werden soll (Ecke)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    aenderung();

    print "tk_cm_titelausrichtung ($ax, $ay) Objekt $objekt\n"
        if $Konfig{debug};

    my @positionen = (qw/NW W  SW
                         N  *  S
                         NO O  SO/);
    my $index = 0;
    my $p = $objekt->titelpos() || $positionen[0];
    for (0..$#positionen) {
        if ($p eq $positionen[$_]) {
            $index = $_;
            last;
        }
    }
    my $pos = $positionen[$index];

    my $menu = $Konfig{mw}->Menu(-tearoff => 0);

    for (@positionen) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$pos,
            -command     => sub {
                                  $objekt->titelpos($pos) unless $pos eq '*';
                                  refresh();
                                },
            -columnbreak => 0,
        );
    }

    for (0..$#positionen) {
        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 3;
    }

    $menu->post($ax, $ay);

} # sub tk_cm_titelausrichtung


sub tk_cm_wertwahl ($$$$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ w e r t w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Titelauswahl fr das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Gre gesetzt werden soll (Ecke)
    #                4) Ecke oder Kante
    #                5) Nummer des Objekts
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt, $element, $nr) = @_;

    aenderung();

    print "tk_cm_wertwahl_klein ($ax, $ay) Objekt $objekt\n" if $Konfig{debug};

    my $t = $Konfig{mw}->Toplevel();
    $t->transient($Konfig{mw});
    $t->overrideredirect(1);
    $t->geometry("+$ax+$ay");
    #
    #$t->title("Bitte Wert fr $element $nr eingeben");
    # braucht man nicht wegen overrideredirect(1)
    $t->label("Bitte Wert fr $element $nr eingeben");

    my $tvar = $objekt->gewicht();
    my $txt = $t->Entry(-width        => 20,
                        -textvariable => \$tvar,
                       )
                 ->pack(-side         => 'left',
                        -fill         => 'x',
                        -expand       => 0,
                       );
    my $ok = $t->Button(-text        => 'OK',
                        -command     => sub {
                                             $objekt->gewicht($tvar);
                                             #use Data::Dumper; print Dumper $Graph;
                                             $t->destroy();
                                             $Konfig{mw}->focus();
                                             refresh();
                                            },
                        -default     => 'active',
                        -padx        => 15,
                       )
                 ->pack(-expand      => '0',
                        -fill        => 'none',
                        -side        => 'left',
                       );

    $t->bind('<Return>', sub{$ok->invoke()});

    $t->grab();
    $t->focus();
    $txt->focus();

} # sub tk_cm_wertwahl


sub tk_cm_wertausrichtung ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ w e r t a u s r i c h t u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Wertausrichtung whlen fr das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Gre gesetzt werden soll (Ecke)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    aenderung();

    print "tk_cm_wertausrichtung ($ax, $ay) Objekt $objekt\n"
        if $Konfig{debug};

    my @positionen = (qw/NW W  SW
                         N  *  S
                         NO O  SO/);
    my $index = 0;
    my $p = $objekt->gewichtpos() || $positionen[8];
    for (0..$#positionen) {
        if ($p eq $positionen[$_]) {
            $index = $_;
            last;
        }
    }
    my $pos = $positionen[$index];

    my $menu = $Konfig{mw}->Menu(-tearoff => 0);

    for (@positionen) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$pos,
            -command     => sub {
                                  $objekt->gewichtpos($pos) unless $pos eq '*';
                                  refresh();
                                },
            -columnbreak => 0,
        );
    }

    for (0..$#positionen) {
        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 3;
    }

    $menu->post($ax, $ay);

} # sub tk_cm_wertausrichtung


sub tk_beenden () {
    #--------------------------------------------------------------------------
    # sub          : t k _ b e e n d e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Programm beenden
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    # 0.0.2 - 03.01.2005 - CD - Bei nderungen nachfragen.
    #--------------------------------------------------------------------------

    if ($Konfig{changed} eq '*') {
        my $icon = $Konfig{mw}->Photo(
            -file   => "$FindBin::RealBin/res/ged.bmp",
            -height => 32,
            -width  => 32,
        );
        my $dialog = $Konfig{mw}->Dialog(
            -text           => 'Sie haben nicht gesicherte nderungen! Was wollen sie tun?',
            #-bitmap         => 'info',
            -image          => $icon,
            -title          => 'GED beenden?',
            -default_button => 'GED Beenden',
            -buttons        => [ 'GED Beenden', 'Speichern', 'Abbrechen' ],
            #-font           => '{Courier New} 10 {normal}',
            -font           => '{Times New Roman} 12 {normal}',
        );
        my $antwort = $dialog->Show(-global);
        return if $antwort eq 'Abbrechen';
        speichere_datei() if $antwort eq 'Speichern';
        $Konfig{mw}->destroy();
    }
    else {
        $Konfig{mw}->destroy();
    }
}


###############################################################################
###############################################################################
####                                                                       ####
####              L        OOO     GGG      I     K   K                    ####
####              L       O   O   G         I     K  K                     ####
####              L       O   O   G  GG     I     KKK                      ####
####              L       O   O   G   G     I     K  K                     ####
####              LLLLL    OOO     GGG      I     K   K                    ####
####                                                                       ####
###############################################################################
###############################################################################



sub beenden () {
    #--------------------------------------------------------------------------
    # sub          : b e e n d e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Wird zum Ende des Programms aufgerufen.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 14.02.2003 - CD - History wird gespeichert.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 06.02.2004 - CD - Statt exit wird tk_beenden aufgerufen.
    # 0.0.5 - 10.02.2004 - CD - Module in Namensraum Graph::GED:: verschoben.
    #--------------------------------------------------------------------------

    print "beenden\n" if $Konfig{debug};
    dump_data() if $Konfig{debug};

    Graph::GED::FHist::save();

    tk_beenden();
} # sub beenden


sub struktur_einfuegen ($) {
    #--------------------------------------------------------------------------
    # sub          : s t r u k t u r _ e i n f u e g e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt Kreise / Vollstndige Graphen ein
    # Parameter    : Strukturkennung, beginnt mt C ode K gefolgt von einer
    #                natrlichen Zahl > 2.
    #                Oder 'M' fr den Mittelpunkt.
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2004 - CD - Funktionalitt in die Klasse Graph2D ausge-
    #                           lagert.
    # 0.0.2 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    #--------------------------------------------------------------------------

    #----------------------------------------------------------------------
    # Aktuellen Graphen in Historie sichern:
    #----------------------------------------------------------------------
    aenderung();

    $Graph->struktur_einfuegen(@_);
    refresh();
} # sub struktur_einfuegen


sub raste_ein ($$) {
    #--------------------------------------------------------------------------
    # sub          : r a s t e _ e i n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Rastet ein Koordinatenpaar auf den nchsten Gitterlinien-
    #                kreuzungspunkt ein.
    # Parameter    : 1) X-Koordinate
    #                2) Y-Koordinate
    # Rckgabewert : (x-Koordinate, y-Koordinate)
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $x = shift;
    my $y = shift;

    print "x=$x, X:@GitterX\n",
          "y=$y, Y:@GitterY\n"
          if $Konfig{debug} > 1;

    # X-Koordinate bearbeiten:
    my $xindex = 0;
    my $xminab = abs($x-$GitterX[0]);
    for (1..$#GitterX) {
        if ($xminab > abs($x-$GitterX[$_])) {
            $xminab = abs($x-$GitterX[$_]);
            $xindex = $_;
        }
    }
    print "naechster Wert zu $x ist Index $xindex ($GitterX[$xindex])\n"
          if $Konfig{debug} > 1;

    # Y-Koordinate bearbeiten:
    my $yindex = 0;
    my $yminab = abs($y-$GitterY[0]);
    for (1..$#GitterY) {
        if ($yminab > abs($y-$GitterY[$_])) {
            $yminab = abs($y-$GitterY[$_]);
            $yindex = $_;
        }
    }
    print "naechster Wert zu $y ist Index $yindex ($GitterY[$yindex])\n"
          if $Konfig{debug} > 1;


    return ($GitterX[$xindex], $GitterY[$yindex]);
} # sub raste_ein


sub erzeuge_liniengraphen () {
    #--------------------------------------------------------------------------
    # sub          : e r z e u g e _ l i n i e n g r a p h e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Erzeugt aus dem gegebenen Graphen seinen Liniengraphen.
    # Parameter    : keine
    # Rckgabewert : keiner (ein neuer Graph wird erzeugt)
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    # 0.0.2 - 07.02.2004 - CD - Waitcursor anzeigen whrend Berechnung und
    #                           refresh.
    # 0.0.3 - 08.02.2004 - CD - eigentliche Funktionalitt in Klasse Graph2D
    #                           ausgelagert.
    # 0.0.4 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    #--------------------------------------------------------------------------

    print "erzeuge_liniengraphen\n" if $Konfig{debug};


    $Konfig{mw}->Busy();

    #----------------------------------------------------------------------
    # Aktuellen Graphen in Historie sichern:
    #----------------------------------------------------------------------
    aenderung();

    #----------------------------------------------------------------------
    # Eigentlichen Liniengraphen erzeugen:
    #----------------------------------------------------------------------
    $Graph->erzeuge_liniengraphen();

    #----------------------------------------------------------------------
    # Weitere Eigenschaften zurcksetzen:
    #----------------------------------------------------------------------
    $Graph->kantenform('k_ins');
    $Konfig{startecke}  = -1;
    $Konfig{zx}         = 0;
    $Konfig{zy}         = 0;

    refresh();

    $Konfig{mw}->Unbusy();
} # sub erzeuge_liniengraphen


sub rueckgaengig () {
    #--------------------------------------------------------------------------
    # sub          : r u e c k g a e n g i g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : nderungen rckgngig machen.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "rueckgaengig\n" if $Konfig{debug};

#print "vor rueckgaengig: ", Dumper($Konfig{historie});
    my ($g, $alt, $neu) = $Konfig{historie}->rueckgaengig($Graph);
    $Graph->clear();
    $Graph = undef;
    $Graph = $g;
    historie_schalter_aktualisieren($alt, $neu);
    refresh();
#print "nach rueckgaengig: ", Dumper($Konfig{historie});

} # sub rueckgaengig


sub wiederholung () {
    #--------------------------------------------------------------------------
    # sub          : w i e d e r h o l u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Rckgngig wieder rckgngig machen...
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "wiederholung\n" if $Konfig{debug};

#print "vor wiederholung: ", Dumper($Konfig{historie});
    $Graph->clear();
    $Graph = undef;
    my ($g, $alt, $neu) = $Konfig{historie}->wiederholung();
    $Graph = $g;
    historie_schalter_aktualisieren($alt, $neu);
    refresh();
#print "nach wiederholung: ", Dumper($Konfig{historie});

} # sub rueckgaengig


sub aenderung () {
    #--------------------------------------------------------------------------
    # sub          : a e n d e r u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Daten werden gleich gendert, aktuelle Version in
    #                Historie sichern.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    # 0.0.7 - 03.01.2005 - CD - nderungsmerker $Konfig{changed} wird gesetzt.
    #--------------------------------------------------------------------------

    print "aenderung\n" if $Konfig{debug};

#print "vor aenderung: ", Dumper($Konfig{historie});
    my ($alt, $neu) = $Konfig{historie}->objekt_eintragen($Graph);
    historie_schalter_aktualisieren($alt, $neu);
#print "nach aenderung: ", Dumper($Konfig{historie});
    $Konfig{changed} = '*';

} # sub aenderung


sub do_all_the_tests () {
    #--------------------------------------------------------------------------
    # sub          : d o _ a l l _ t h e _ t e s t s
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Testet alles mgliche und hlt dann an.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2003 - CD - Erstellt
    # 0.0.2 - 13.02.2003 - CD - Tests umgebrochen, so dass alles in die ersten
    #                           80 Zeichen passt.
    # 0.0.3 - 10.02.2004 - CD - Module in Namensraum Graph::GED:: verschoben.
    #--------------------------------------------------------------------------

    #
    # floor
    #
    print "floor($_)=", floor($_), "\n" for (1.4,566.2,0,-24,-55.1);

    #
    # punkt_abstand
    #
    my ($x1, $y1, $x2, $y2);
    $x1=0;  $y1=0;  $x2=10; $y2=0;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=10; $y1=0;  $x2=0;  $y2=0;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=0;  $y2=10; print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=10; $x2=0;  $y2=0;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";

    $x1=1;  $y1=0;  $x2=10; $y2=0;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=1;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=2;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=3;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=4;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=5;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=6;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=7;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=8;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=9;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";
    $x1=0;  $y1=0;  $x2=10; $y2=10; print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";

    $x1=2;  $y1=2;  $x2=3;  $y2=3;  print "Abstand ($x1,$y1)-($x2,$y2) = ",
                        Graph::GED::Geom::punkt_abstand($x1,$y1,$x2,$y2), "\n";

    #
    # abstand_punkt_linie
    #
    my ($x, $y, $ax, $ay, $bx, $by);

    $x=1; $y=1; $ax=0; $ay=0; $bx=2; $by=2;
    print "Abstand ($x,$y) von ($ax,$ay)---($bx,$by) ist ",
          Graph::GED::Geom::abstand_punkt_linie($x,$y, $ax,$ay, $bx,$by), "\n\n";

    $x=2; $y=2; $ax=0; $ay=0; $bx=6; $by=6;
    print "Abstand ($x,$y) von ($ax,$ay)---($bx,$by) ist ",
          Graph::GED::Geom::abstand_punkt_linie($x,$y, $ax,$ay, $bx,$by), "\n\n";

    # Murks, da es nur fr ganze Zahlen geht:
    #$x=2.432; $y=2.432; $ax=0; $ay=0; $bx=6; $by=6;
    #print "Abstand ($x,$y) von ($ax,$ay)---($bx,$by) ist ",
    #      Graph::GED::Geom::abstand_punkt_linie($x,$y, $ax,$ay, $bx,$by), "\n\n";

    $x=2432; $y=2432; $ax=0; $ay=0; $bx=6000; $by=6000;
    print "Abstand ($x,$y) von ($ax,$ay)---($bx,$by) ist ",
          Graph::GED::Geom::abstand_punkt_linie($x,$y, $ax,$ay, $bx,$by), "\n\n";

    $x=-1; $y=-1; $ax=0; $ay=0; $bx=-2; $by=-2;
    print "Abstand ($x,$y) von ($ax,$ay)---($bx,$by) ist ",
          Graph::GED::Geom::abstand_punkt_linie($x,$y, $ax,$ay, $bx,$by), "\n\n";
    $x=-2; $y=-2; $ax=0; $ay=0; $bx=-6; $by=-6;
    print "Abstand ($x,$y) von ($ax,$ay)---($bx,$by) ist ",
          Graph::GED::Geom::abstand_punkt_linie($x,$y, $ax,$ay, $bx,$by), "\n\n";

    $x=-1; $y=-1; $ax=-2; $ay=-2; $bx=0; $by=0;
    print "Abstand ($x,$y) von ($ax,$ay)---($bx,$by) ist ",
          Graph::GED::Geom::abstand_punkt_linie($x,$y, $ax,$ay, $bx,$by), "\n\n";
    $x=-2; $y=-2; $ax=-6; $ay=-6; $bx=0; $by=0;
    print "Abstand ($x,$y) von ($ax,$ay)---($bx,$by) ist ",
          Graph::GED::Geom::abstand_punkt_linie($x,$y, $ax,$ay, $bx,$by), "\n\n";

    exit;
} # sub do_all_the_tests


sub clone_test () {
    #--------------------------------------------------------------------------
    # sub          : c l o n e _ t e s t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Testet die Clone-Methoden.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 11.02.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    $Graph->ecke_einfuegen(0,0); # 0
    $Graph->ecke_einfuegen(50,100); # 1
    $Graph->ecke_einfuegen(100,0); # 2
    $Graph->kante_einfuegen(0,1); # 0
    $Graph->kante_einfuegen(1,2); # 1

    use Data::Dumper;
    print "Graph: ", Dumper($Graph), "\n";

    my $neu = $Graph->new();
    print "Neu: ", Dumper($neu), "\n";

    my $clone = $Graph->clone();
    print "Clone: ", Dumper($clone), "\n";

    print "Graph          : ", $Graph                 , "\n",
          "Clone          : ", $clone                 , "\n",
          "Graphs Ecken   : ", $Graph->_ecken()       , "\n",
          "Clones Ecken   : ", $clone->_ecken()       , "\n",
          "Graphs Kanten  : ", $Graph->_kanten()      , "\n",
          "Clones Kanten  : ", $clone->_kanten()      , "\n",
          "Graphs 1. Ecke : ", @{$Graph->_ecken()}[0] , "\n",
          "Clones 1. Ecke : ", @{$clone->_ecken()}[0] , "\n",
          "Graphs 1. Kante: ", @{$Graph->_kanten()}[0], "\n",
          "Clones 1. Kante: ", @{$clone->_kanten()}[0], "\n";

    exit();
} # sub clone_test


sub historie_test () {
    #--------------------------------------------------------------------------
    # sub          : h i s t o r i e _ t e s t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Testet die Historien-Klasse.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    use Data::Dumper;

    my $Hist = new Graph::GED::Historie;
    print "1 - Graph=$Graph - ", Dumper($Graph);
    print "Hist: ", Dumper($Hist);

    $Hist->objekt_eintragen($Graph);
    print "2 - Graph=$Graph - ", Dumper($Graph);
    print "Hist: ", Dumper($Hist);

    $Graph->ecke_einfuegen(0,0);    # E 0
    $Graph->ecke_einfuegen(50,100); # E 1
    $Graph->ecke_einfuegen(100,0);  # E 2
    $Graph->kante_einfuegen(0,1);   # K 0
    $Graph->kante_einfuegen(1,2);   # K 1
    print "3 - Graph=$Graph - ", Dumper($Graph);
    print "Hist: ", Dumper($Hist);

    $Hist->objekt_eintragen($Graph);
    print "4 - Graph=$Graph - ", Dumper($Graph);
    print "Hist: ", Dumper($Hist);

    $Graph->clear();
    $Graph = undef;
    ($Graph, undef, undef) = $Hist->rueckgaengig();
    print "5 - Graph=$Graph - ", Dumper($Graph);
    print "Hist: ", Dumper($Hist);

    $Graph->clear();
    $Graph = undef;
    ($Graph, undef, undef) = $Hist->wiederholung();
    print "6 - Graph=$Graph - ", Dumper($Graph);
    print "Hist: ", Dumper($Hist);


    $Graph->ecke_loeschen(1);
    $Hist->objekt_eintragen($Graph);
    print "7 - Graph=$Graph - ", Dumper($Graph);
    print "Hist: ", Dumper($Hist);

    $Graph->ecke_loeschen(1);
    $Hist->objekt_eintragen($Graph);
    print "8 - Graph=$Graph - ", Dumper($Graph);
    print "Hist: ", Dumper($Hist);

    exit();
} # sub historie_test



###############################################################################
###############################################################################
####                                                                       ####
####              DDDD      A     TTTTT   EEEEE   N   N                    ####
####              D   D    A A      T     E       NN  N                    ####
####              D   D   AAAAA     T     EEEE    N N N                    ####
####              D   D   A   A     T     E       N  NN                    ####
####              DDDD    A   A     T     EEEEE   N   N                    ####
####                                                                       ####
###############################################################################
###############################################################################



sub dump_data () {
    #--------------------------------------------------------------------------
    # sub          : d u m p _ d a t a
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Gibt die internen Daten auf dem Bildschirm aus.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 03.02.2003 - CD - In dump_data umbenannt.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.4 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.5 - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    print "dump_data\n" if $Konfig{debug};

    print $Graph->n(), "\n";
    for my $i (0..$Graph->n-1) {
        print $Graph->ecke($i)->x(), ";",
              $Graph->ecke($i)->y(), "\n";
    }

    print $Graph->m(), "\n";
    for my $i (0..$Graph->m()-1) {
        print $Graph->kante($i)->a(), ";",
              $Graph->kante($i)->e(), ";",
              $Graph->kante($i)->form(), "\n";
    }
} # sub dump_data


sub save ($) {
    #--------------------------------------------------------------------------
    # sub          : s a v e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Interne Speicherroutine.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt.
    # 0.0.2 - 14.02.2003 - CD - Dateiname der History zugefgt.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.4 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.5 - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.6 - 01.01.2005 - CD - Funktionalitt nach Graph2D::save_ged() aus-
    #                           gelagert.
    # 0.0.7 - 03.01.2005 - CD - nderungsmerker $Konfig{changed} wird zurck
    #                           gesetzt.
    # 0.0.8 - 16.01.2005 - CD - Graph2D::save() verwendet.
    #--------------------------------------------------------------------------

    my $dateiname = shift;

    print "save, Name = '$dateiname' <--------\n" if $Konfig{debug};

    if ($Graph->save($dateiname)) {
        $Konfig{statuszeile} = "Datei '$dateiname' gespeichert.";
        add($dateiname);
        $Konfig{changed} = ' ';
    }

} # sub save


sub load ($) {
    #--------------------------------------------------------------------------
    # sub          : l o a d
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Interne Laderoutine.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 17.02.2003 - CD - Dateinamen in History gespeichert.
    # 0.0.3 - 18.03.2003 - CD - Neue Linientypenkrzel verwendet. Dafr werden
    #                           beim Laden die ausfhrlichen Formen in die
    #                           kurzen bersetzt, um alte Dateien richtig la-
    #                           den zu knnen.
    # 0.0.4 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.5 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.6 - 12.02.2004 - CD - aenderung() vor nderungen aufgerufen.
    # 0.0.7 - 01.01.2005 - CD - Funktionalitt nach Graph2D::load_ged() aus-
    #                           gelagert.
    # 0.0.8 - 03.01.2005 - CD - nderungsmerker $Konfig{changed} wird zurck
    #                           gesetzt.
    # 0.0.9 - 16.01.2005 - CD - Graph2D::load() verwendet.
    #--------------------------------------------------------------------------

    my $dateiname = shift;

    print "load ($dateiname)\n" if $Konfig{debug};

    aenderung();                  # Aktuellen Graphen in Historie sichern.

    if ($Graph->load($dateiname)) {
        $Konfig{statuszeile} = "Datei '$dateiname' geladen.";
        add($dateiname);              # neuen Dateiname zu Historie hinzufgen
        $Konfig{changed} = ' ';
    }

} # sub load


sub neue_datei () {
    #--------------------------------------------------------------------------
    # sub          : n e u e _ d a t e i
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Lscht alles. Eventuell wird hier mal mehr gemacht, als nur
    #                clear aufzurufen, wenn man sich noch Dateinamen merkt o.a..
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf %Konfig.
    # 0.0.3 - 03.01.2005 - CD - nderungsmarker zurckgesetzt.
    # 0.0.4 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    #--------------------------------------------------------------------------

    print "neue_datei\n" if $Konfig{debug};
    clear();
    $Graph->dateiname('');
    $Konfig{mw}->title('G - E - D   ' . $Konfig{version});
    $Konfig{changed} = ' ';
} # sub neue_datei


sub add ($) {
    #--------------------------------------------------------------------------
    # sub          : a d d
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt den Dateinamen in die Liste ein und aktualisiert das
    #                Men.
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 10.02.2004 - CD - Module in Namensraum Graph::GED:: verschoben.
    #--------------------------------------------------------------------------

    my $dateiname = shift;

    Graph::GED::FHist::add($dateiname);
    add_file_menu();
} # sub add


sub clear_context_menu () {
    #--------------------------------------------------------------------------
    # sub          : c l e a r _ c o n t e x t _ m e n u
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Entfernt alle Kontextmens (ntig wegen Linux).
    # Parameter    : Dateiname
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    if (exists $Konfig{Contextmenu}) {
        for my $i (reverse 0..$#{$Konfig{Contextmenu}}) {
            my ($c) = splice @{ $Konfig{Contextmenu} }, $i, 1;
            $c->destroy if defined $c;
            $c = undef;
        }
    }
} # sub clear_context_menu


__END__

=head1 NAME

ged.pl - a Graph EDitor with a Perl/Tk gui

=head1 COPYRIGHT


Copyright 2004-2005 Christian Duehl. All rights reserved.

This program is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
